@@ -577,7 +577,7 @@
  (lambda_match
   ((Satom _ Alid s) (&& (>= (String.length s) 2) (= ([] s 0) '''))
    (let ((s (String.sub s 1 (- (String.length s) 1))))
-      (, <:vala< (Some s) >> (, None False))))
+      (, <:vala< (Some s) >> <:vala< "" >>)))
   (se
    (error se "type_parameter")))
  ctyp_se
