@@ -1135,12 +1135,12 @@
   (match se
    ((when (Slid _ s) (&& (>= (String.length s) 2) (= s.[0] ''')))
     (let ((s (String.sub s 1 (- (String.length s) 1))))
-     (values <:vala< (Some s) >> (values None False))))
+     (values <:vala< (Some s) >> <:vala< "" >>)))
    ((when (Slid _ s) (&& (>= (String.length s) 3) (= s.[1] ''')))
     (let
      ((vara
-       (if (= s.[0] '+') (values (Some True) False)
-        (if (= s.[0] '-') (values (Some False) False) (error se "type_param"))))
+       (if (= s.[0] '+') <:vala< "+" >>
+        (if (= s.[0] '-') <:vala< "-" >> (error se "type_param"))))
       (s (String.sub s 2 (- (String.length s) 2))))
      (values <:vala< (Some s) >> vara)))
    (se (error se "type_param"))))
@@ -1177,7 +1177,10 @@
    ((Sexpr loc [(Slid _ "as") se1 se2])
     (let* ((t1 (ctyp_se se1)) (t2 (ctyp_se se2))) <:ctyp< ($t1$ as $t2$) >>))
    ((Sexpr loc [(Slid _ "*") . sel])
-    (let ((tl (anti_list_map ctyp_se sel))) <:ctyp< ($_list:tl$) >>))
+    (let* ((tl (anti_list_map ctyp_se sel))
+          (tl (Pcaml.vala_map (List.map (lambda (ct) (values <:vala< None >> ct))) tl))
+         )
+         <:ctyp< ($_list:tl$) >>))
    ((Sexpr loc [(Slid _ "==") se1 se2])
     (let* ((t1 (ctyp_se se1)) (t2 (ctyp_se se2))) <:ctyp< $t1$ == $t2$ >>))
    ((Sexpr loc [(Slid _ "?") se1 se2])
