@@ -409,8 +409,8 @@ EXTEND
   tyvarseq: [ [ -> not_impl loc "tyvarseq" ] ];
 
   tyvar_pc:
-    [ [ "'"; x1 = V tyvar -> [(x1, (None, False))]
-      | "'"; x1 = V tyvar; ","; l = tyvar_pc -> [(x1, (None, False)) :: l] ] ]
+    [ [ "'"; x1 = V tyvar -> [(x1, <:vala< "" >>)]
+      | "'"; x1 = V tyvar; ","; l = tyvar_pc -> [(x1, <:vala< "" >>) :: l] ] ]
   ;
   tyvar:
     [ [ v = LIDENT -> Some v ] ]
@@ -479,7 +479,10 @@ EXTEND
   ctyp:
     [ RIGHTA
       [ x1 = ctyp; "->"; x2 = ctyp -> <:ctyp< $x1$ -> $x2$ >> ]
-    | [ x1 = ctyp; "*"; x2 = tuple_ty -> <:ctyp< ($list:[x1 :: x2]$) >> ]
+    | [ x1 = ctyp; "*"; x2 = tuple_ty ->
+      let xl = [x1 :: x2] in
+      let xl = List.map (fun x -> (<:vala< None >>, x)) xl in
+      <:ctyp< ($list:xl$) >> ]
     | "ty'"
       [ "'"; x1 = LIDENT -> <:ctyp< '$x1$ >>
       | "'"; "'"; x1 = LIDENT -> <:ctyp< '$x1$ >>
@@ -705,7 +708,7 @@ EXTEND
            MLast.tdAttributes = <:vala< [] >>} ] ]
   ;
   tyvars:
-    [ [ "'"; x1 = V tyvar -> [(x1, (None, False))]
+    [ [ "'"; x1 = V tyvar -> [(x1, <:vala< "" >>)]
       | "("; x1 = tyvar_pc; ")" -> x1
       | -> [] ] ]
   ;
