@@ -1,6 +1,6 @@
 (* camlp5r *)
 (* pr_extend.ml,v *)
-(* Copyright (c) INRIA 2007-2014 *)
+(* Copyright (c) INRIA 2007-2017 *)
 
 #directory ".";
 #load "q_MLast.cmo";
@@ -29,12 +29,6 @@ value not_impl name pc x =
   pprintf pc "\"pr_extend, not impl: %s; %s\"" name (String.escaped desc)
 ;
 
-IFDEF OCAML_VERSION <= OCAML_1_07 THEN
-  value with_ind = Pprintf.with_ind;
-  value with_bef = Pprintf.with_bef;
-  value with_aft = Pprintf.with_aft;
-END;
-
 value bar_before elem pc x = pprintf pc "| %p" elem x;
 value semi_after elem pc x = pprintf pc "%p;" elem x;
 
@@ -51,6 +45,7 @@ type symbol =
   | Sflag of symbol
   | Sself
   | Snext
+  | Scut
   | Stoken of alt Plexing.pattern MLast.expr
   | Srules of list (list (option MLast.patt * symbol) * option MLast.expr)
   | Svala of list string and option string and symbol ]
@@ -61,7 +56,7 @@ and alt 'a 'b =
 
 value rec get_globals =
   fun
-  [ [(<:patt< _ >>, <:expr< ($e$ : $uid:gmod1$.Entry.e '$_$) >>) :: pel] ->
+  [ [(<:patt< _ >>, <:expr< ($e$ : $uid:gmod1$.Entry.e '$_$) >>, _) :: pel] ->
       let (gmod, gl) = get_globals pel in
       if gmod = "" || gmod = gmod1 then (gmod1, [e :: gl])
       else raise Not_found
@@ -71,7 +66,7 @@ value rec get_globals =
 
 value rec get_locals =
   fun
-  [ [(<:patt< $_$ >>, <:expr< (grammar_entry_create $_$ : $_$) >>) :: pel] ->
+  [ [(<:patt< $_$ >>, <:expr< (grammar_entry_create $_$ : $_$) >>, _) :: pel] ->
       get_locals pel
   | [] -> ()
   | _ -> raise Not_found ]
@@ -168,10 +163,38 @@ value rec unrule =
       in
       let sl = unpsymbol_list (List.rev pl) e1 in
       (sl, a)
+  | <:expr< Grammar.production ($e1$, $hash$, $e2$) >> ->
+      let (pl, a) =
+        match unaction e2 with
+        [ ([], None) -> let loc = Ploc.dummy in ([], Some <:expr< () >>)
+        | x -> x ]
+      in
+      let e1 =
+        loop [] e1 where rec loop rel =
+          fun
+          [ <:expr< Grammar.r_next $el$ $e$ >> -> loop [e :: rel] el
+          | <:expr< Grammar.r_stop >> -> rel
+          | <:expr:< Grammar.r_cut $el$ >> -> loop [<:expr< cut >> :: rel] el
+          | _ -> raise Not_found ]
+      in
+      let sl = safe_unpsymbol_list (List.rev pl) e1 in
+      (sl, a)
   | _ -> raise Not_found ]
+and safe_unpsymbol_list pl el =
+  match (pl, el) with
+  | ([], []) -> []
+  | (pl, [<:expr< cut >> :: el]) ->
+      [(None, Scut) :: safe_unpsymbol_list pl el]
+  | ([p :: pl], [e :: el]) ->
+      let op = match p with [ <:patt< _ >> -> None | _ -> Some p ] in
+      [(op, safe_unsymbol e) :: safe_unpsymbol_list pl el]
+  | _ -> raise Not_found
+  end
 and unpsymbol_list pl e =
   match (pl, e) with
   [ ([], <:expr< [] >>) -> []
+  | (_, <:expr< [Gramext.Scut :: $el$] >>) ->
+      [(None, Scut) :: unpsymbol_list pl el]
   | ([p :: pl], <:expr< [$e$ :: $el$] >>) ->
       let op =
         match p with
@@ -180,6 +203,27 @@ and unpsymbol_list pl e =
       in
       [(op, unsymbol e) :: unpsymbol_list pl el]
   | _ -> raise Not_found ]
+and safe_unsymbol =
+  fun
+  | <:expr< Grammar.s_facto $e$ >> -> safe_unsymbol e
+  | <:expr< Grammar.s_nterm ($e$ : $_$) >> -> Snterm e
+  | <:expr< Grammar.s_nterml ($e$ : $_$) $str:s$ >> -> Snterml e s
+  | <:expr< Grammar.s_list0 $e$ >> -> Slist0 (safe_unsymbol e)
+  | <:expr< Grammar.s_list0sep $e1$ $e2$ $b$ >> ->
+      Slist0sep (safe_unsymbol e1) (safe_unsymbol e2) (unbool b)
+  | <:expr< Grammar.s_list1 $e$ >> -> Slist1 (safe_unsymbol e)
+  | <:expr< Grammar.s_list1sep $e1$ $e2$ $b$ >> ->
+      Slist1sep (safe_unsymbol e1) (safe_unsymbol e2) (unbool b)
+  | <:expr< Grammar.s_opt $e$ >> -> Sopt (safe_unsymbol e)
+  | <:expr< Grammar.s_flag $e$ >> -> Sflag (safe_unsymbol e)
+  | <:expr< Grammar.s_self >> -> Sself
+  | <:expr< Grammar.s_next >> -> Snext
+  | <:expr< Grammar.s_token $e$ >> -> Stoken (untoken e)
+  | <:expr< Grammar.s_rules $e$ >> -> Srules (rev_unlist unrule [] e)
+  | <:expr< Grammar.s_vala $ls$ $e$ >> ->
+      Svala (unlist unstring ls) None (safe_unsymbol e)
+  | _ -> raise Not_found
+  end
 and unsymbol =
   fun
   [ <:expr< Gramext.Sfacto $e$ >> -> unsymbol e
@@ -202,6 +246,7 @@ and unsymbol =
   | <:expr< Gramext.Sflag $e$ >> -> Sflag (unsymbol e)
   | <:expr< Gramext.Sself >> -> Sself
   | <:expr< Gramext.Snext >> -> Snext
+  | <:expr< Gramext.Scut >> -> Scut
   | <:expr< Gramext.Stoken $e$ >> -> Stoken (untoken e)
   | <:expr< Gramext.srules $e$ >> -> Srules (rev_unlist unrule [] e)
   | <:expr< Gramext.Svala $ls$ $e$ >> ->
@@ -222,6 +267,10 @@ value unentry =
       (Grammar.Entry.obj ($e$ : Grammar.Entry.e '$_$), $pos$, $ll$)
     >> ->
       (e, unposition pos, unlist unlevel ll)
+  | <:expr<
+      Grammar.extension ($e$ : Grammar.Entry.e '$_$) $pos$ $ll$
+    >> ->
+      (e, unposition pos, unlist unlevel ll)
   | _ -> raise Not_found ]
 ;
 
@@ -315,6 +364,7 @@ value anti_of_tok =
   | "QUESTIONIDENT" -> ["?"]
   | "QUESTIONIDENTCOLON" -> ["?:"]
   | "STRING" -> ["str"]
+  | "RAWSTRING" -> ["locstr"]
   | "TILDEIDENT" -> ["~"]
   | "TILDEIDENTCOLON" -> ["~:"]
   | "UIDENT" -> ["uid"; ""]
@@ -328,7 +378,7 @@ value rec rule force_vertic pc (sl, a) =
   [ None -> not_impl "rule 1" pc sl
   | Some a ->
       if sl = [] then
-        pprintf pc "@[<4>->%p %q@]" comment (MLast.loc_of_expr a)
+        pprintf pc "@[<4>→%p %q@]" comment (MLast.loc_of_expr a)
           (action expr) a "|"
       else
         match
@@ -336,7 +386,7 @@ value rec rule force_vertic pc (sl, a) =
             (fun () ->
                let s =
                  let pc = {(pc) with aft = ""} in
-                 pprintf pc "%p ->" (hlistl (semi_after psymbol) psymbol) sl
+                 pprintf pc "%p →" (hlistl (semi_after psymbol) psymbol) sl
                in
                Some s)
             (fun () -> None)
@@ -351,7 +401,7 @@ value rec rule force_vertic pc (sl, a) =
                  pprintf pc "%s@;<1 4>%q" s1 (action expr) a "|")
         | None ->
             let sl = List.map (fun s -> (s, ";")) sl in
-            pprintf pc "@[<2>%p ->@;%q@]" (plist psymbol 0) sl
+            pprintf pc "@[<2>%p →@;%q@]" (plist psymbol 0) sl
               (action expr) a "|" ] ]
 and psymbol pc (p, s) =
   match p with
@@ -397,6 +447,7 @@ and simple_symbol pc sy =
   [ Snterm <:expr< $lid:s$ >> -> pprintf pc "%s" s
   | Sself -> pprintf pc "SELF"
   | Snext -> pprintf pc "NEXT"
+  | Scut -> pprintf pc "/"
   | Srules rl ->
       match check_slist rl with
       [ Some _ -> pprintf pc "(%p)" symbol sy
@@ -576,15 +627,23 @@ value extend pc e =
         pprintf pc "EXTEND@;%p@ END" extend_body ex
       with
       [ Not_found -> pprintf pc "Grammar.extend@;@[<1>(%p)@]" expr e ]
+  | <:expr< Grammar.safe_extend $e$ >> ->
+      try
+        let ex = unextend_body e in
+        pprintf pc "EXTEND@;%p@ END" extend_body ex
+      with
+      [ Not_found -> pprintf pc "Grammar.safe_extend@;@[<1>(%p)@]" expr e ]
   | e -> expr pc e ]
 ;
 
 EXTEND_PRINTER
   pr_expr: LEVEL "apply"
-    [ [ <:expr< Grammar.extend $_$ >> as e -> next pc e ] ]
+    [ [ <:expr< Grammar.extend $_$ >> as e -> next pc e
+      | <:expr< Grammar.safe_extend $_$ >> as e -> next pc e ] ]
   ;
   pr_expr: LEVEL "simple"
-    [ [ <:expr< Grammar.extend $_$ >> as e -> extend pc e ] ]
+    [ [ <:expr< Grammar.extend $_$ >> as e -> extend pc e
+      | <:expr< Grammar.safe_extend $_$ >> as e -> extend pc e ] ]
   ;
 END;
 
