@@ -103,12 +103,11 @@ value rec is_irrefut_patt =
   | <:patt< { $list:fpl$ } >> ->
       List.for_all (fun (_, p) -> is_irrefut_patt p) fpl
   | <:patt< ($p$ : $_$) >> -> is_irrefut_patt p
-  | <:patt< ($list:pl$) >> -> List.for_all is_irrefut_patt pl
+  | <:patt< ($list:pl$, $closed:_$) >> -> List.for_all is_irrefut_patt pl
   | <:patt< (type $lid:_$) >> -> True
   | <:patt< (module $uidopt:_$ : $_$) >> -> True
   | <:patt< (module $uidopt:_$) >> -> True
-  | <:patt< ~{$list:lppo$} >> ->
-      List.for_all (fun (p, _) -> is_irrefut_patt p) lppo
+  | <:patt< ~{$p$ $opt:_$} >> -> is_irrefut_patt p
   | <:patt< ?{$p$ $opt:_$} >> -> is_irrefut_patt p
   | <:patt< [% $_extension:_$ ] >> -> flag_extensions_are_irrefutable.val
   | _ -> False ]
@@ -125,7 +124,7 @@ value rec get_defined_ident =
   | <:patt< $str:_$ >> -> []
   | <:patt< $chr:_$ >> -> []
   | <:patt< [| $list:pl$ |] >> -> List.flatten (List.map get_defined_ident pl)
-  | <:patt< ($list:pl$) >> -> List.flatten (List.map get_defined_ident pl)
+  | <:patt< ($list:pl$, $closed:_$) >> -> List.flatten (List.map get_defined_ident pl)
   | <:patt< ` $_$ >> -> []
   | <:patt< # $lilongid:_$ >> -> []
   | <:patt< $p1$ $p2$ >> -> get_defined_ident p1 @ get_defined_ident p2
@@ -283,6 +282,13 @@ value pr_extension atstring pc attr =
   pprintf pc "[%s%p]" atstring attribute_body (Pcaml.unvala attr)
 ;
 
+value labeled_ctyp pc (lab, ct) =
+  match (Pcaml.unvala lab) with [
+      None -> pprintf pc "%p" ctyp ct
+    | Some l -> pprintf pc "%s:%p" (Pcaml.unvala l) ctyp ct
+    ]
+;
+
 value longident_lident pc (lio, id) =
   match lio with
   [ None -> pprintf pc "%p" lident (Pcaml.unvala id)
@@ -829,20 +835,14 @@ value typevar pc s =
    ]
 ;
 
-value type_param pc (tv, (vari, inj)) =
+value type_param pc (tv, vastr) =
   let tv = Pcaml.unvala tv in
+  let vastr = Pcaml.unvala vastr in
   let tv_or_blank pc = fun [
     Some tv -> pprintf pc "%p" typevar tv
   | None -> pprintf pc "_" ] in
-  pprintf pc "%s%s%p"
-    (match vari with
-     [ Some True -> "+"
-     | Some False -> "-"
-     | None -> "" ])
-    (match inj with
-       [ True -> "!"
-       | False -> ""
-       ])
+  pprintf pc "%s%p"
+    vastr
     tv_or_blank tv
 ;
 
@@ -1791,7 +1791,7 @@ EXTEND_PRINTER
           pprintf pc "%p" (pr_extension "%") e
       | <:expr< $chr:s$ >> ->
           pprintf pc "'%s'" s
-      | MLast.ExOlb loc _ _ | MLast.ExLab loc _ ->
+      | MLast.ExOlb loc _ _ | MLast.ExLab loc _ _ ->
           error loc "labels not pretty printed (in expr); add pr_ro.cmo"
       | <:expr< $_$ $_$ >> | <:expr< assert $_$ >> | <:expr< lazy $_$ >> |
         <:expr< $_$ := $_$ >> |
@@ -1853,6 +1853,9 @@ EXTEND_PRINTER
       | <:patt< ($list:pl$) >> ->
           let pl = List.map (fun p -> (p, ",")) pl in
           pprintf pc "@[<1>(%p)@]" (plist patt 0) pl
+      | <:patt< ($list:pl$, ..) >> ->
+          let pl = List.map (fun p -> (p, ",")) pl in
+          pprintf pc "@[<1>(%p, ..)@]" (plist patt 0) pl
       | <:patt< {$list:lpl$} >> ->
           let lxl = List.map (fun lx -> (lx, ";")) lpl in
           pprintf pc "@[<1>{%p}@]" (plist (binding patt) 0) lxl
@@ -1904,7 +1907,7 @@ EXTEND_PRINTER
           pprintf pc "\"%s\"" s
       | <:patt< _ >> ->
           pprintf pc "_"
-      | MLast.PaLab loc _ | MLast.PaOlb loc _ _ ->
+      | MLast.PaLab loc _ _ | MLast.PaOlb loc _ _ ->
           error loc "labels not pretty printed (in patt); add pr_ro.cmo"
       | <:patt< `$s$ >> ->
           failwith "variants not pretty printed (in patt); add pr_ro.cmo"
@@ -1983,7 +1986,7 @@ EXTEND_PRINTER
                    vdl)
       | <:ctyp< ($list:tl$) >> ->
           let tl = List.map (fun t -> (t, " *")) tl in
-          pprintf pc "@[<1>(%p)@]" (plist ctyp 0) tl
+          pprintf pc "@[<1>(%p)@]" (plist labeled_ctyp 0) tl
       | <:ctyp< ( module $mt$ ) >> ->
           pprintf pc "@[(module@ %p)@]" module_type mt
       | <:ctyp:< $lid:t$ >> ->
