@@ -68,17 +68,11 @@ value amp_before elem pc x = pprintf pc "& %p" elem x;
 value and_before elem pc x = pprintf pc "and %p" elem x;
 value bar_before elem pc x = pprintf pc "| %p" elem x;
 
-value type_var pc (tv, (vari, inj)) =
+value type_var pc (tv, vastr) =
   let tv = Pcaml.unvala tv in
-  pprintf pc "%s%s%s"
-    (match vari with
-     [ Some True -> "+"
-     | Some False -> "-"
-     | None -> "" ])
-    (match inj with
-       [ True -> "!"
-       | False -> ""
-       ])
+  let vastr = Pcaml.unvala vastr in
+  pprintf pc "%s%s"
+    vastr
     (match tv with
      [ Some v -> "'" ^ v
      | None -> "_" ])
@@ -268,9 +262,8 @@ value sig_method_or_method_virtual pc virt priv s t item_attrs =
 
 EXTEND_PRINTER
   pr_patt: LEVEL "simple"
-    [ [ <:patt< ~{$list:lpop$} >> ->
-          let lpoe = List.map (fun pop -> (pop, ";")) lpop in
-          pprintf pc "@[~{%p}@]" (plist ipatt_tcon_opt_eq_patt 1) lpoe
+    [ [ <:patt< ~{$p$ = $p2$} >> ->
+          pprintf pc "@[~{%p}@]" ipatt_tcon_opt_eq_patt (p,<:vala< Some p2 >>)
       | <:patt< ~{$p$} >> ->
           pprintf pc "~{%p}" patt p
 
@@ -297,9 +290,8 @@ EXTEND_PRINTER
       | <:expr< object $opt:csp$ $list:csl$ end >> ->
           class_object pc (csp, csl) ]
     | "label"
-      [ <:expr< ~{$list:lpoe$} >> ->
-          let lpoe = List.map (fun poe -> (poe, ";")) lpoe in
-          pprintf pc "@[~{%p}@]" (plist ipatt_tcon_fun_binding 1) lpoe
+      [ <:expr< ~{$p$ $_opt:oe$} >> ->
+          pprintf pc "@[~{%p}@]" ipatt_tcon_fun_binding (p,oe)
       | <:expr< ?{$p$ = $e$} >> ->
           pprintf pc "@[<2>?{%p =@;%p}@]" patt p curr e
       | <:expr< ?{$p$} >> ->
