@@ -132,17 +132,11 @@ value braceless pc b =
    aft = sprintf ">}%s" pc.aft}
 ;
 
-value type_param pc (tv, (vari, inj)) =
+value type_param pc (tv, vastr) =
   let tv = Pcaml.unvala tv in
-  pprintf pc "%s%s%s"
-    (match vari with
-     [ Some True -> "+"
-     | Some False -> "-"
-     | None -> "" ])
-    (match inj with
-       [ True -> "!"
-       | False -> ""
-       ])
+  let vastr = Pcaml.unvala vastr in
+  pprintf pc "%s%s"
+   vastr
     (match tv with
      [ Some v -> "'" ^ v
      | None -> "_" ])
@@ -552,6 +546,7 @@ EXTEND_PRINTER
           plistbf 0 (paren pc "variantsgreater")
             (List.map (fun v -> (fun pc -> poly_variant_decl pc v, "")) vl)
       | <:ctyp< ( $list:tl$ ) >> ->
+          let tl = List.map snd tl in
           let tl = List.map (fun t -> (t, "")) tl in
           plistb curr 0 (paren pc "*") tl
       | <:ctyp< $t1$ -> $t2$ >> ->
