@@ -334,6 +334,10 @@ value parse_antiquot s =
   with [ Not_found | Failure _ -> None ]
 ;
 
+value int_of_string_result s =
+  try Some(int_of_string s) with [ Failure _ -> None ]
+;
+
 value parse_antiloc s =
   try
     let i = String.index s ':' in
@@ -474,6 +478,35 @@ and linedir_quote n s =
   | _ -> False ]
 ;
 
+value linedir_result n s =
+  let rec linedir n s =
+    match stream_peek_nth n s with
+      [ Some (' ' | '\t') -> linedir (n + 1) s
+      | Some ('0'..'9' as c) -> linedir_digits [c] (n + 1) s
+      | _ -> None ]
+  and linedir_digits acc n s =
+    match stream_peek_nth n s with
+      [ Some ('0'..'9' as c) -> linedir_digits [c::acc] (n + 1) s
+      | _ ->
+         match int_of_string_result (rev_implode acc) with [
+             Some lnum -> linedir_quote lnum n s
+           | None -> None
+           ]
+      ]
+  and linedir_quote lnum n s =
+    match stream_peek_nth n s with
+      [ Some (' ' | '\t') -> linedir_quote lnum (n + 1) s
+      | Some '"' -> linedir_qs lnum [] (n+1) s
+      | _ -> None ]
+  and linedir_qs lnum acc n s =
+    match stream_peek_nth n s with
+      [ Some '"' -> Some(lnum, rev_implode acc)
+      | Some ('\n' | '\r') -> None
+      | Some c -> linedir_qs lnum [c::acc] (n+1) s
+      | _ -> None ]
+  in linedir n s
+;
+
 value rec any_to_nl =
   lexer
   [ "\r" | "\n"
@@ -540,6 +573,21 @@ value raw_string_starter_p ctx strm =
   in predrec 1
 ;
 
+value zerobuf f buf strm =
+  f $empty strm
+;
+
+value rec ws =
+  lexer
+  [ [' '/ | '\t'/ | '\n'/] [ ws | ]
+  ]
+;
+
+value rec extattrident =
+  lexer
+  [ ident [ "." extattrident | ] ]
+;
+
 value comment_rawstring ctx bp (buf : Plexing.Lexbuf.t) strm =
   if not (raw_string_starter_p ctx strm) then
     buf
@@ -549,11 +597,32 @@ value comment_rawstring ctx bp (buf : Plexing.Lexbuf.t) strm =
   add_string buf rs
 ;
 
+
+value comment_quoted_extension1 ctx bp extid buf strm =
+  let (delim, s) = rawstring0 ctx bp $empty strm in
+  let exts = Printf.sprintf "{%%%s %s|%s|%s}" extid delim s delim in
+  add_string buf exts
+;
+
+value comment_quoted_extension0 ctx bp extid =
+  lexer
+  [ ws (zerobuf (comment_quoted_extension1 ctx bp extid))
+  | (zerobuf (comment_quoted_extension1 ctx bp extid))
+  ]
+;
+
+value comment_quoted_extension ctx bp =
+  lexer [
+    extattrident (zerobuf (comment_quoted_extension0 ctx bp $buf))
+  ]
+;
+
 value comment ctx bp =
   comment where rec comment =
     lexer
     [ "*)"
     | "*" comment!
+    | "{%"/ (comment_quoted_extension ctx bp)! comment!
     | "{" (comment_rawstring ctx bp)! comment!
     | "(*" comment! comment!
     | "(" comment!
@@ -576,39 +645,32 @@ value keyword_or_error_or_rawstring ctx bp (loc,s) buf strm =
     ]
   else
     let (delim, s) = rawstring0 ctx bp $empty strm in
-    ("STRING", String.escaped s)
+    ("RAWSTRING", Printf.sprintf "%d:%s" (String.length delim) (String.escaped s))
 ;
 
-value zerobuf f buf strm =
-  f $empty strm
-;
-
-value rec ws =
+value rec keepws =
   lexer
-  [ [' '/ | '\t'/ | '\n'/] [ ws | ]
+  [ [' ' | '\t' | '\n'] [ keepws | ]
   ]
 ;
 
-value rec extattrident =
-  lexer
-  [ ident [ "." extattrident | ] ]
-;
-
-value quoted_extension1 ctx (bp, _) extid buf strm =
+value quoted_extension1 kind ctx (bp, _) buf strm =
   let (delim, s) = rawstring0 ctx bp $empty strm in
-  ("QUOTEDEXTENSION", extid^":"^(String.escaped s))
+  let lhs = $buf in
+  let qext = Printf.sprintf "%s%s|%s|%s}" lhs delim s delim in
+  (kind, qext)
 ;
 
-value quoted_extension0 ctx (bp, _) extid =
+value quoted_extension0 kind ctx (bp, _) =
   lexer
-  [ ws (zerobuf (quoted_extension1 ctx (bp, $pos) extid))
-  | (zerobuf (quoted_extension1 ctx (bp, $pos) extid))
+  [ keepws (quoted_extension1 kind ctx (bp, $pos))
+  | (quoted_extension1 kind ctx (bp, $pos))
   ]
 ;
 
-value quoted_extension ctx (bp, _) =
+value quoted_extension (kind : string) ctx (bp, _) =
   lexer [
-    extattrident (zerobuf (quoted_extension0 ctx (bp, $pos) $buf))
+    extattrident (quoted_extension0 kind ctx (bp, $pos))
   ]
 ;
 value dotsymbolchar = lexer
@@ -708,7 +770,8 @@ value next_token_after_spaces ctx bp =
   | "[" -> keyword_or_error ctx (bp, $pos) $buf
   | "{" ?= [ "<<" | "<:" ] -> keyword_or_error ctx (bp, $pos) $buf
   | "{<" -> keyword_or_error ctx (bp, $pos) $buf
-  | "{%"/ (zerobuf (quoted_extension ctx (bp, $pos)))
+  | "{%%" (quoted_extension "QUOTEDEXTENSION_ITEM" ctx (bp, $pos))
+  | "{%" (quoted_extension "QUOTEDEXTENSION_EXPR" ctx (bp, $pos))
   | "{:" -> keyword_or_error ctx (bp, $pos) $buf
   | "{" (keyword_or_error_or_rawstring ctx bp ((bp, $pos),$buf))
   | ".." -> keyword_or_error ctx (bp, $pos) ".."
@@ -746,17 +809,20 @@ value rec next_token ctx buf =
     }
   | [: `'#' when bp = Plexing.bol_pos.val.val; s :] ->
       let comm = get_comment buf () in
-      if linedir 1 s then do {
-        let buf = any_to_nl ($add '#') s in
-        incr Plexing.line_nb.val;
-        Plexing.bol_pos.val.val := Stream.count s;
-        ctx.set_line_nb ();
-        ctx.after_space := True;
-        next_token ctx buf s
-      }
-      else
-        let loc = ctx.make_lined_loc (bp, bp + 1) comm in
-        (keyword_or_error ctx (bp, bp + 1) "#", loc)
+      match linedir_result 1 s with [
+          Some (linenum, fname) -> do {
+            let buf = any_to_nl ($add '#') s in
+            Plexing.line_nb.val.val := linenum ;
+            Plexing.input_file.val := fname ;
+            Plexing.bol_pos.val.val := Stream.count s;
+            ctx.set_line_nb ();
+            ctx.after_space := True;
+            next_token ctx buf s
+          }
+        | None ->
+           let loc = ctx.make_lined_loc (bp, bp + 1) comm in
+           (keyword_or_error ctx (bp, bp + 1) "#", loc)
+        ]
   | [: `'(';
        a =
          parser
@@ -923,8 +989,8 @@ value using_token ctx kwd_table (p_con, p_prm) =
         | _ -> () ]
   | "TILDEIDENT" | "TILDEIDENTCOLON" | "QUESTIONIDENT" |
     "QUESTIONIDENTCOLON" | "INT" | "INT_l" | "INT_L" | "INT_n" | "FLOAT" |
-    "QUOTEDEXTENSION" |
-    "CHAR" | "STRING" | "QUOTATION" | "GIDENT" |
+    "QUOTEDEXTENSION_EXPR" | "QUOTEDEXTENSION_ITEM" |
+    "CHAR" | "STRING" | "RAWSTRING" | "QUOTATION" | "GIDENT" |
     "ANTIQUOT" | "ANTIQUOT_LOC" | "EOI" ->
       ()
   | _ ->
@@ -951,6 +1017,7 @@ value text =
   | ("INT", s) -> "'" ^ s ^ "'"
   | ("FLOAT", "") -> "float"
   | ("STRING", "") -> "string"
+  | ("RAWSTRING", "") -> "raw string"
   | ("CHAR", "") -> "char"
   | ("QUOTATION", "") -> "quotation"
   | ("ANTIQUOT", k) -> "antiquot \"" ^ k ^ "\""
