@@ -1,6 +1,6 @@
 (* camlp5r *)
 (* plexing.ml,v *)
-(* Copyright (c) INRIA 2007-2014 *)
+(* Copyright (c) INRIA 2007-2017 *)
 
 open Versdep;
 
@@ -10,29 +10,17 @@ exception Error of string;
 
 type location = Ploc.t;
 type location_function = int -> location;
-type lexer_func 'te = Stream.t char -> (Stream.t 'te * location_function);
-
-type lexer 'te =
-  { tok_func : lexer_func 'te;
-    tok_using : pattern -> unit;
-    tok_removing : pattern -> unit;
-    tok_match : mutable pattern -> 'te -> string;
-    tok_text : pattern -> string;
-    tok_comm : mutable option (list location) }
-;
 
 value make_loc = Ploc.make_unlined;
 value dummy_loc = Ploc.dummy;
 
-value lexer_text (con, prm) =
-  if con = "" then "'" ^ prm ^ "'"
-  else if prm = "" then con
-  else con ^ " '" ^ prm ^ "'"
-;
+module Locations = struct
+  type t = { locations : ref (array (option Ploc.t)) ; overflow : ref bool } ;
+  value locerr () = failwith "Lexer: location function" ;
+  value create () = { locations = ref (array_create 1024 None) ; overflow = ref False } ;
 
-value locerr () = failwith "Lexer: location function";
-value loct_create () = (ref (array_create 1024 None), ref False);
-value loct_func (loct, ov) i =
+value lookup t i =
+  let (loct, ov) = (t.locations, t.overflow) in
   match
     if i < 0 || i >= Array.length loct.val then
       if ov.val then Some dummy_loc else None
@@ -41,7 +29,8 @@ value loct_func (loct, ov) i =
   [ Some loc -> loc
   | None -> locerr () ]
 ;
-value loct_add (loct, ov) i loc =
+value add t i loc =
+  let (loct, ov) = (t.locations, t.overflow) in
   if i >= Array.length loct.val then
     let new_tmax = Array.length loct.val * 2 in
     if new_tmax < Sys.max_array_length then do {
@@ -54,17 +43,38 @@ value loct_add (loct, ov) i loc =
   else loct.val.(i) := Some loc
 ;
 
+end ;
+
+type lexer_func 'te = Stream.t char -> (Stream.t 'te * Locations.t);
+
+type lexer 'te =
+  { tok_func : lexer_func 'te;
+    tok_using : pattern -> unit;
+    tok_removing : pattern -> unit;
+    tok_match : mutable pattern -> 'te -> string;
+    tok_text : pattern -> string;
+    tok_comm : mutable option (list location);
+    kwds : Hashtbl.t string string
+  }
+;
+
+value lexer_text (con, prm) =
+  if con = "" then "'" ^ prm ^ "'"
+  else if prm = "" then con
+  else con ^ " '" ^ prm ^ "'"
+;
+
 value make_stream_and_location next_token_loc =
-  let loct = loct_create () in
+  let loct = Locations.create () in
   let ts =
     Stream.from
       (fun i -> do {
          let (tok, loc) = next_token_loc () in
-         loct_add loct i loc;
+         Locations.add loct i loc;
          Some tok
        })
   in
-  (ts, loct_func loct)
+  (ts, loct)
 ;
 
 value lexer_func_of_parser next_token_loc cs =
@@ -73,32 +83,37 @@ value lexer_func_of_parser next_token_loc cs =
   make_stream_and_location (fun () -> next_token_loc (cs, line_nb, bolpos))
 ;
 
-value lexer_func_of_ocamllex lexfun cs =
+value lexer_func_of_ocamllex_located lexfun cs =
   let lb =
     Lexing.from_function
       (fun s n ->
          try do { string_set s 0 (Stream.next cs); 1 } with
          [ Stream.Failure -> 0 ])
   in
-  let next_token_loc _ =
+  let next_token_func () = lexfun lb in
+  make_stream_and_location next_token_func
+;
+
+value lexer_func_of_ocamllex lexfun cs =
+  let lexfun_located lb =
     let tok = lexfun lb in
     let loc = make_loc (Lexing.lexeme_start lb, Lexing.lexeme_end lb) in
     (tok, loc)
   in
-  make_stream_and_location next_token_loc
+  lexer_func_of_ocamllex_located lexfun_located cs
 ;
 
 (* Char and string tokens to real chars and string *)
 
 value buff = ref (string_create 80);
 value store len x = do {
-  if len >= String.length buff.val then
-    buff.val := buff.val ^ string_create (String.length buff.val)
+  if len >= string_length buff.val then
+    buff.val := string_cat buff.val (string_create (string_length buff.val))
   else ();
   string_set buff.val len x;
   succ len
 };
-value get_buff len = String.sub buff.val 0 len;
+value get_buff len = string_sub buff.val 0 len;
 
 value valch x = Char.code x - Char.code '0';
 value valch_a x = Char.code x - Char.code 'a' + 10;
@@ -175,7 +190,7 @@ value eval_char s =
 ;
 
 value eval_string loc s =
-  loop 0 0 where rec loop len i =
+  bytes_to_string (loop 0 0) where rec loop len i =
     if i = String.length s then get_buff len
     else
       let (len, i) =
@@ -219,7 +234,7 @@ value restore_lexing_info = ref None;
 
 value rev_implode l =
   let s = string_create (List.length l) in
-  loop (String.length s - 1) l where rec loop i =
+  bytes_to_string (loop (string_length s - 1) l) where rec loop i =
     fun
     [ [c :: l] -> do { string_unsafe_set s i c; loop (i - 1) l }
     | [] -> s ]
