@@ -1,6 +1,6 @@
 (* camlp5r *)
 (* ploc.ml,v *)
-(* Copyright (c) INRIA 2007-2014 *)
+(* Copyright (c) INRIA 2007-2017 *)
 
 #load "pa_macro.cmo";
 
@@ -42,19 +42,6 @@ value bol_pos_last loc = loc.bol_pos_last;
 value comment loc = loc.comm;
 value comment_last loc = loc.ecomm;
 
-IFDEF OCAML_VERSION <= OCAML_1_07 OR COMPATIBLE_WITH_OLD_OCAML THEN
-  value with_bp_ep l bp ep =
-    {fname = l.fname; line_nb = l.line_nb; bol_pos = l.bol_pos;
-     line_nb_last = l.line_nb_last; bol_pos_last = l.bol_pos_last; bp = bp;
-     ep = ep; comm = l.comm; ecomm = l.ecomm}
-  ;
-  value with_comm l comm =
-    {fname = l.fname; line_nb = l.line_nb; bol_pos = l.bol_pos;
-     line_nb_last = l.line_nb_last; bol_pos_last = l.bol_pos_last; bp = l.bp;
-     ep = l.ep; comm = comm; ecomm = l.ecomm}
-  ;
-END;
-
 value encl loc1 loc2 =
   if loc1.bp < loc2.bp then
     if loc1.ep < loc2.ep then
@@ -77,6 +64,9 @@ value after loc sh len =
   {(loc) with bp = loc.ep + sh; ep = loc.ep + sh + len}
 ;
 value with_comment loc comm = {(loc) with comm = comm};
+value with_comment_last loc ecomm = {(loc) with ecomm = ecomm};
+value with_line_nb_last loc n = {(loc) with line_nb_last = n};
+value with_bol_pos_last loc n = {(loc) with bol_pos_last = n};
 
 value name = ref "loc";
 
@@ -215,3 +205,17 @@ value make line_nb bol_pos (bp, ep) =
   {fname = ""; line_nb = line_nb; bol_pos = bol_pos; line_nb_last = line_nb;
    bol_pos_last = bol_pos; bp = bp; ep = ep; comm = ""; ecomm = ""}
 ;
+
+
+value string_of_loc fname line bp ep =
+  match Sys.os_type with
+  [ "MacOS" ->
+      Printf.sprintf "File \"%s\"; line %d; characters %d to %d\n### " fname line
+        bp ep
+  | _ ->
+      Printf.sprintf "File \"%s\", line %d, characters %d-%d:\n" fname line bp ep ]
+;
+
+value string_of_location {fname=fname; bp=bp; ep=ep; line_nb=line; bol_pos=bol} =
+  string_of_loc fname line (bp - bol) (ep - bol)
+;
