@@ -73,22 +73,51 @@ value mknoloc txt =
   mkloc loc_none txt
 ;
 
-value ocaml_id_or_li_of_string_list loc sl =
-    let mkli s =
-      loop (fun s -> Lident s) where rec loop f =
-        fun
-        [ [i :: il] -> loop (fun s -> Ldot (f i) s) il
-        | [] -> f s ]
-    in
-    match List.rev sl with
-    [ [] -> None
-    | [s :: sl] -> Some (mkli s (List.rev sl)) ]
+value ocaml_longident_ldot (loc : Location.t) li s =
+IFDEF OCAML_VERSION < OCAML_5_4_0 THEN
+ Ldot li s
+ELSE
+ Ldot (mkloc loc li) (mkloc loc s)
+END
+;
+
+value ocaml_longident_lapply (loc : Location.t) li1 li2 =
+IFDEF OCAML_VERSION < OCAML_5_4_0 THEN
+ Lapply li1 li2
+ELSE
+ Lapply (mkloc loc li1) (mkloc loc li2)
+END
+;
+value ocaml_longident_lident (loc : Location.t) s = Lident s ;
+
+value ocaml_longident_option_of_string_list (loc : Location.t) sl =
+  match sl with [
+      [] -> None
+    | [s :: sl] ->
+       let rec loop acc = fun [
+             [] -> acc
+           | [s :: sl] -> loop (ocaml_longident_ldot loc acc s) sl
+           ]
+       in Some (loop (ocaml_longident_lident loc s) sl)
+    ]
+;
+
+value ocaml_longident_of_string_list loc sl =
+  match ocaml_longident_option_of_string_list loc sl with [
+      None -> failwith "ocaml_longident_of_string_list: empty argument"
+    | Some li -> li
+    ]
 ;
 
 value not_extended_longident =
   let rec not_extended = fun [
     Lident _ -> True
-  | Ldot li _ -> not_extended li
+  | Ldot li _ -> 
+IFDEF OCAML_VERSION < OCAML_5_4_0 THEN
+  not_extended li
+ELSE
+  not_extended li.txt
+END
   | Lapply _ _ -> False
   ] in 
   not_extended
@@ -414,28 +443,65 @@ value ocaml_mkfield_var loc =
   []
 ;
 
+value ocaml_variance_map =
 IFDEF OCAML_VERSION < OCAML_4_12_0 THEN
-value convert_camlp5_variance (va, inj) = do {
-  assert (inj = False) ;
-  let va = match va with [
-    Some False -> Contravariant
-  | Some True -> Covariant
-  | _ -> Invariant ] in
-  va
-}
-;
+[("",("",Invariant))
+;("+",("+",Covariant))
+;("-",("-",Contravariant))
+]
+ELSIFDEF OCAML_VERSION < OCAML_5_4_0 THEN
+[("",("",(NoVariance, NoInjectivity)))
+;("+",("+",(Covariant, NoInjectivity)))
+;("-",("-",(Contravariant, NoInjectivity)))
+;("!",("!",(NoVariance, Injective)))
+;("+!",("+!",(Covariant, Injective)))
+;("!+",("+!",(Covariant, Injective)))
+;("-!",("-!",(Contravariant, Injective)))
+;("!-",("-!",(Contravariant, Injective)))
+;("+!",("+!",(Covariant, Injective)))
+;("!+",("+!",(Covariant, Injective)))
+;("-!",("-!",(Contravariant, Injective)))
+;("!-",("-!",(Contravariant, Injective)))
+]
 ELSE
-value convert_camlp5_variance (va, inj) =
-  let va = match va with [
-    Some False -> Contravariant
-  | Some True -> Covariant
-  | _ -> NoVariance ] in
-  let inj = match inj with [
-    True -> Injective
-  | False -> NoInjectivity ] in
-  (va, inj)
-  ;
-END;
+[("",("",(NoVariance, NoInjectivity)))
+;("+",("+",(Covariant, NoInjectivity)))
+;("-",("-",(Contravariant, NoInjectivity)))
+;("!",("!",(NoVariance, Injective)))
+;("+!",("+!",(Covariant, Injective)))
+;("!+",("+!",(Covariant, Injective)))
+;("-!",("-!",(Contravariant, Injective)))
+;("!-",("-!",(Contravariant, Injective)))
+;("+!",("+!",(Covariant, Injective)))
+;("!+",("+!",(Covariant, Injective)))
+;("-!",("-!",(Contravariant, Injective)))
+;("!-",("-!",(Contravariant, Injective)))
+;("+-",("+-",(Bivariant, NoInjectivity)))
+;("-+",("-+",(Bivariant, NoInjectivity)))
+;("+-!",("+-!",(Bivariant, Injective)))
+;("-+!",("-+!",(Bivariant, Injective)))
+;("!+-",("+-!",(Bivariant, Injective)))
+;("!-+",("-+!",(Bivariant, Injective)))
+]
+END
+;
+
+value ocaml_normalize_camlp5_variance vastr =
+  match List.assoc vastr ocaml_variance_map with [
+      (v,_) -> v
+    | exception Not_found ->
+       failwith (Printf.sprintf "ocaml_normalize_camp5_variance: internal error, variance \"%s\" not found" vastr)
+    ]
+;
+
+
+value convert_camlp5_variance vastr =
+  match List.assoc vastr ocaml_variance_map with [
+      (_,v) -> v
+    | exception Not_found ->
+       failwith (Printf.sprintf "convert_camp5_variance: internal error, variance \"%s\" not found" vastr)
+    ]
+;
 
 IFDEF OCAML_VERSION < OCAML_4_08_0 THEN
 value ocaml_ec_tuple ?{alg_attributes=[]} _ _ _ _ = assert False ;
@@ -694,8 +760,17 @@ value ocaml_ptyp_variant loc catl clos sl_opt =
   END
 ;
 
-value ocaml_package_type li ltl =
+value ocaml_package_type loc li ltl : package_type =
+IFDEF OCAML_VERSION < OCAML_5_4_0 THEN
   (mknoloc li, List.map (fun (li, t) → (mkloc t.ptyp_loc li, t)) ltl)
+ELSE
+  {
+    ppt_path = mkloc loc li
+  ; ppt_cstrs = List.map (fun (li, t) -> (mkloc t.ptyp_loc li, t)) ltl
+  ; ppt_loc = loc
+  ; ppt_attrs = []
+  }
+END
 ;
 
 value ocaml_pconst_char c =
@@ -770,6 +845,17 @@ value ocaml_pexp_constraint e ot1 ot2 =
         end
     end
 ;
+value ocaml_pexp_tuple x =
+  IFDEF OCAML_VERSION < OCAML_5_4_0 THEN
+    let x = List.map (fun [
+                          (None, t) -> t
+                        | (Some _, t) -> failwith "Pexp_tuple: labeled tuples only available with ocaml >= 5.04"
+                        ]) x in
+    Pexp_tuple x
+  ELSE
+    Pexp_tuple x
+  END
+;
 
 value ocaml_pexp_construct loc li po chk_arity =
     Pexp_construct (mkloc loc li) po
@@ -781,8 +867,9 @@ value ocaml_pexp_construct_args =
     | _ -> None ]
 ;
 
-value mkexp_ocaml_pexp_construct_arity loc li_loc li al =
-  let a = ocaml_mkexp loc (Pexp_tuple al) in
+value mkexp_ocaml_pexp_construct_arity loc li_loc li (al : list expression) =
+  let al = List.map (fun x -> (None, x)) al in
+  let a = ocaml_mkexp loc (ocaml_pexp_tuple al) in
   IFDEF OCAML_VERSION < OCAML_4_08_0 THEN
     {pexp_desc = ocaml_pexp_construct li_loc li (Some a) True;
      pexp_loc = loc;
@@ -925,11 +1012,32 @@ value ocaml_pexp_override sel =
   Pexp_override sel
 ;
 
-value ocaml_pexp_pack =
-  (Some (Right (fun me -> Pexp_pack me, fun pt -> Ptyp_package pt)) :
-     option (choice ('a -> 'b -> 'c) 'd))
+value ocaml_ptyp_pack pt = Ptyp_package pt ;
+
+value ocaml_pexp_pack loc me pto =
+IFDEF OCAML_VERSION < OCAML_5_4_0 THEN
+  let e = Pexp_pack me in
+  if pto = None then e
+  else
+    let pto : option Parsetree.core_type_desc = option_map ocaml_ptyp_pack pto in
+    let pto : option Parsetree.core_type = option_map (ocaml_mktyp loc) pto in
+    ocaml_pexp_constraint (ocaml_mkexp loc e) pto None
+ELSE
+  Pexp_pack me pto
+END
 ;
 
+value ocaml_ptyp_tuple x =
+  IFDEF OCAML_VERSION < OCAML_5_4_0 THEN
+    let x = List.map (fun [
+                          (None, t) -> t
+                        | (Some _, t) -> failwith "Ptyp_tuple: labeled tuples only available with ocaml >= 5.04"
+                        ]) x in
+    Ptyp_tuple x
+  ELSE
+    Ptyp_tuple x
+  END
+;
 value ocaml_pexp_poly =
   Some (fun e t -> Pexp_poly e t)
 ;
@@ -1041,14 +1149,27 @@ value ocaml_ppat_construct_args =
     | _ -> None ]
 ;
 
+value ocaml_ppat_tuple l cflag =
+  IFDEF OCAML_VERSION < OCAML_5_4_0 THEN
+    let l = List.map (fun [ (None, p) -> p | (Some _, _) -> failwith "Ppat_tuple: labeled tuples only available at ocaml >= 5.4" ]) l in
+    if (cflag = Open) then failwith "Ppat_tuple: labeled tuples only available at ocaml >= 5.4"
+    else Ppat_tuple l
+  ELSE
+    Ppat_tuple l cflag
+  END
+;
+
 value mkpat_ocaml_ppat_construct_arity loc li_loc li tyvl al =
+  let al = 
+      List.map (fun x -> (None, x)) al
+  in
   IFDEF OCAML_VERSION < OCAML_4_08_0 THEN
-    let a = ocaml_mkpat loc (Ppat_tuple al) in
+    let a = ocaml_mkpat loc (ocaml_ppat_tuple al Closed) in
     {ppat_desc = ocaml_ppat_construct li_loc li (Some (tyvl, a)) True;
      ppat_loc = loc;
      ppat_attributes = [(mkloc loc "ocaml.explicit_arity", PStr [])]}
   ELSE
-    let a = ocaml_mkpat loc (Ppat_tuple al) in
+    let a = ocaml_mkpat loc (ocaml_ppat_tuple al Closed) in
     {ppat_desc = ocaml_ppat_construct li_loc li (Some (tyvl, a)) True;
      ppat_loc = loc; ppat_loc_stack = [];
      ppat_attributes =
@@ -1465,8 +1586,7 @@ value ocaml_pcl_constraint =
   Some (fun ce ct -> Pcl_constraint ce ct)
 ;
 
-value ocaml_pcl_fun =
-    Some (fun lab ceo p ce -> Pcl_fun (labelled lab) ceo p ce)
+value ocaml_pcl_fun lab ceo p ce = Pcl_fun (labelled lab) ceo p ce
 ;
 
 value ocaml_pcl_let =
