@@ -67,15 +67,15 @@ value mklazy loc e =
       let ghpat = mkpat loc in
       let ghexp = mkexp loc in
       let void_pat =
-        ghpat (ocaml_ppat_construct (mkloc loc) (Lident "()") None False)
+        ghpat (ocaml_ppat_construct (mkloc loc) (ocaml_longident_lident (mkloc loc) "()") None False)
       in
       let pwe = ocaml_case (void_pat, None, mkloc loc, e) in
       let f = ghexp (ocaml_pexp_function "" None [pwe]) in
-      let delayed = Ldot (Lident "Lazy") "Delayed" in
+      let delayed = ocaml_longident_of_string_list (mkloc loc) ["Lazy"; "Delayed"] in
       let cloc = mkloc loc in
       let df = ghexp (ocaml_pexp_construct cloc delayed (Some f) False) in
       let r =
-        ghexp (ocaml_pexp_ident cloc (Ldot (Lident "Pervasives") "ref"))
+        ghexp (ocaml_pexp_ident cloc (ocaml_longident_of_string_list (mkloc loc) ["Pervasives"; "ref"]))
       in
       ghexp (ocaml_pexp_apply r [("", df)]) ]
 ;
@@ -136,8 +136,8 @@ value conv_lab = do {
   fun s → try Hashtbl.find t s with [ Not_found → s ]
 };
 
-value array_function str name =
-  Ldot (Lident str) (if fast.val then "unsafe_" ^ name else name)
+value array_function loc str name =
+  ocaml_longident_of_string_list (mkloc loc) [str; (if fast.val then "unsafe_" ^ name else name)]
 ;
 
 value uv c =
@@ -159,42 +159,30 @@ value uident_True__True = fun [
 ]
 ;
 
-value mkli s =
-  loop (fun s → Lident s) where rec loop f =
-    fun
-    [ [i :: il] → loop (fun s → Ldot (f i) s) il
-    | [] → f s ]
-;
+value mkli loc s sl = ocaml_longident_of_string_list (mkloc loc) [s :: sl] ;
 
-value long_id_of_string_list loc sl =
-  match List.rev sl with
+value longident_of_string_list loc sl =
+  match sl with
   [ [] → error loc "bad ast"
-  | [s :: sl] → mkli s (List.rev sl) ]
+  | _ → ocaml_longident_of_string_list (mkloc loc) sl ]
 ;
 
-value concat_long_ids l1 l2 =
-let rec crec lhs = fun [
-  Lident s -> Ldot lhs s
-| Ldot li s -> Ldot (crec lhs li) s
-| Lapply lif liarg -> Lapply (crec lhs lif) liarg
-] in
-crec l1 l2
-;
-
-value longid_long_id_gen ~{extended} =
+value longid_to_longident_gen ~{extended} =
   let rec lrec = fun
-  [ <:extended_longident< $longid:me1$ ( $longid:me2$ ) >> when extended ->
-      Lapply (lrec me1) (lrec me2)
+  [ <:extended_longident:< $longid:me1$ ( $longid:me2$ ) >> when extended ->
+      ocaml_longident_lapply (mkloc loc) (lrec me1) (lrec me2)
   | <:extended_longident:< $longid:_$ ( $longid:_$ ) >>  ->
       Ploc.raise loc (Failure "longid_long_id: extended longid forbidden here (application not allowed)")
-  | <:extended_longident< $longid:me1$ . $_uid:uid$ >> → Ldot (lrec me1) (uident_True__True (uv uid))
-  | <:extended_longident< $_uid:s$ >> → Lident (uident_True__True(uv s))
+  | <:extended_longident:< $longid:me1$ . $_uid:uid$ >> →
+      ocaml_longident_ldot (mkloc loc) (lrec me1) (uident_True__True (uv uid))
+  | <:extended_longident:< $_uid:s$ >> →
+      ocaml_longident_lident (mkloc loc) (uident_True__True(uv s))
   | LiXtr loc _ _ -> Ploc.raise loc (Failure "longid_long_id: LiXtr forbidden here")
   ]
   in lrec
 ;
-value longid_long_id = longid_long_id_gen ~{extended=False} ;
-value extended_longid_long_id = longid_long_id_gen ~{extended=True} ;
+value longid_to_longident = longid_to_longident_gen ~{extended=False} ;
+value extended_longid_to_longident = longid_to_longident_gen ~{extended=True} ;
 
 value rec ctyp_fa al =
   fun
@@ -202,57 +190,67 @@ value rec ctyp_fa al =
   | f → (f, al) ]
 ;
 
-value rec module_expr_long_id =
-  fun
-  [ <:module_expr< $me1$ ( $me2$ ) >> →
-      Lapply (module_expr_long_id me1) (module_expr_long_id me2)
-  | <:module_expr< $m$ . $uid:s$ >> → Ldot (module_expr_long_id m) s
-  | <:module_expr< $uid:s$ >> → Lident s
+value module_expr_to_extended_longid =
+  let rec mrec = fun
+  [ <:module_expr:< $me1$ ( $me2$ ) >> →
+    <:extended_longident< $longid:(mrec me1)$ ( $longid:(mrec me2)$ ) >>
+  | <:module_expr:< $m$ . $uid:s$ >> → <:extended_longident< $longid:(mrec m)$ . $uid:s$ >>
+  | <:module_expr:< $uid:s$ >> → <:extended_longident< $uid:s$ >>
   | t → error (loc_of_module_expr t) "bad module expr long ident" ]
+in mrec
 ;
 
-value rec expr_long_id = fun
-  [ <:expr< $longid:li$ >> -> longid_long_id li
+value module_expr_to_longident me =
+  me |> module_expr_to_extended_longid |> extended_longid_to_longident ;
+
+value rec expr_to_longident = fun
+  [ <:expr< $longid:li$ >> -> longid_to_longident li
   | _ -> failwith "expr_long_id: unexpected expr"
   ]
 ;
 
-value longid_lident_long_id_gen ~{extended} (lio,s) = match (lio, s) with [
-    (Some li, s) → Ldot (longid_long_id_gen ~{extended=extended} (uv li)) (uv s)
-  | (None, s) → Lident (uv s)
+value lilongid_to_longident_gen ~{extended} loc (lio,s) = match (lio, s) with [
+    (Some li, s) →
+      ocaml_longident_ldot (mkloc loc) (longid_to_longident_gen ~{extended=extended} (uv li)) (uv s)
+  | (None, s) →
+      ocaml_longident_lident (mkloc loc) (uv s)
 ]
 ;
-value longid_lident_long_id = longid_lident_long_id_gen ~{extended=False} ;
-value extended_longid_lident_long_id = longid_lident_long_id_gen ~{extended=True} ;
+value lilongid_to_longident loc = lilongid_to_longident_gen ~{extended=False} loc ;
+value extended_lilongid_to_longident loc = lilongid_to_longident_gen ~{extended=True} loc ;
 
-value rec ctyp_long_id =
-  fun
+value ctyp_to_longident =
+let rec crec = fun
   [
-    <:ctyp< $m1$ $m2$ >> →
-      let (is_cls, li1) = ctyp_long_id m1 in
-      let (_, li2) = ctyp_long_id m2 in
-      (is_cls, Lapply li1 li2)
+    <:ctyp:< $m1$ $m2$ >> →
+      let (is_cls, li1) = crec m1 in
+      let (_, li2) = crec m2 in
+      (is_cls, ocaml_longident_lapply (mkloc loc) li1 li2)
+
   | <:ctyp:< $longid:m$ . $lid:id$ >> →
-      let li1 = extended_longid_long_id m in
-      let (is_cls, li2) = ctyp_long_id <:ctyp< $lid:id$ >> in
-      (is_cls, concat_long_ids li1 li2)
-  | <:ctyp< $lid:s$ >> → (False, Lident s)
-  | TyCls loc cli → (True, extended_longid_lident_long_id (uv cli))
+      let li1 = extended_longid_to_longident m in
+      (False, ocaml_longident_ldot (mkloc loc) li1 id)
+
+  | <:ctyp:< $lid:s$ >> → (False, ocaml_longident_lident (mkloc loc) s)
+
+  | TyCls loc cli → (True, extended_lilongid_to_longident loc (uv cli))
   | t → error (loc_of_ctyp t) "incorrect type" ]
+in crec
 ;
 
-value module_type_long_id = fun [
-  <:module_type< $longid:li$ . $_lid:lid$ >> -> Ldot (extended_longid_long_id li) (uv lid)
-| <:module_type< $longid:li$ >> -> extended_longid_long_id li
-| _ -> failwith "module_type_long_id"
+value module_type_to_longident = fun [
+  <:module_type:< $longid:li$ . $_lid:lid$ >> ->
+    ocaml_longident_ldot (mkloc loc) (extended_longid_to_longident li) (uv lid)
+| <:module_type< $longid:li$ >> -> extended_longid_to_longident li
+| _ -> failwith "module_type_to_longident"
 ]
 ;
 
 value class_type_long_id = fun [
-  <:class_type< $longid:li$ . $_lid:lid$ >> ->
-    Ldot (extended_longid_long_id li) (uv lid)
-| <:class_type< $_lid:lid$ >> ->
-    Lident (uv lid)
+  <:class_type:< $longid:li$ . $_lid:lid$ >> ->
+    ocaml_longident_ldot (mkloc loc) (extended_longid_to_longident li) (uv lid)
+| <:class_type:< $_lid:lid$ >> ->
+    ocaml_longident_lident (mkloc loc) (uv lid)
 | _ -> failwith "class_type_long_id"
 ]
 ;
@@ -260,6 +258,7 @@ value class_type_long_id = fun [
 value mktype ~{item_attributes} loc tn tl cl tk pf tm =
   let (params, variance) = List.split tl in
   let params = List.map uv params in
+  let variance = List.map uv variance in
   match ocaml_type_declaration tn params cl tk pf tm (mkloc loc) variance item_attributes with
   [ Right td → td
   | Left msg → error loc msg ]
@@ -300,11 +299,14 @@ value rec mkrangepat loc c1 c2 =
          (mkrangepat loc (Char.chr (Char.code c1 + 1)) c2))
 ;
 
-value rec patt_label_long_id =
+value patt_label_long_id =
   fun
-  [ <:patt< $longid:li$ . $lid:s$ >> → Ldot (longid_long_id li) (conv_lab s)
-  | <:patt< $longid:li$ . $uid:s$ >> → longid_long_id li
-  | <:patt< $lid:s$ >> → Lident (conv_lab s)
+  [ <:patt:< $longid:li$ . $lid:s$ >> →
+      ocaml_longident_ldot (mkloc loc) (longid_to_longident li) (conv_lab s)
+  | <:patt:< $longid:li$ . $uid:s$ >> →
+      longid_to_longident li
+  | <:patt:< $lid:s$ >> →
+      ocaml_longident_lident (mkloc loc) (conv_lab s)
   | p → error (loc_of_patt p) "bad label" ]
 ;
 
@@ -342,7 +344,7 @@ value class_info item_attributes_fun class_expr ci =
             ~{item_attributes=item_attributes_fun ci.ciAttributes}
             (if uv ci.ciVir then Virtual else Concrete)
             (params, mkloc (fst ci.ciPrm)) (uv ci.ciNam) (class_expr ci.ciExp)
-            (mkloc ci.ciLoc) variance
+            (mkloc ci.ciLoc) (List.map uv variance)
       | None → error ci.ciLoc "no '_' type parameter allowed" ]
   | None → error ci.ciLoc "no class_info in this ocaml version" ]
 ;
@@ -369,14 +371,17 @@ value neg_string n =
 ;
 
 value varify_constructors var_names =
-  loop where rec loop =
+  let rec loop =
     fun
     [ <:ctyp:< $t1$ → $t2$ >> → <:ctyp< $loop t1$ → $loop t2$ >>
     | <:ctyp:< $t1$ $t2$ >> → <:ctyp:< $loop t1$ $loop t2$ >>
-    | <:ctyp:< ($list:tl$) >> → <:ctyp:< ($list:List.map loop tl$) >>
+    | <:ctyp:< ($list:tl$) >> → <:ctyp:< ($list:List.map labloop tl$) >>
     | <:ctyp:< $lid:s$ >> as t →
         if List.mem s var_names then <:ctyp< '$"&"^s$ >> else t
     | t → t ]
+  and labloop (lab, t) = (lab, loop t)
+  in
+  loop
 ;
 
 value label_of_patt =
@@ -391,15 +396,23 @@ value ctyp_mentions s cty =
     <:ctyp< '$s2$ >> ->  s = s2
   | <:ctyp< $t1$ $t2$ >> ->  crec t1 || crec t2
   | <:ctyp< $t1$ -> $t2$ >> ->  crec t1 || crec t2
-  | <:ctyp< ($list:tl$) >> -> List.exists crec tl
+  | <:ctyp< ($list:tl$) >> -> List.exists crec (List.map snd tl)
   | _ -> False
   ] in
   crec cty
 ;
 
+value add_empty_labels l = List.map (fun x -> (None, x)) l ;
+value strip_empty_labels loc l = do {
+  assert (List.for_all (fun (lab, _) -> None = uv lab) l) ;
+  List.map snd l
+}
+;
+
 value rec type_decl_of_with_type loc tn tpl pf ct =
   let (params, variance) = List.split (uv tpl) in
   let params = List.map uv params in
+  let variance = List.map uv variance in
   let ct = Some (ctyp ct) in
   let tk = if pf then ocaml_ptype_abstract else Ptype_abstract in
   let pf = if pf then Private else Public in
@@ -408,31 +421,31 @@ value rec type_decl_of_with_type loc tn tpl pf ct =
 and mkwithc =
   fun
   [ WcMod loc id m →
-      let mname = longid_long_id (uv id) in
-      (mname, ocaml_pwith_module (mkloc loc) mname (module_expr_long_id m))
+      let mname = longid_to_longident (uv id) in
+      (mname, ocaml_pwith_module (mkloc loc) mname (module_expr_to_longident m))
   | WcMos loc id m →
       match ocaml_pwith_modsubst with
       [ Some pwith_modsubst →
-          let li = longid_long_id (uv id) in
+          let li = longid_to_longident (uv id) in
           (li,
-           pwith_modsubst (mkloc loc) li (module_expr_long_id m))
+           pwith_modsubst (mkloc loc) li (module_expr_to_longident m))
       | None → error loc "no with module := in this ocaml version" ]
   | WcMty loc id mty ->
       match ocaml_pwith_modtype with [
         Some pwith_modtype ->
-      let mname = longid_long_id (uv id) in
+      let mname = longid_to_longident (uv id) in
       (mname, pwith_modtype (mkloc loc) mname (module_type mty))
       | None -> error loc "no with module := in this ocaml version"
         ]
   | WcMts loc id mty ->
       match ocaml_pwith_modtypesubst with [
         Some pwith_modtypesubst ->
-          let li = longid_long_id (uv id) in
+          let li = longid_to_longident (uv id) in
           (li, pwith_modtypesubst (mkloc loc) li (module_type mty))
       | None -> error loc "no with module type := in this ocaml version"
         ]
   | WcTyp loc id tpl pf ct →
-          let li = longid_lident_long_id (uv id) in
+          let li = lilongid_to_longident loc (uv id) in
           let (_, tname) = (uv id) in
           match type_decl_of_with_type loc (uv tname) tpl (uv pf) ct with
           | Right td -> (li, ocaml_pwith_type (mkloc loc) (li, td))
@@ -445,7 +458,7 @@ and mkwithc =
           let (_,last) = ids in
           match type_decl_of_with_type loc (uv last) tpl False t with
           [ Right td →
-              let li = longid_lident_long_id ids in
+              let li = lilongid_to_longident loc ids in
               (li, pwith_typesubst (mkloc loc) li td)
           | Left msg →
               error loc msg ]
@@ -497,7 +510,7 @@ and ctyp =
   [ TyAtt loc ct a ->
     ocaml_coretype_addattr (attr (uv a)) (ctyp ct)
   | TyAcc loc _ _ as f →
-      let (is_cls, li) = ctyp_long_id f in
+      let (is_cls, li) = ctyp_to_longident f in
       if is_cls then mktyp loc (ocaml_ptyp_class li [] [])
       else mktyp loc (ocaml_ptyp_constr (mkloc loc) li [])
 
@@ -512,7 +525,7 @@ and ctyp =
   | TyAny loc → mktyp loc Ptyp_any
   | TyApp loc _ _ as f →
       let (f, al) = ctyp_fa [] f in
-      let (is_cls, li) = ctyp_long_id f in
+      let (is_cls, li) = ctyp_to_longident f in
       if is_cls then mktyp loc (ocaml_ptyp_class li (List.map ctyp al) [])
       else mktyp loc (ocaml_ptyp_constr (mkloc loc) li (List.map ctyp al))
   | TyArr loc (TyLab loc1 lab t1) t2 →
@@ -524,9 +537,9 @@ and ctyp =
       mktyp loc
         (ocaml_ptyp_object (mkloc loc) (meth_list loc (uv fl) v) (uv v))
   | TyCls loc cli →
-      mktyp loc (ocaml_ptyp_class (extended_longid_lident_long_id (uv cli)) [] [])
+      mktyp loc (ocaml_ptyp_class (extended_lilongid_to_longident loc (uv cli)) [] [])
   | TyLab loc _ _ → error loc "labeled type not allowed here"
-  | TyLid loc s → mktyp loc (ocaml_ptyp_constr (mkloc loc) (Lident (uv s)) [])
+  | TyLid loc s → mktyp loc (ocaml_ptyp_constr (mkloc loc) (ocaml_longident_lident (mkloc loc) (uv s)) [])
   | TyMan loc _ _ _ → error loc "type manifest not allowed here"
   | TyOlb loc lab _ → error loc "labeled type not allowed here"
   | TyOpn loc ->  error loc "open (parsed as '..') type not allowed here"
@@ -547,7 +560,7 @@ and ctyp =
   | TyQuo loc s → mktyp loc (Ptyp_var (uv s))
   | TyRec loc _ → error loc "record type not allowed here"
   | TySum loc _ → error loc "sum type not allowed here"
-  | TyTup loc tl → mktyp loc (Ptyp_tuple (List.map ctyp (uv tl)))
+  | TyTup loc tl → mktyp loc (ocaml_ptyp_tuple (List.map (fun (lab, t) -> (lab |> uv |> option_map uv, ctyp t)) (uv tl)))
   | TyVrn loc catl ool →
       let catl =
         List.map
@@ -570,7 +583,7 @@ and ctyp =
   | TyXtr loc _ _ → error loc "bad ast TyXtr"
   | TyExten loc ebody -> mktyp loc (ocaml_ptyp_extension (extension (uv ebody)))
   | TyOpen loc li t ->
-     let li = longid_long_id li in
+     let li = longid_to_longident li in
      mktyp loc (ocaml_ptyp_open (mkloc loc) li (ctyp t))
   ]
 and meth_list loc fl v =
@@ -602,7 +615,7 @@ and package_of_module_type loc mt =
           List.map
             (fun
              [ WcTyp loc id tpl pf ct → do {
-                 let id_or_li = longid_lident_long_id (uv id)
+                 let id_or_li = lilongid_to_longident loc (uv id)
                  in
                  if uv tpl <> [] then
                    error loc "no type parameters allowed here"
@@ -621,8 +634,8 @@ and package_of_module_type loc mt =
         (mt, with_con)
     | _ → (mt, []) ]
   in
-  let li = module_type_long_id mt in
-  ocaml_package_type li with_con
+  let li = module_type_to_longident mt in
+  ocaml_package_type (mkloc loc) li with_con
 
 and type_decl ?{item_attributes=[]} tn tl priv (cl,tdCon) =
   fun
@@ -663,14 +676,14 @@ and extension_constructor ec = match ec with [
         }
       ]
     | EcRebind loc n li alg_attrs ->
-      ocaml_ec_rebind (mkloc loc) (uv n) (longid_long_id (uv li))
+      ocaml_ec_rebind (mkloc loc) (uv n) (longid_to_longident (uv li))
     ]
 and type_extension loc te =
   let pf = uv te.tePrv in
   let ecstrs = List.map extension_constructor (uv te.teECs) in
   ocaml_type_extension ~{item_attributes=uv_item_attributes te.teAttributes} (mkloc loc)
-    (longid_lident_long_id (uv te.teNam))
-    (List.map (fun (p,v) -> (uv p, v)) (uv te.tePrm))
+    (lilongid_to_longident loc (uv te.teNam))
+    (List.map (fun (p,v) -> (uv p, uv v)) (uv te.tePrm))
     (if pf then Private else Public)
     ecstrs
 and patt =
@@ -678,14 +691,14 @@ and patt =
   [ PaAtt loc p1 a ->
     ocaml_patt_addattr (attr (uv a)) (patt p1)
   | PaPfx loc li p2 ->
-    mkpat loc (ocaml_ppat_open (mkloc loc) (longid_long_id li) (patt p2))
+    mkpat loc (ocaml_ppat_open (mkloc loc) (longid_to_longident li) (patt p2))
   | PaLong loc li _ ->
     let li = match li with [
       <:longident:< $uid:s$ >> -> <:longident< $uid:conv_con s$ >>
     | <:longident:< $longid:li$ . $uid:s$ >> -> <:longident< $longid:li$ . $uid:conv_con s$ >>
     | _ -> failwith "Lapply not allowed here"
     ] in
-    let li = longid_long_id li in
+    let li = longid_to_longident li in
     mkpat loc (ocaml_ppat_construct (mkloc loc) li
                  None (not Prtools.no_constructors_arity.val))
 
@@ -723,7 +736,9 @@ and patt =
             let a =
               match al with
               [ [a] -> a
-              | _ -> mkpat loc (Ppat_tuple al) ]
+              | _ ->
+                 let al = add_empty_labels al in
+                 mkpat loc (ocaml_ppat_tuple al Closed) ]
             in
             mkpat loc (ocaml_ppat_construct li_loc li (Some ([],a)) False)
           else mkpat_ocaml_ppat_construct_arity (mkloc loc) li_loc li [] al
@@ -735,7 +750,10 @@ and patt =
                   let a =
                     match al with
                     [ [a] → a
-                    | _ → mkpat loc (Ppat_tuple al) ]
+                    | _ →
+                      let al = add_empty_labels al in
+                      mkpat loc (ocaml_ppat_tuple al Closed)
+                    ]
                   in
                   mkpat loc (ppat_variant (s, Some a))
               | Some _ | None →
@@ -757,7 +775,7 @@ and patt =
   | PaInt loc s c →
       mkpat loc (Ppat_constant (mkconst loc (pconst_of_const (mkintconst loc (uv s) c))))
   | PaFlo loc s → mkpat loc (Ppat_constant (mkconst loc (ocaml_pconst_float (uv s))))
-  | PaLab loc _ → error loc "labeled pattern not allowed here"
+  | PaLab loc _ _ → error loc "labeled pattern not allowed here"
   | PaLaz loc p →
       match ocaml_ppat_lazy with
       [ Some ppat_lazy → mkpat loc (ppat_lazy (patt p))
@@ -787,13 +805,28 @@ and patt =
       mkpat loc
         (Ppat_constant
            (mkconst loc (ocaml_pconst_string (string_of_string_token loc (uv s)) (mkloc loc) None)))
-  | PaTup loc pl → mkpat loc (Ppat_tuple (List.map patt (uv pl)))
+  | PaTup loc pl clflag →
+      let labeled_patt = fun [
+            PaLab ploc (PaLid _ s as p) po ->
+            let p =
+              match uv po with
+                [ Some p → p
+                | None → p ]
+            in
+            (Some (uv s), patt p)
+          | PaLab ploc (PaTyc _ (PaLid _ s) ty as p) <:vala< None >> ->
+             (Some (uv s), patt p)
+          | p -> (None, patt p)
+          ] in
+      let l = List.map labeled_patt (uv pl) in
+      let clflag = if uv clflag then Closed else Open in
+      mkpat loc (ocaml_ppat_tuple l clflag)
   | PaTyc loc p t → mkpat loc (Ppat_constraint (patt p) (ctyp t))
   | PaTyp loc lili →
       match ocaml_ppat_type with
       [ Some ppat_type →
           mkpat loc
-            (ppat_type (mkloc loc) (extended_longid_lident_long_id (uv lili)))
+            (ppat_type (mkloc loc) (extended_lilongid_to_longident loc (uv lili)))
       | None → error loc "no #type in this ocaml version" ]
   | PaUnp loc s mto →
       match ocaml_ppat_unpack with
@@ -828,7 +861,7 @@ and expr =
     | <:longident:< $longid:li$ . $uid:s$ >> -> <:longident< $longid:li$ . $uid:conv_con s$ >>
     | _ -> failwith "Lapply not allowed here"
     ] in
-    let li = longid_long_id li in
+    let li = longid_to_longident li in
     let cloc = mkloc loc in
     mkexp loc (ocaml_pexp_construct cloc li None ca)
 
@@ -837,7 +870,7 @@ and expr =
     expr (ExFle loc (ExLong loc li) <:vala< (None, vid) >>)
 
   | ExOpen loc li e ->
-    let li = longid_long_id li in
+    let li = longid_to_longident li in
     let me = mkmod loc (ocaml_pmod_ident li) in
     match ocaml_pexp_open with [
         Some pexp_open ->
@@ -848,11 +881,11 @@ and expr =
   | ExFle loc e <:vala< (None, s) >> when uv s = "val" ->
       mkexp loc
         (ocaml_pexp_apply
-           (mkexp loc (ocaml_pexp_ident (mkloc loc) (Lident "!")))
+           (mkexp loc (ocaml_pexp_ident (mkloc loc) (ocaml_longident_lident (mkloc loc) "!")))
            [("", expr e)])
 
   | ExFle loc (ExLong _ li) <:vala< (None, s) >> ->
-    let li = longid_lident_long_id (Some <:vala< li >>, s) in
+    let li = lilongid_to_longident loc (Some <:vala< li >>, s) in
     mkexp loc (ocaml_pexp_ident (mkloc loc) li)
 
   | ExFle loc (ExLong _ li) <:vala< (Some _, _) >> ->
@@ -862,7 +895,7 @@ and expr =
     error loc "().<lident>.val not valid syntax"
 
   | ExFle loc e lili ->
-    let li = longid_lident_long_id (uv lili) in
+    let li = lilongid_to_longident loc (uv lili) in
     mkexp loc (ocaml_pexp_field (mkloc loc) (expr e) li)
 
   | ExAnt _ e → expr e
@@ -883,7 +916,7 @@ and expr =
             | _ → f ]
         | _ → f ]
       in
-      let al = List.rev (List.fold_left label_expr [] al) in
+      let al = List.map label_expr al in
       match ocaml_pexp_construct_args (expr f).pexp_desc with
       [ Some (li, li_loc, None, _) →
           let al = List.map snd al in
@@ -891,7 +924,7 @@ and expr =
             let a =
               match al with
               [ [a] -> a
-              | _ -> mkexp loc (Pexp_tuple al) ]
+              | _ -> mkexp loc (ocaml_pexp_tuple (add_empty_labels al)) ]
             in
             mkexp loc (ocaml_pexp_construct li_loc li (Some a) False)
           else
@@ -906,7 +939,7 @@ and expr =
                   let a =
                     match al with
                     [ [a] → a
-                    | _ → mkexp loc (Pexp_tuple al) ]
+                    | _ → mkexp loc (ocaml_pexp_tuple (add_empty_labels al)) ]
                   in
                   mkexp loc (pexp_variant (s, Some a))
               | Some _ | None →
@@ -919,7 +952,7 @@ and expr =
           mkexp loc
             (ocaml_pexp_apply
                (mkexp loc
-                  (ocaml_pexp_ident cloc (array_function "Array" "get")))
+                  (ocaml_pexp_ident cloc (array_function loc "Array" "get")))
                [("", expr e1); ("", expr e2)])
       | (".", e2l) -> assert False
       | (dotop, [e2]) -> 
@@ -937,7 +970,7 @@ and expr =
           let cloc = mkloc loc in
           mkexp loc
             (ocaml_pexp_apply
-               (mkexp loc (ocaml_pexp_ident cloc (Lident ":=")))
+               (mkexp loc (ocaml_pexp_ident cloc (ocaml_longident_lident (mkloc loc) ":=")))
                [("", expr x); ("", expr v)])
 
       | ExFle loc _ _ ->
@@ -952,7 +985,7 @@ and expr =
               mkexp loc
                 (ocaml_pexp_apply
                    (mkexp loc
-                      (ocaml_pexp_ident cloc (array_function "Array" "set")))
+                      (ocaml_pexp_ident cloc (array_function loc "Array" "set")))
                    [("", expr e1); ("", expr e2); ("", expr v)])
           | (".", e2l) -> assert False
           | (dotop, [e2]) ->
@@ -983,7 +1016,7 @@ and expr =
                 (ocaml_pexp_apply
                    (mkexp loc
                       (ocaml_pexp_ident cloc
-                         (array_function bytes_modname "set")))
+                         (array_function loc bytes_modname "set")))
                    [("", expr e1); ("", expr e2); ("", expr v)])
           | (".", e2l) -> assert False
           | (dotop, [e2]) ->
@@ -1025,17 +1058,15 @@ and expr =
       mkexp loc (ocaml_pexp_for (patt i) (expr e1) (expr e2) df (expr e3))
   | ExFun loc pel →
       match uv pel with
-      [ [(PaLab ploc lppo, w, e)] →
-          match uv lppo with
-          [ [(p, po)] →
-              let lab = label_of_patt p in
-              let p =
-                match uv po with
-                [ Some p → p
-                | None → p ]
-              in
-              mkexp loc (ocaml_pexp_function lab None [mkpwe (p, w, e)])
-          | _ → error loc "bad AST" ]
+      [ [(PaLab ploc p po, w, e)] →
+        let lab = label_of_patt p in
+        let p =
+          match uv po with
+            [ Some p → p
+            | None → p ]
+        in
+        mkexp loc (ocaml_pexp_function lab None [mkpwe (p, w, e)])
+
       | [(PaNty loc s, w, e)] →
           mkexp loc (ocaml_pexp_newtype (mkloc loc) (uv s) (expr e))
       | [(PaOlb loc p eo, w, e)] →
@@ -1064,14 +1095,14 @@ and expr =
       mkexp loc (Pexp_ifthenelse (expr e1) (expr e2) e3o)
   | ExInt loc s c →
       mkexp loc (Pexp_constant (mkconst loc (pconst_of_const (mkintconst loc (uv s) c))))
-  | ExLab loc _ → error loc "labeled expression not allowed here 1"
+  | ExLab loc _ _ → error loc "labeled expression not allowed here 1"
   | ExLaz loc e → mklazy loc (expr e)
   | ExLet loc rf pel e →
       mkexp loc (Pexp_let (mkrf (uv rf)) (List.map mkpe (uv pel)) (expr e))
   | ExLEx loc exnname exntyl body alg_attrs ->
       let exdef = ocaml_extension_exception (mkloc loc) (uv exnname) (List.map ctyp (uv exntyl)) (uv_alg_attributes alg_attrs) in
       mkexp loc (ocaml_pexp_letexception exdef (expr body))
-  | ExLid loc s → mkexp loc (ocaml_pexp_ident (mkloc loc) (Lident (uv s)))
+  | ExLid loc s → mkexp loc (ocaml_pexp_ident (mkloc loc) (ocaml_longident_lident (mkloc loc) (uv s)))
   | ExLmd loc i me e →
       match ocaml_pexp_letmodule with
       [ Some pexp_letmodule →
@@ -1094,7 +1125,7 @@ and expr =
       mkexp loc (Pexp_match (expr e) (List.map mkpwe pel))
   | ExNew loc cli →
       mkexp loc
-        (ocaml_pexp_new (mkloc loc) (longid_lident_long_id (uv cli)))
+        (ocaml_pexp_new (mkloc loc) (lilongid_to_longident loc (uv cli)))
   | ExObj loc po cfl →
       match ocaml_pexp_object with
       [ Some pexp_object →
@@ -1110,25 +1141,10 @@ and expr =
   | ExOvr loc iel →
       mkexp loc (ocaml_pexp_override (List.map mkideexp (uv iel)))
   | ExPck loc me mto →
-      match ocaml_pexp_pack with
-      [ Some (Left pexp_pack) →
-          match mto with
-          [ Some mt →
-              let pt = package_of_module_type loc mt in
-              mkexp loc (pexp_pack (module_expr me) pt)
-          | None → error loc "no such module pack in this ocaml version" ]
-      | Some (Right (pexp_pack, ptyp_package)) →
-          let e = pexp_pack (module_expr me) in
-          let e =
-            match mto with
-            [ Some mt →
-                let pt = package_of_module_type loc mt in
-                ocaml_pexp_constraint (mkexp loc e)
-                  (Some (mktyp loc (ptyp_package pt))) None
-            | None → e ]
-          in
-          mkexp loc e
-      | None → error loc "no module pack in this ocaml version" ]
+      let pto : option Parsetree.package_type = option_map (package_of_module_type loc) mto in
+      let e = ocaml_pexp_pack (mkloc loc) (module_expr me) pto in
+      mkexp loc e
+
   | ExRec loc lel eo →
       let lel = uv lel in
       if lel = [] then error loc "empty record"
@@ -1154,7 +1170,7 @@ and expr =
           let cloc = mkloc loc in
           mkexp loc
             (ocaml_pexp_apply
-               (mkexp loc (ocaml_pexp_ident cloc (array_function "String" "get")))
+               (mkexp loc (ocaml_pexp_ident cloc (array_function loc "String" "get")))
                [("", expr e1); ("", expr e2)])
       | (".", e2l) -> assert False
       | (dotop, [e2]) ->
@@ -1171,7 +1187,14 @@ and expr =
         (Pexp_constant
            (mkconst loc (ocaml_pconst_string (string_of_string_token loc (uv s)) (mkloc loc) None)))
   | ExTry loc e pel → mkexp loc (Pexp_try (expr e) (List.map mkpwe (uv pel)))
-  | ExTup loc el → mkexp loc (Pexp_tuple (List.map expr (uv el)))
+  | ExTup loc el →
+      let el = List.map label_expr (uv el) in
+      let el = List.map (fun [
+                             ("", e) -> (None, e)
+                           | (s, e) when s.[0] = '?' -> error loc "cannot use optional labels in tuple"
+                           | (s,e) -> (Some s, e)
+                           ]) el in
+      mkexp loc (ocaml_pexp_tuple  el)
   | ExTyc loc e t →
       mkexp loc (ocaml_pexp_constraint (expr e) (Some (ctyp t)) None)
   | ExVrn loc s →
@@ -1185,21 +1208,16 @@ and expr =
   | ExExten loc ebody -> mkexp loc (ocaml_pexp_extension (extension (uv ebody)))
   | ExUnr loc -> error loc "bad ast ExUnr (parses as '.'; cannot have an ExUnr except at the rhs of match-case)"
   ]
-and label_expr rev_al =
+and label_expr =
   fun
-  [ ExLab loc lpeo →
-      List.fold_left
-        (fun rev_al (p, eo) →
-           match p with
-           [ PaLid loc lab →
-               let e =
-                 match uv eo with
-                 [ Some e → e
-                 | None → ExLid loc lab ]
-               in
-               [(uv lab, expr e) :: rev_al]
-           | _ → error loc "ExLab case not impl" ])
-        rev_al (uv lpeo)
+  [ ExLab loc p eo →
+      match (p, uv eo) with [
+          (PaLid loc lab, Some e) -> (uv lab, expr e)
+        | (PaLid loc lab, None) -> (uv lab, expr (ExLid loc lab))
+        | (PaTyc loc (PaLid _ lab) ty, None) -> (uv lab, expr (ExTyc loc (ExLid loc lab) ty))
+        | _ → error loc "ExLab case not impl"
+        ]
+
   | ExOlb loc p eo →
       match p with
       [ PaLid loc lab →
@@ -1208,9 +1226,9 @@ and label_expr rev_al =
             [ Some e → e
             | None → ExLid loc lab ]
           in
-          [("?" ^ uv lab, expr e) :: rev_al]
+          ("?" ^ uv lab, expr e)
       | _ → error loc "ExOlb case not impl" ]
-  | e → [("", expr e) :: rev_al] ]
+  | e → ("", expr e) ]
 and mkpe (p, e, attrs) =
   let loc = Ploc.encl (loc_of_patt p) (loc_of_expr e) in
 (*
@@ -1265,10 +1283,10 @@ and module_type =
   [ MtAtt loc e a ->
       ocaml_pmty_addattr (attr (uv a)) (module_type e)
   | MtLongLid loc _ _ as f →
-      mkmty loc (ocaml_pmty_ident (mkloc loc) (module_type_long_id f))
+      mkmty loc (ocaml_pmty_ident (mkloc loc) (module_type_to_longident f))
   | MtLong loc _ as f →
-      mkmty loc (ocaml_pmty_ident (mkloc loc) (module_type_long_id f))
-  | MtLid loc s → mkmty loc (ocaml_pmty_ident (mkloc loc) (Lident (uv s)))
+      mkmty loc (ocaml_pmty_ident (mkloc loc) (module_type_to_longident f))
+  | MtLid loc s → mkmty loc (ocaml_pmty_ident (mkloc loc) (ocaml_longident_lident (mkloc loc) (uv s)))
   | MtFun loc arg mt →
     let arg = option_map (fun (idopt, mt) -> (option_map uv (uv idopt), module_type mt)) (uv arg) in
       mkmty loc
@@ -1343,19 +1361,19 @@ and sig_item s l =
       [mksig loc (ocaml_psig_modtypesubst ~{item_attributes=uv_item_attributes item_attrs} (mkloc loc) (uv n) mto) :: l]
 
   | SgMtyAlias loc n li item_attrs ->
-      let li = longid_long_id (uv li) in
+      let li = longid_to_longident (uv li) in
       let mty = mkmty loc (ocaml_pmty_alias (mkloc loc) li) in
       let m = ocaml_psig_module ~{item_attributes=uv_item_attributes item_attrs} (mkloc loc) (Some (uv n)) mty in
       [mksig loc m :: l]
 
   | SgModSubst loc s li item_attrs ->
-      let li = longid_long_id li in
+      let li = longid_to_longident li in
       let attrs = uv_item_attributes item_attrs in
       let m = ocaml_psig_modsubst ~{item_attributes=attrs} (mkloc loc) (uv s) li in
       [mksig loc m :: l]
 
   | SgOpn loc lid attrs →
-      let lid = extended_longid_long_id lid in
+      let lid = extended_longid_to_longident lid in
       [mksig loc
          (ocaml_psig_open ~{item_attributes=uv_item_attributes attrs} (mkloc loc) lid) ::
        l]
@@ -1390,7 +1408,7 @@ and module_expr =
   fun
   [ MeAtt loc e a ->
       ocaml_pmod_addattr (attr (uv a)) (module_expr e)
-  | MeAcc loc _ _ as f → mkmod loc (ocaml_pmod_ident (module_expr_long_id f))
+  | MeAcc loc _ _ as f → mkmod loc (ocaml_pmod_ident (module_expr_to_longident f))
   | MeApp loc me1 me2 →
       mkmod loc (Pmod_apply (module_expr me1) (module_expr me2))
   | MeFun loc arg me →
@@ -1400,7 +1418,7 @@ and module_expr =
       mkmod loc (Pmod_structure (List.fold_right str_item (uv sl) []))
   | MeTyc loc me mt →
       mkmod loc (Pmod_constraint (module_expr me) (module_type mt))
-  | MeUid loc s → mkmod loc (ocaml_pmod_ident (Lident (uv s)))
+  | MeUid loc s → mkmod loc (ocaml_pmod_ident (ocaml_longident_lident (mkloc loc) (uv s)))
   | MeUnp loc e mto1 mto2 →
       match ocaml_pmod_unpack with
       [ Some (Left pmod_unpack) →
@@ -1457,7 +1475,7 @@ and str_item s l =
             match ocaml_pstr_exn_rebind with
             [ Some pstr_exn_rebind →
                 pstr_exn_rebind (mkloc loc) (uv n)
-                  (longid_long_id (uv li))
+                  (longid_to_longident (uv li))
             | None →
                 error loc "no exception renaming in this ocaml version" ]
       in
@@ -1537,7 +1555,7 @@ and class_type =
       | None → error loc "no class type desc in this ocaml version" ]
 
   | CtLop loc ovf me ct →
-      let li = longid_long_id me in
+      let li = longid_to_longident me in
       mkcty loc
         (ocaml_pcty_open (mkloc loc) li (mkoverride (uv ovf)) (class_type ct))
   | CtCon loc ct tl →
@@ -1622,47 +1640,37 @@ and class_expr =
       match ocaml_pcl_apply with
       [ Some pcl_apply →
           let (ce, el) = class_expr_fa [] c in
-          let el = List.rev (List.fold_left label_expr [] el) in
+          let el = List.map label_expr el in
           mkpcl loc (pcl_apply (class_expr ce) el)
       | None → error loc "no class expr desc in this ocaml version" ]
   | CeCon loc cli tl →
       match ocaml_pcl_constr with
       [ Some pcl_constr →
           mkpcl loc
-            (pcl_constr (longid_lident_long_id (uv cli))
+            (pcl_constr (lilongid_to_longident loc (uv cli))
                (List.map ctyp (uv tl)))
       | None → error loc "no class expr desc in this ocaml version" ]
-  | CeFun loc (PaLab ploc lppo) ce →
-      match ocaml_pcl_fun with
-      [ Some pcl_fun →
-          match uv lppo with
-          [ [(p, po)] →
-              let lab = label_of_patt p in
-              let p =
-                match uv po with
-                [ Some p → p
-                | None → p ]
-              in
-              mkpcl loc (pcl_fun lab None (patt p) (class_expr ce))
-          | [] | [_ :: _] → error ploc "case class multi lab not yet impl" ]
-      | None → error loc "no class expr desc in this ocaml version" ]
+  | CeFun loc (PaLab ploc p po) ce →
+    let lab = label_of_patt p in
+    let p =
+      match uv po with
+        [ Some p → p
+        | None → p ]
+    in
+    mkpcl loc (ocaml_pcl_fun lab None (patt p) (class_expr ce))
+
   | CeFun loc (PaOlb _ p eo) ce →
-      match ocaml_pcl_fun with
-      [ Some pcl_fun →
-          let lab = label_of_patt p in
-          let (p, eo) =
-            match uv eo with
-            [ Some (ExOlb _ p eo) → (p, eo)
-            | Some _ | None → (p, eo) ]
-          in
-          mkpcl loc
-            (pcl_fun ("?" ^ lab) (option_map expr (uv eo)) (patt p)
-               (class_expr ce))
-      | None → error loc "no class expr desc in this ocaml version" ]
+    let lab = label_of_patt p in
+    let (p, eo) =
+      match uv eo with
+        [ Some (ExOlb _ p eo) → (p, eo)
+        | Some _ | None → (p, eo) ]
+    in
+    mkpcl loc
+      (ocaml_pcl_fun ("?" ^ lab) (option_map expr (uv eo)) (patt p)
+         (class_expr ce))
   | CeFun loc p ce →
-      match ocaml_pcl_fun with
-      [ Some pcl_fun → mkpcl loc (pcl_fun "" None (patt p) (class_expr ce))
-      | None → error loc "no class expr desc in this ocaml version" ]
+    mkpcl loc (ocaml_pcl_fun "" None (patt p) (class_expr ce))
   | CeLet loc rf pel ce →
       match ocaml_pcl_let with
       [ Some pcl_let →
@@ -1670,7 +1678,7 @@ and class_expr =
             (pcl_let (mkrf (uv rf)) (List.map mkpe (uv pel)) (class_expr ce))
       | None → error loc "no class expr desc in this ocaml version" ]
   | CeLop loc ovf me ce →
-      let li = longid_long_id me in
+      let li = longid_to_longident me in
       mkpcl loc
         (ocaml_pcl_open (mkloc loc) li (mkoverride (uv ovf)) (class_expr ce))
   | CeStr loc po cfl →
@@ -1815,9 +1823,9 @@ value directive loc =
       match ocaml_pdir_bool with
       [ Some pdir_bool → pdir_bool False
       | None → error loc "no such kind of directive in this ocaml version" ]
-  | <:expr< $longid:li$ >> -> Pdir_ident (longid_long_id li)
-  | <:expr< $longid:li$ . $lid:id$ >> -> Pdir_ident (longid_lident_long_id (Some <:vala< li >>,<:vala< id >>))
-  | <:expr<  $lid:id$ >> -> Pdir_ident (Lident id)
+  | <:expr< $longid:li$ >> -> Pdir_ident (longid_to_longident li)
+  | <:expr< $longid:li$ . $lid:id$ >> -> Pdir_ident (lilongid_to_longident loc (Some <:vala< li >>,<:vala< id >>))
+  | <:expr<  $lid:id$ >> -> Pdir_ident (ocaml_longident_lident (mkloc loc) id)
   | e →
       let sl =
         loop e where rec loop =
@@ -1826,7 +1834,7 @@ value directive loc =
           | <:expr< $uid:i$ >> → [i]
           | e → Ploc.raise (loc_of_expr e) (Failure "bad ast") ]
       in
-      Pdir_ident (long_id_of_string_list loc sl) ]
+      Pdir_ident (longident_of_string_list loc sl) ]
 ;
 
 value directive_args loc d =
