@@ -1,27 +1,32 @@
 (* camlp5r *)
 (* exparser.ml,v *)
-(* Copyright (c) INRIA 2007-2014 *)
+(* Copyright (c) INRIA 2007-2017 *)
 
 #load "q_MLast.cmo";
 
-type spat_comp =
+type spat_comp = Exparser_types.spat_comp ==
   [ SpTrm of MLast.loc and MLast.patt and MLast.v (option MLast.expr)
   | SpNtr of MLast.loc and MLast.patt and MLast.expr
   | SpLet of MLast.loc and MLast.patt and MLast.expr
-  | SpLhd of MLast.loc and list (list MLast.patt)
+  | SpLhd of MLast.loc and list (list MLast.patt) and MLast.v (option MLast.expr)
   | SpStr of MLast.loc and MLast.patt ]
 ;
-type sexp_comp =
+type sexp_comp = Exparser_types.sexp_comp ==
   [ SeTrm of MLast.loc and MLast.expr
   | SeNtr of MLast.loc and MLast.expr ]
 ;
 
-type spat_comp_opt =
+type spat_comp_opt = Exparser_types.spat_comp_opt ==
   [ SpoNoth
   | SpoBang
   | SpoQues of MLast.expr ]
 ;
 
+type spat_parser_ast =
+  (option MLast.patt *
+   list (list (spat_comp * spat_comp_opt) * option MLast.patt * MLast.expr))
+;
+
 value strm_n = "strm__";
 value peek_fun loc = <:expr< Stream.peek >>;
 value junk_fun loc = <:expr< Stream.junk >>;
@@ -60,7 +65,7 @@ value rec handle_failure e =
          | _ -> False ])
         pel
   | <:expr< let $list:pel$ in $e$ >> ->
-      List.for_all (fun (p, e) -> handle_failure e) pel && handle_failure e
+      List.for_all (fun (p, e, _) -> handle_failure e) pel && handle_failure e
   | <:expr< do { $list:el$ } >> -> List.for_all handle_failure el
   | <:expr< $uid:_$ . $_$ >> | <:expr< $lid:_$ >> | <:expr< $int:_$ >> |
     <:expr< $str:_$ >> | <:expr< $chr:_$ >> | <:expr< fun [ $list:_$ ] >> |
@@ -97,9 +102,9 @@ value rec subst v e =
   | <:expr:< $e1$ $e2$ >> -> <:expr< $subst v e1$ $subst v e2$ >>
   | <:expr:< ( $list:el$ ) >> -> <:expr< ( $list:List.map (subst v) el$ ) >>
   | _ -> raise Not_found ]
-and subst_pe v (p, e) =
+and subst_pe v (p, e, attrs) =
   match p with
-  [ <:patt< $lid:v'$ >> when v <> v' -> (p, subst v e)
+  [ <:patt< $lid:v'$ >> when v <> v' -> (p, subst v e, attrs)
   | _ -> raise Not_found ]
 ;
 
@@ -159,7 +164,7 @@ value stream_pattern_component skont ckont =
                 [ Some $p$ -> $skont$
                 | _ -> $ckont$ ] >>
   | SpLet _ _ _ -> assert False
-  | SpLhd loc [pl :: pll] ->
+  | SpLhd loc [pl :: pll] wo ->
       let mklistpat loc pl =
         List.fold_right (fun p1 p2 -> <:patt< [$p1$ :: $p2$] >>) pl
           <:patt< [] >>
@@ -175,13 +180,17 @@ value stream_pattern_component skont ckont =
           List.fold_left (fun p1 p2 -> <:patt< $p1$ | $p2$ >>) p pl
         in
         <:expr< match Stream.npeek $int:string_of_int len$ strm__ with
-                [ $p$ -> $skont$
+                [ $p$ $_opt:wo$ -> $skont$
                 | _ -> $ckont$ ] >>
-  | SpLhd loc [] -> ckont
+  | SpLhd loc [] wo -> ckont
   | SpStr loc p ->
       try
         match p with
-        [ <:patt< $lid:v$ >> -> subst v skont
+        [ <:patt< $lid:v$ >> ->
+          <:expr< let $lid:v$ = strm__ in $skont$ >>
+(*
+ subst v skont
+*)
         | _ -> raise Not_found ]
       with
       [ Not_found -> <:expr< let $p$ = $lid:strm_n$ in $skont$ >> ] ]
@@ -350,7 +359,7 @@ value left_factorize rl =
 
 (* Converting into AST *)
 
-value cparser loc bpo pc =
+value cparser loc (bpo, pc) =
   let pc = left_factorize pc in
   let e = parser_cases loc pc in
   let e =
@@ -373,7 +382,7 @@ value rec is_not_bound s =
   | _ -> False ]
 ;
 
-value cparser_match loc me bpo pc =
+value cparser_match loc me (bpo, pc) =
   let pc = left_factorize pc in
   let iloc = Ploc.with_comment loc "" in
   let pc = parser_cases iloc pc in
