@@ -21,7 +21,7 @@ END;
 
 type v 'a = V 'a;
 
-type type_var = (V (option string) * (option bool * bool));
+type type_var = (V (option string) * V string);
 
 type class_infos 'a =
   { ciLoc : loc;
@@ -57,7 +57,7 @@ and ctyp =
   | TyQuo of loc and V string
   | TyRec of loc and V (list (loc * string * bool * ctyp * attributes))
   | TySum of loc and V (list generic_constructor)
-  | TyTup of loc and V (list ctyp)
+  | TyTup of loc and V (list (V (option (V string)) * ctyp))
   | TyVrn of loc and V (list poly_variant) and
       option (option (V (list string)))
   | TyXtr of loc and string and option (V ctyp)
@@ -81,7 +81,7 @@ and patt =
   | PaEff of loc and patt and patt
   | PaFlo of loc and V string
   | PaInt of loc and V string and string
-  | PaLab of loc and V (list (patt * V (option patt)))
+  | PaLab of loc and patt and V (option patt)
   | PaLaz of loc and patt
   | PaLid of loc and V string
   | PaNty of loc and V string
@@ -90,7 +90,7 @@ and patt =
   | PaRec of loc and V (list (patt * patt))
   | PaRng of loc and patt and patt
   | PaStr of loc and V string
-  | PaTup of loc and V (list patt)
+  | PaTup of loc and V (list patt) and V bool
   | PaTyc of loc and patt and ctyp
   | PaTyp of loc and V longid_lident
   | PaUnp of loc and V (option (V string)) and option module_type
@@ -116,7 +116,7 @@ and expr =
   | ExFun of loc and V (list case_branch)
   | ExIfe of loc and expr and expr and expr
   | ExInt of loc and V string and string
-  | ExLab of loc and V (list (patt * V (option expr)))
+  | ExLab of loc and patt and V (option expr)
   | ExLaz of loc and expr
   | ExLet of loc and V bool and V (list (patt * expr * attributes)) and expr
   | ExLEx of loc and V string and V (list ctyp) and expr and attributes
