@@ -1,6 +1,6 @@
 (* camlp5r *)
 (* prtools.ml,v *)
-(* Copyright (c) INRIA 2007-2014 *)
+(* Copyright (c) INRIA 2007-2017 *)
 
 #load "q_MLast.cmo";
 #load "pa_macro.cmo";
@@ -15,14 +15,6 @@ type pr_context =
 
 type pr_fun 'a = pr_context -> 'a -> string;
 
-IFDEF OCAML_VERSION <= OCAML_1_07 OR COMPATIBLE_WITH_OLD_OCAML THEN
-  value with_ind_bef = Pprintf.with_ind_bef;
-  value with_bef = Pprintf.with_bef;
-  value with_bef_aft_dang = Pprintf.with_bef_aft_dang;
-  value with_aft = Pprintf.with_aft;
-  value with_aft_dang = Pprintf.with_aft_dang;
-END;
-
 value tab ind = String.make ind ' ';
 
 (* horizontal list *)
@@ -355,8 +347,9 @@ module Buff =
   struct
     value buff = ref (string_create 80);
     value store len x = do {
-      if len >= String.length buff.val then
-        buff.val := buff.val ^ string_create (String.length buff.val)
+      if len >= string_length buff.val then
+        buff.val :=
+          string_cat buff.val (string_create (string_length buff.val))
       else ();
       string_set buff.val len x;
       succ len
@@ -366,7 +359,7 @@ module Buff =
         if i == String.length s then len
         else add_rec (store len s.[i]) (succ i)
     ;
-    value get len = String.sub buff.val 0 len;
+    value get len = string_sub buff.val 0 len;
   end
 ;
 
@@ -392,7 +385,7 @@ value adjust_comment_indentation ind s nl_bef ind_bef =
     let ind_bef = if nl_bef > 0 then ind_bef else ind in
     let len = i_bef_ind + 1 in
     let olen = Buff.mstore 0 (String.make ind ' ') in
-    loop olen 0 where rec loop olen i =
+    bytes_to_string (loop olen 0) where rec loop olen i =
       if i = len then Buff.get olen
       else
         let olen = Buff.store olen s.[i] in
@@ -417,7 +410,7 @@ value adjust_comment_indentation ind s nl_bef ind_bef =
 
 value eight_chars = String.make 8 ' ';
 value expand_tabs s =
-  loop 0 0 where rec loop len i =
+  bytes_to_string (loop 0 0) where rec loop len i =
     if i = String.length s then Buff.get len
     else if s.[i] = '\t' then loop (Buff.mstore len eight_chars) (i + 1)
     else loop (Buff.store len s.[i]) (i + 1)
@@ -437,10 +430,10 @@ value expand_module_prefix m =
     [ [(p, e) :: rest] -> do {
         let p =
           match p with
-          [ <:patt< $uid:_$.$_$ >> -> p
+          [ <:patt< $uid:_$ . $_$ >> -> p
           | _ ->
               let loc = MLast.loc_of_patt p in
-              <:patt< $uid:m$.$p$ >> ]
+              <:patt< $uid:m$ . $p$ >> ]
         in
         loop [(p, e) :: rev_lel] rest
       }
@@ -485,30 +478,3 @@ value do_split_or_patterns_with_bindings pel =
         loop rev_pel pel
     | [] -> List.rev rev_pel ]
 ;
-
-value record_without_with loc e lel =
-  try
-    let (m, name) =
-      let (m, sl) =
-        List.fold_right
-          (fun (p, _) (m, sl) ->
-             match p with
-              [ <:patt< $lid:lab$ >> -> (m, [lab :: sl])
-              | <:patt< $uid:m$.$lid:lab$ >> -> (m, [lab :: sl])
-              | _ -> raise Exit ])
-          lel ("", [])
-      in
-      (m, String.concat "_" ["with" :: sl])
-    in
-    let f =
-      let f = <:expr< $lid:name$ >> in
-      if m = "" then f else <:expr< $uid:m$.$f$ >>
-    in
-    let e =
-      List.fold_left (fun e1 (_, e2) -> <:expr< $e1$ $e2$ >>)
-        <:expr< $f$ $e$ >> lel
-    in
-    Some e
-  with
-  [ Exit -> None ]
-;
