@@ -125,7 +125,7 @@ value rec ctyp floc sh =
              x1)
     | TyTup loc x1 →
         let loc = floc loc in
-        TyTup loc (vala_map (List.map self) x1)
+        TyTup loc (vala_map (List.map (fun (a, b) -> (v_option_v_string a, self b))) x1)
       | TyVrn loc x1 x2 →
         let loc = floc loc in
         TyVrn loc (vala_map (List.map (poly_variant floc sh)) x1) x2
@@ -139,6 +139,7 @@ value rec ctyp floc sh =
        let loc = floc loc in
        TyOpen loc (longid floc sh li) (ctyp floc sh t)
     ]
+and v_option_v_string x = vala_map (option_map (vala_map (fun x -> x))) x
 and generic_constructor floc sh = fun (loc, x1, x2, x3, x4, x5) ->
     (floc loc, x1, x2, vala_map (List.map (ctyp floc sh)) x3,
      vala_map (option_map (ctyp floc sh)) x4, attributes floc sh x5)
@@ -192,13 +193,9 @@ and patt floc sh =
     | PaInt loc x1 x2 →
         let loc = floc loc in
         PaInt loc x1 x2
-    | PaLab loc x1 →
+    | PaLab loc x1 x2 →
         let loc = floc loc in
-        PaLab loc
-          (vala_map
-             (List.map
-                (fun (x1, x2) → (self x1, vala_map (option_map self) x2)))
-             x1)
+        PaLab loc (self x1) (vala_map (option_map self) x2)
     | PaLaz loc x1 →
         let loc = floc loc in
         PaLaz loc (self x1)
@@ -223,9 +220,9 @@ and patt floc sh =
     | PaStr loc x1 →
         let loc = floc loc in
         PaStr loc x1
-    | PaTup loc x1 →
+    | PaTup loc x1 x2 →
         let loc = floc loc in
-        PaTup loc (vala_map (List.map self) x1)
+        PaTup loc (vala_map (List.map self) x1) x2
     | PaTyc loc x1 x2 →
         let loc = floc loc in
         PaTyc loc (self x1) (ctyp floc sh x2)
@@ -311,14 +308,9 @@ and expr floc sh =
     | ExInt loc x1 x2 →
         let loc = floc loc in
         ExInt loc x1 x2
-    | ExLab loc x1 →
+    | ExLab loc x1 x2 →
         let loc = floc loc in
-        ExLab loc
-          (vala_map
-             (List.map
-                (fun (x1, x2) →
-                   (patt floc sh x1, vala_map (option_map self) x2)))
-             x1)
+        ExLab loc (patt floc sh x1) (vala_map (option_map self) x2)
     | ExLaz loc x1 →
         let loc = floc loc in
         ExLaz loc (self x1)
