@@ -254,8 +254,8 @@ module MetaAction =
           <:expr< MLast.ExIfe $mloc$ $mexpr e1$ $mexpr e2$ $mexpr e3$ >>
       | MLast.ExInt loc s c ->
           <:expr< MLast.ExInt $mloc$ $mvala mstring s$ $str:c$ >>
-      | MLast.ExLab loc peoptl ->
-          <:expr< MLast.ExLab $mloc$ $mvala (mlist mpeopt) peoptl$ >>
+      | MLast.ExLab loc p eo ->
+          <:expr< MLast.ExLab $mloc$ $mpatt p$ $mvala (moption mexpr) eo$ >>
       | MLast.ExFlo loc s -> <:expr< MLast.ExFlo $mloc$ $mvala mstring s$ >>
       | MLast.ExLet loc rf pel e ->
           let rf = mvala mbool rf in
@@ -287,6 +287,8 @@ module MetaAction =
       | MLast.PaLong loc li <:vala< [] >> ->
           <:expr< MLast.PaLong $mloc$ $mlongid li$ (Ploc.VaVal []) >>
       | MLast.PaLong loc li _ -> assert False
+      | MLast.PaAli loc p1 p2 ->
+          <:expr< MLast.PaAli $mloc$ $mpatt p1$ $mpatt p2$ >>
       | MLast.PaAny loc -> <:expr< MLast.PaAny $mloc$ >>
       | MLast.PaApp loc p1 p2 ->
           <:expr< MLast.PaApp $mloc$ $mpatt p1$ $mpatt p2$ >>
@@ -297,8 +299,8 @@ module MetaAction =
           <:expr< MLast.PaOrp $mloc$ $mpatt p1$ $mpatt p2$ >>
       | MLast.PaStr loc s ->
           <:expr< MLast.PaStr $mloc$ $mvala mstring_escaped s$ >>
-      | MLast.PaTup loc pl ->
-          <:expr< MLast.PaTup $mloc$ $mvala (mlist mpatt) pl$ >>
+      | MLast.PaTup loc pl clflag ->
+          <:expr< MLast.PaTup $mloc$ $mvala (mlist mpatt) pl$ $mvala mbool clflag$ >>
       | MLast.PaTyc loc p t ->
           <:expr< MLast.PaTyc $mloc$ $mpatt p$ $mctyp t$ >>
       | x -> not_impl "mpatt" x ]
@@ -322,8 +324,10 @@ module MetaAction =
       | MLast.TyLid loc s -> <:expr< MLast.TyLid $mloc$ $mvala mstring s$ >>
       | MLast.TyQuo loc s -> <:expr< MLast.TyQuo $mloc$ $mvala mstring s$ >>
       | MLast.TyTup loc tl ->
-          <:expr< MLast.TyTup $mloc$ $mvala (mlist mctyp) tl$ >>
+          <:expr< MLast.TyTup $mloc$ $mvala (mlist mlabctyp) tl$ >>
       | x -> not_impl "mctyp" x ]
+    and mlabctyp (lab, t) =
+      <:expr< ($mvala (moption (mvala mstring)) lab$, $mctyp t$) >>
     and mpea (p, e, attrs) =
       do { assert ([] = Pcaml.unvala attrs) ;
       <:expr< ($mpatt p$, $mexpr e$, Ploc.VaVal []) >>
