@@ -1,6 +1,6 @@
 (* camlp5r *)
 (* pa_lexer.ml,v *)
-(* Copyright (c) INRIA 2007-2014 *)
+(* Copyright (c) INRIA 2007-2017 *)
 
 #load "pa_extend.cmo";
 #load "q_MLast.cmo";
@@ -45,11 +45,11 @@ value conv_rules loc rl =
 ;
 
 value mk_lexer loc rl =
-  cparser loc None (conv_rules loc rl)
+  cparser loc (None, (conv_rules loc rl))
 ;
 
 value mk_lexer_match loc e rl =
-  cparser_match loc e None (conv_rules loc rl)
+  cparser_match loc e (None, (conv_rules loc rl))
 ;
 
 (* group together consecutive rules just containing one character *)
@@ -110,7 +110,7 @@ value make_rules loc rl sl cl errk =
         else
           let s =
             let b = accum_chars loc cl in
-            let e = cparser loc None [([], None, b)] in
+            let e = cparser loc (None, [([], None, b)]) in
             (SpNtr loc <:patt< $lid:var ()$ >> e, SpoBang)
           in
           [s :: sl]
@@ -133,6 +133,16 @@ value make_any loc norec sl cl errk =
   ([s :: sl], cl)
 ;
 
+value make_named c wo loc norec sl cl errk =
+  let (p, cl) =
+    if norec then (<:patt< $lid:c$ >>, cl)
+    else
+      (<:patt< $lid:c$ >>, [<:expr< $lid:c$ >> :: cl])
+  in
+  let s = (SpTrm loc p wo, errk) in
+  ([s :: sl], cl)
+;
+
 value fold_string_chars f s a =
   loop 0 a where rec loop i a =
     if i = String.length s then a
@@ -178,15 +188,15 @@ value make_sub_lexer loc f sl cl errk =
   ([s :: sl], [])
 ;
 
-value make_lookahd loc pll sl cl errk =
-  let s = (SpLhd loc pll, errk) in
+value make_lookahd loc pll wo sl cl errk =
+  let s = (SpLhd loc pll wo, errk) in
   ([s :: sl], cl)
 ;
 
 value gcl = ref [];
 
 EXTEND
-  GLOBAL: expr;
+  GLOBAL: expr ext_attributes;
   expr: LIKE "match"
     [ [ "lexer"; rl = rules ->
           let rl =
@@ -199,11 +209,12 @@ EXTEND
             | (None, rl) -> rl ]
           in
           <:expr< fun $lid:var ()$ -> $mk_lexer loc rl$ >>
-      | "match"; e = SELF; "with"; "lexer"; rl = rules ->
-          mk_lexer_match loc e rl ] ]
+      | "match"; (ext,attrs) = ext_attributes; e = SELF; "with"; "lexer"; rl = rules ->
+          Asttools.expr_to_inline (mk_lexer_match loc e rl) ext attrs ] ]
   ;
+  string_or_rawstring: [ [ s = STRING -> s | s = RAWSTRING -> snd(Asttools.split_rawstring s) ] ] ;
   expr: LEVEL "simple"
-    [ [ "$"; LIDENT "add"; s = STRING ->
+    [ [ "$"; LIDENT "add"; s = string_or_rawstring ->
           let s = Plexing.eval_string loc s in
           loop (accum_chars loc gcl.val) 0 where rec loop v i =
             if i = String.length s then v
@@ -234,11 +245,14 @@ EXTEND
   ;
   symb:
     [ [ "_"; norec = no_rec -> make_any loc norec
-      | s = STRING; norec = no_rec -> make_chars loc s norec
+      | "_" ; "as" ; id = LIDENT; eo = V (OPT [ "when"; e = expr LEVEL "simple" -> e ]) ; norec = no_rec ->
+         make_named id eo loc norec
+      | s = string_or_rawstring; norec = no_rec -> make_chars loc s norec
       | c = CHAR; norec = no_rec -> make_char loc c norec
       | c = CHAR; "-"; d = CHAR; norec = no_rec -> make_range loc c d norec
       | f = simple_expr -> make_sub_lexer loc f
-      | "?="; "["; pll = LIST1 lookahead SEP "|"; "]" -> make_lookahd loc pll
+      | "?="; "["; pll = LIST1 lookahead SEP "|"; "]" ; eo = V (OPT [ "when"; e = expr LEVEL "simple" -> e ]) ->
+         make_lookahd loc pll eo
       | rl = rules -> make_rules loc rl ] ]
   ;
   simple_expr:
@@ -248,7 +262,7 @@ EXTEND
   ;
   lookahead:
     [ [ pl = LIST1 lookahead_char -> pl
-      | s = STRING ->
+      | s = string_or_rawstring ->
           let s = Plexing.eval_string loc s in
           List.rev
             (fold_string_chars (fun c pl -> [<:patt< $chr:c$ >> :: pl]) s
@@ -257,7 +271,9 @@ EXTEND
   lookahead_char:
     [ [ c = CHAR -> <:patt< $chr:c$ >>
       | c = CHAR; "-"; d = CHAR -> <:patt< $chr:c$..$chr:d$ >>
-      | "_" -> <:patt< _ >> ] ]
+      | id = LIDENT -> <:patt< $lid:id$ >>
+      | "_" -> <:patt< _ >>
+      ] ]
   ;
   no_rec:
     [ [ "/" -> True
@@ -265,7 +281,7 @@ EXTEND
   ;
   err_kont:
     [ [ "!" -> SpoBang
-      | "?"; s = STRING -> SpoQues <:expr< $str:s$ >>
+      | "?"; s = string_or_rawstring -> SpoQues <:expr< $str:s$ >>
       | "?"; e = simple_expr -> SpoQues e
       | -> SpoNoth ] ]
   ;
