@@ -272,7 +272,7 @@ value dotop =
 ;
 
 value mktupexp loc e el = <:expr< ($list:[e::el]$) >>;
-value mktuppat loc p pl = <:patt< ($list:[p::pl]$) >>;
+value mktuppat loc p pl clflag = <:patt< ($list:[p::pl]$, $closed:clflag$) >>;
 value mktuptyp loc t tl = <:ctyp< ( $list:[t::tl]$ ) >>;
 
 value mklabdecl loc i mf t attrs = (loc, i, mf, t, attrs);
@@ -1289,11 +1289,26 @@ EXTEND
       | "("; p = paren_patt; ")" → p
       | "_" → <:patt< _ >> ] ]
   ;
+  tuple_patt_body1:
+    [ [ ","; ".." -> ([], False)
+      | ","; p = patt ; (pl, clflag) = tuple_patt_body1 -> ([p::pl], clflag)
+      |  -> ([], True)
+      ] ]
+  ;
+  tuple_patt_body:
+    [ [ ".." -> ([], False)
+      | p = patt ; (pl, clflag) = tuple_patt_body1 -> ([p::pl], clflag)
+      ] ]
+  ;
   paren_patt:
     [ [ p = patt; ":"; t = ctyp → <:patt< ($p$ : $t$) >>
       | p = patt; "as"; p2 = patt → <:patt< ($p$ as $p2$) >>
-      | p = patt; ","; pl = LIST1 patt SEP "," → mktuppat loc p pl
+      | p = patt; ","; (pl, clflag) = tuple_patt_body → mktuppat loc p pl clflag
       | p = patt → <:patt< $p$ >>
+      | pl = V (LIST1 patt SEP ","); ","; clflag = V [ ".." -> False | -> True ] "closed" → <:patt< ($_list:pl$, $_closed:clflag$) >>
+(*CHET
+      | pl = V (LIST1 patt SEP ","); ","; ".." → <:patt< ($_list:pl$, ..) >>
+ *)
       | pl = V (LIST1 patt SEP ",") → <:patt< ($_list:pl$) >>
       | "type"; s = V LIDENT → <:patt< (type $_lid:s$) >>
       | "module"; s = V uidopt "uidopt"; ":"; mt = module_type →
@@ -1334,12 +1349,27 @@ EXTEND
       | e = alg_extension -> <:patt< [% $_extension:e$ ] >>
       | "_" → <:patt< _ >> ] ]
   ;
+  tuple_ipatt_body1:
+    [ [ ","; ".." -> ([], False)
+      | ","; p = ipatt ; (pl, clflag) = tuple_ipatt_body1 -> ([p::pl], clflag)
+      |  -> ([], True)
+      ] ]
+  ;
+  tuple_ipatt_body:
+    [ [ ".." -> ([], False)
+      | p = ipatt ; (pl, clflag) = tuple_ipatt_body1 -> ([p::pl], clflag)
+      ] ]
+  ;
   paren_ipatt:
     [ [ p = ipatt; ":"; t = ctyp → <:patt< ($p$ : $t$) >>
       | p = ipatt; "as"; p2 = ipatt → <:patt< ($p$ as $p2$) >>
-      | p = ipatt; ","; pl = LIST1 ipatt SEP "," → mktuppat loc p pl
+      | p = ipatt; ","; (pl, clflag) = tuple_ipatt_body → mktuppat loc p pl clflag
       | p = ipatt → <:patt< $p$ >>
-      | pl = V (LIST1 ipatt SEP ",") → <:patt< ( $_list:pl$) >>
+      | pl = V (LIST1 ipatt SEP ","); ","; clflag = V [ ".." -> False | -> True ] "closed" → <:patt< ( $_list:pl$, $_closed:clflag$) >>
+(*CHET
+      | pl = V (LIST1 ipatt SEP ","); ","; ".." → <:patt< ( $_list:pl$, ..) >>
+ *)
+      | pl = V (LIST1 ipatt SEP ",") → <:patt< ( $_list:pl$ ) >>
       | "type"; s = V LIDENT → <:patt< (type $_lid:s$) >>
       | "module"; s  = V uidopt "uidopt"; ":"; mt = module_type →
           <:patt< (module $_uidopt:s$ : $mt$) >>
@@ -1374,19 +1404,30 @@ EXTEND
   constrain:
     [ [ "constraint"; t1 = ctyp; "="; t2 = ctyp → (t1, t2) ] ]
   ;
+  type_variance:
+    [ [ "+" -> "+"
+      | "+"; "!" -> "+!"
+      | "-" -> "-"
+      | "-"; "!" -> "-!"
+      | "!" -> "!"
+      | "!" ; "+" -> "!+"
+      | "!" ; "-" -> "!-"
+      | "!+" -> "!+"
+      | "+!" -> "+!"
+      | "!-" -> "!-"
+      | "-!" -> "-!"
+      | "+-" -> "+-"
+      | "-+" -> "-+"
+      | "+-!" -> "+-!"
+      | "-+!" -> "-+!"
+      | "!+-" -> "!+-"
+      | "!-+" -> "!-+"
+      | -> ""
+      ] ]
+  ;
   type_parameter:
-    [ [ "+"; p = V simple_type_parameter -> (p, (Some True, False))
-      | "+"; "!" ; p = V simple_type_parameter -> (p, (Some True, True))
-      | "-"; p = V simple_type_parameter -> (p, (Some False, False))
-      | "-"; "!" ; p = V simple_type_parameter -> (p, (Some False, True))
-      | "!" ; p = V simple_type_parameter -> (p, (None, True))
-      | "!" ; "+" ; p = V simple_type_parameter -> (p, (Some True, True))
-      | "!" ; "-" ; p = V simple_type_parameter -> (p, (Some False, True))
-      | "!+" ; p = V simple_type_parameter -> (p, (Some True, True))
-      | "+!" ; p = V simple_type_parameter -> (p, (Some True, True))
-      | "!-" ; p = V simple_type_parameter -> (p, (Some False, True))
-      | "-!" ; p = V simple_type_parameter -> (p, (Some False, True))
-      | p = V simple_type_parameter -> (p, (None, False))
+    [ [ tv = V type_variance "variance" ; p = V simple_type_parameter "var" ->
+        (p, Pcaml.vala_map Versdep.ocaml_normalize_camlp5_variance tv)
       ] ]
   ;
   simple_type_parameter:
@@ -1456,11 +1497,17 @@ EXTEND
   ;
   paren_ctyp:
     [ [ "(" ; "module"; mt = module_type ; ")" → <:ctyp< ( module $mt$ ) >>
-      | "("; t = ctyp; "*"; tl = LIST1 ctyp SEP "*"; ")" → mktuptyp loc t tl
-      | "("; t = ctyp; ")" → <:ctyp< $t$ >>
-      | "("; tl = V (LIST1 ctyp SEP "*"); ")" → <:ctyp< ( $_list:tl$ ) >>
+      | "("; t = maybe_labeled_ctyp; "*"; tl = LIST1 maybe_labeled_ctyp SEP "*"; ")" → mktuptyp loc t tl
+      | "("; t = maybe_labeled_ctyp; ")" → <:ctyp< $snd t$ >>
+      | "("; tl = V (LIST1 maybe_labeled_ctyp SEP "*"); ")" → <:ctyp< ( $_list:tl$ ) >>
     ] ]
   ;
+  maybe_labeled_ctyp:
+  [ [ check_lident_colon ; li = LIDENT ; ":" ; t = ctyp ->
+      let li = <:vala< li >> in
+      (<:vala< Some li >>, t)
+    | t = ctyp -> (<:vala< None >>, t)
+    ] ] ;
   ctyp_below_alg_attribute:
   [ [ x = ctyp LEVEL "below_alg_attribute" -> x ]
   ]
@@ -1771,8 +1818,8 @@ EXTEND
   patt: LEVEL "simple"
     [ [ "`"; s = V ident "" → <:patt< ` $_:s$ >>
       | "#"; lili = V extended_longident_lident "lilongid" → <:patt< # $_lilongid:lili$ >>
-      | "~"; "{"; lppo = V (LIST1 patt_tcon_opt_eq_patt SEP ";"); "}" →
-          <:patt< ~{$_list:lppo$} >>
+      | "~"; "{"; (p,po) = patt_tcon_opt_eq_patt; "}" →
+          <:patt< ~{$p$ $_opt:po$} >>
       | "?"; "{"; p = patt_tcon; eo = V (OPT [ "="; e = expr → e ]); "}" →
           <:patt< ?{$p$ $_opt:eo$ } >>
       | i = V TILDEIDENTCOLON; p = SELF →
@@ -1793,8 +1840,8 @@ EXTEND
       | p = patt; ":"; t = ctyp → <:patt< ($p$ : $t$) >> ] ]
   ;
   ipatt:
-    [ [ "~"; "{"; lppo = V (LIST1 ipatt_tcon_opt_eq_patt SEP ";"); "}" →
-          <:patt< ~{$_list:lppo$} >>
+    [ [ "~"; "{"; (p,po) = ipatt_tcon_opt_eq_patt; "}" →
+          <:patt< ~{$p$ $_opt:po$} >>
       | "?"; "{"; p = ipatt_tcon; eo = V (OPT [ "="; e = expr → e ]); "}" →
           <:patt< ?{$p$ $_opt:eo$ } >>
 
@@ -1838,8 +1885,8 @@ EXTEND
   ;
   expr: AFTER "apply"
     [ "label" NONA
-      [ "~"; "{"; lpeo = V (LIST1 ipatt_tcon_fun_binding SEP ";"); "}" →
-          <:expr< ~{$_list:lpeo$ } >>
+      [ "~"; "{"; (p,eo) = ipatt_tcon_fun_binding; "}" →
+          <:expr< ~{$p$ $_opt:eo$} >>
       | "?"; "{"; p = ipatt_tcon; eo = V (OPT fun_binding); "}" →
           <:expr< ?{$p$ $_opt:eo$ } >>
       | i = V TILDEIDENTCOLON; e = SELF →
