@@ -1,28 +1,29 @@
 (* camlp5r *)
 (* pa_rp.ml,v *)
-(* Copyright (c) INRIA 2007-2014 *)
+(* Copyright (c) INRIA 2007-2017 *)
 
 #load "pa_extend.cmo";
 #load "q_MLast.cmo";
 
+open Asttools;
 open Exparser;
 open Pcaml;
 
 (* Syntax extensions in Revised Syntax grammar *)
 
 EXTEND
-  GLOBAL: expr ipatt;
+  GLOBAL: expr ipatt ext_attributes;
   expr: LEVEL "top"
     [ [ "parser"; po = OPT ipatt; "["; pcl = LIST0 parser_case SEP "|"; "]" ->
-          <:expr< $cparser loc po pcl$ >>
+          <:expr< $cparser loc (po, pcl)$ >>
       | "parser"; po = OPT ipatt; pc = parser_case ->
-          <:expr< $cparser loc po [pc]$ >>
-      | "match"; e = SELF; "with"; "parser"; po = OPT ipatt; "[";
+          <:expr< $cparser loc (po, [pc])$ >>
+      | "match"; (ext,attrs) = ext_attributes; e = SELF; "with"; "parser"; po = OPT ipatt; "[";
         pcl = LIST0 parser_case SEP "|"; "]" ->
-          <:expr< $cparser_match loc e po pcl$ >>
-      | "match"; e = SELF; "with"; "parser"; po = OPT ipatt;
+          expr_to_inline <:expr< $cparser_match loc e (po, pcl)$ >> ext attrs
+      | "match"; (ext,attrs) = ext_attributes; e = SELF; "with"; "parser"; po = OPT ipatt;
         pc = parser_case ->
-          <:expr< $cparser_match loc e po [pc]$ >> ] ]
+          expr_to_inline <:expr< $cparser_match loc e (po, [pc])$ >> ext attrs ] ]
   ;
   parser_case:
     [ [ "[:"; sp = stream_patt; ":]"; po = OPT ipatt; "->"; e = expr ->
@@ -48,7 +49,7 @@ EXTEND
   stream_patt_comp:
     [ [ "`"; p = patt; eo = V (OPT [ "when"; e = expr -> e ]) ->
           SpTrm loc p eo
-      | "?="; pll = LIST1 lookahead SEP "|" -> SpLhd loc pll
+      | "?="; pll = LIST1 lookahead SEP "|"; eo = V (OPT [ "when"; e = expr -> e ]) -> SpLhd loc pll eo
       | p = patt; "="; e = expr -> SpNtr loc p e
       | p = patt -> SpStr loc p ] ]
   ;
