@@ -263,8 +263,8 @@ value mktupexp _ e el =
   Qast.Node "ExTup" [Qast.Loc; Qast.VaVal (Qast.Cons e (Qast.List el))]
 ;
 
-value mktuppat _ p pl =
-  Qast.Node "PaTup" [Qast.Loc; Qast.VaVal (Qast.Cons p (Qast.List pl))]
+value mktuppat _ p pl clflag =
+  Qast.Node "PaTup" [Qast.Loc; Qast.VaVal (Qast.Cons p (Qast.List pl)); Qast.VaVal (Qast.Bool clflag)]
 ;
 
 value mktuptyp _ t tl =
@@ -465,6 +465,22 @@ value check_type_binder =
     check_type_binder_f
 ;
 
+value is_lident_colon_f strm =
+  match Stream.npeek 2 strm with [
+    [("LIDENT",_) ; ("",":") :: _] -> True
+  | _ -> False
+  ]
+;
+
+value check_lident_colon_f strm =
+  if is_lident_colon_f strm then () else raise Stream.Failure
+;
+
+value check_lident_colon =
+  Grammar.Entry.of_parser gram "check_lident_colon"
+    check_lident_colon_f
+;
+
 (* -- begin copy from pa_r to q_MLast -- *)
 
 EXTEND
@@ -1210,12 +1226,24 @@ EXTEND
       | "("; p = paren_patt; ")" → p
       | "_" → Qast.Node "PaAny" [Qast.Loc] ] ]
   ;
+  tuple_patt_body1:
+    [ [ ","; ".." -> ([], False)
+      | ","; p = patt ; (pl, clflag) = tuple_patt_body1 -> ([p::pl], clflag)
+      |  -> ([], True)
+      ] ]
+  ;
+  tuple_patt_body:
+    [ [ ".." -> ([], False)
+      | p = patt ; (pl, clflag) = tuple_patt_body1 -> ([p::pl], clflag)
+      ] ]
+  ;
   paren_patt:
     [ [ p = patt; ":"; t = ctyp → Qast.Node "PaTyc" [Qast.Loc; p; t]
       | p = patt; "as"; p2 = patt → Qast.Node "PaAli" [Qast.Loc; p; p2]
-      | p = patt; ","; pl = LIST1 patt SEP "," → mktuppat Qast.Loc p pl
+      | p = patt; ","; (pl, clflag) = tuple_patt_body → mktuppat Qast.Loc p pl clflag
       | p = patt → p
-      | pl = SV (LIST1 patt SEP ",") → Qast.Node "PaTup" [Qast.Loc; pl]
+      | pl = SV (LIST1 patt SEP ",") ; "," ; clflag = SV [ ".." -> Qast.Bool False | -> Qast.Bool True ] "closed" → Qast.Node "PaTup" [Qast.Loc; pl; clflag]
+      | pl = SV (LIST1 patt SEP ",") → Qast.Node "PaTup" [Qast.Loc; pl; Qast.VaVal (Qast.Bool True)]
       | "type"; s = SV LIDENT → Qast.Node "PaNty" [Qast.Loc; s]
       | "module"; s = SV uidopt "uidopt"; ":"; mt = module_type →
           Qast.Node "PaUnp" [Qast.Loc; s; Qast.Option (Some mt)]
@@ -1251,12 +1279,24 @@ Qast.Node "PaLong" [Qast.Loc; Qast.Node "LiUid" [Qast.Loc; (Qast.VaVal (Qast.Str
       | s = SV GIDENT → Qast.Node "PaLid" [Qast.Loc; s]
       | "_" → Qast.Node "PaAny" [Qast.Loc] ] ]
   ;
+  tuple_ipatt_body1:
+    [ [ ","; ".." -> ([], False)
+      | ","; p = ipatt ; (pl, clflag) = tuple_ipatt_body1 -> ([p::pl], clflag)
+      |  -> ([], True)
+      ] ]
+  ;
+  tuple_ipatt_body:
+    [ [ ".." -> ([], False)
+      | p = ipatt ; (pl, clflag) = tuple_ipatt_body1 -> ([p::pl], clflag)
+      ] ]
+  ;
   paren_ipatt:
     [ [ p = ipatt; ":"; t = ctyp → Qast.Node "PaTyc" [Qast.Loc; p; t]
       | p = ipatt; "as"; p2 = ipatt → Qast.Node "PaAli" [Qast.Loc; p; p2]
-      | p = ipatt; ","; pl = LIST1 ipatt SEP "," → mktuppat Qast.Loc p pl
+      | p = ipatt; ","; (pl, clflag) = tuple_ipatt_body → mktuppat Qast.Loc p pl clflag
       | p = ipatt → p
-      | pl = SV (LIST1 ipatt SEP ",") → Qast.Node "PaTup" [Qast.Loc; pl]
+      | pl = SV (LIST1 ipatt SEP ","); ","; clflag = SV [ ".." -> Qast.Bool False | -> Qast.Bool True ] "closed" → Qast.Node "PaTup" [Qast.Loc; pl; clflag]
+      | pl = SV (LIST1 ipatt SEP ",") → Qast.Node "PaTup" [Qast.Loc; pl; Qast.VaVal (Qast.Bool True)]
       | "type"; s = SV LIDENT → Qast.Node "PaNty" [Qast.Loc; s]
       | "module"; s = SV uidopt "uidopt"; ":"; mt = module_type →
           Qast.Node "PaUnp" [Qast.Loc; s; Qast.Option (Some mt)]
@@ -1294,12 +1334,31 @@ Qast.Node "PaLong" [Qast.Loc; Qast.Node "LiUid" [Qast.Loc; (Qast.VaVal (Qast.Str
   constrain:
     [ [ "constraint"; t1 = ctyp; "="; t2 = ctyp → Qast.Tuple [t1; t2] ] ]
   ;
+  type_variance:
+    [ [ "+" -> Qast.Str "+"
+      | "+"; "!" -> Qast.Str "+!"
+      | "-" -> Qast.Str "-"
+      | "-"; "!" -> Qast.Str "-!"
+      | "!" -> Qast.Str "!"
+      | "!" ; "+" -> Qast.Str "!+"
+      | "!" ; "-" -> Qast.Str "!-"
+      | "!+" -> Qast.Str "!+"
+      | "+!" -> Qast.Str "+!"
+      | "!-" -> Qast.Str "!-"
+      | "-!" -> Qast.Str "-!"
+      | "+-" -> Qast.Str "+-"
+      | "-+" -> Qast.Str "-+"
+      | "+-!" -> Qast.Str "+-!"
+      | "-+!" -> Qast.Str "-+!"
+      | "!+-" -> Qast.Str "!+-"
+      | "!-+" -> Qast.Str "!-+"
+      | -> Qast.Str ""
+      ] ]
+  ;
   type_parameter:
-    [ [ "+"; p = SV simple_type_parameter →
-          Qast.Tuple [p; Qast.Option (Some (Qast.Bool True))]
-      | "-"; p = SV simple_type_parameter →
-          Qast.Tuple [p; Qast.Option (Some (Qast.Bool False))]
-      | p = SV simple_type_parameter → Qast.Tuple [p; Qast.Option None] ] ]
+    [ [ tv = SV type_variance "variance"; p = SV simple_type_parameter "var" →
+          Qast.Tuple [p; tv]
+      ] ]
   ;
   simple_type_parameter:
     [ [ "'"; i = ident → Qast.Option (Some i)
@@ -1367,13 +1426,21 @@ Qast.Node "PaLong" [Qast.Loc; Qast.Node "LiUid" [Qast.Loc; (Qast.VaVal (Qast.Str
       | "("; t = SELF; "*"; tl = LIST1 ctyp SEP "*"; ")" →
           mktuptyp Qast.Loc t tl
       | "("; t = SELF; ")" → t
-      | "("; tl = SV (LIST1 ctyp SEP "*"); ")" →
+      | "("; tl = SV (LIST1 maybe_labeled_ctyp SEP "*"); ")" →
           Qast.Node "TyTup" [Qast.Loc; tl]
       | "["; cdl = SV (LIST0 constructor_declaration SEP "|"); "]" →
           Qast.Node "TySum" [Qast.Loc; cdl]
       | "{"; ldl = SV (LIST1 label_declaration SEP ";"); "}" →
           Qast.Node "TyRec" [Qast.Loc; ldl] ] ]
   ;
+  maybe_labeled_ctyp:
+  [ [ check_lident_colon ; li = LIDENT ; ":" ; t = ctyp ->
+          Qast.Tuple [
+              Qast.VaVal (Qast.Option (Some (Qast.VaVal (Qast.Str li))))
+            ; t
+            ]
+    | t = ctyp -> Qast.Tuple [Qast.VaVal (Qast.Option None); t]
+    ] ] ;
   cons_ident:
   [ [ ci = SV UIDENT -> ci
     ] ] ;
@@ -1676,71 +1743,51 @@ Qast.Node "PaLong" [Qast.Loc; Qast.Node "LiUid" [Qast.Loc; (Qast.VaVal (Qast.Str
   patt: LEVEL "simple"
     [ [ "`"; s = SV ident "" → Qast.Node "PaVrn" [Qast.Loc; s]
       | "#"; lili = SV extended_longident_lident "lilongid" → Qast.Node "PaTyp" [Qast.Loc; lili]
-      | "~"; "{"; lppo = SV (LIST1 patt_tcon_opt_eq_patt SEP ";"); "}" →
-          Qast.Node "PaLab" [Qast.Loc; lppo]
+      | "~"; "{"; p = patt_tcon; po = SV (OPT [ "="; p = patt → p ]); "}" →
+          Qast.Node "PaLab" [Qast.Loc; p; po]
       | "?"; "{"; p = patt_tcon; eo = SV (OPT [ "="; e = expr → e ]); "}" →
           Qast.Node "PaOlb" [Qast.Loc; p; eo]
       | i = SV TILDEIDENTCOLON "~:" a_tic; p = SELF →
           let _ = warning_deprecated_since_6_00 loc in
           Qast.Node "PaLab"
             [Qast.Loc;
-             Qast.VaVal
-               (Qast.List
-                  [Qast.Tuple
-                     [Qast.Node "PaLid" [Qast.Loc; i];
-                      Qast.VaVal (Qast.Option (Some p))]])]
+             Qast.Node "PaLid" [Qast.Loc; i];
+             Qast.VaVal (Qast.Option (Some p))]
       | i = SV TILDEIDENT "~" a_ti →
           let _ = warning_deprecated_since_6_00 loc in
           Qast.Node "PaLab"
             [Qast.Loc;
-             Qast.VaVal
-               (Qast.List
-                  [Qast.Tuple
-                     [Qast.Node "PaLid" [Qast.Loc; i];
-                      Qast.VaVal (Qast.Option None)]])]
+             Qast.Node "PaLid" [Qast.Loc; i];
+             Qast.VaVal (Qast.Option None)]
       | p = patt_option_label →
           let _ = warning_deprecated_since_6_00 loc in
           p ] ]
   ;
-  patt_tcon_opt_eq_patt:
-    [ [ p = patt_tcon; po = SV (OPT [ "="; p = patt → p ]) →
-          Qast.Tuple [p; po] ] ]
-  ;
   patt_tcon:
     [ [ p = patt → p
       | p = patt; ":"; t = ctyp → Qast.Node "PaTyc" [Qast.Loc; p; t] ] ]
   ;
   ipatt:
-    [ [ "~"; "{"; lppo = SV (LIST1 ipatt_tcon_opt_eq_patt SEP ";"); "}" →
-          Qast.Node "PaLab" [Qast.Loc; lppo]
+    [ [ "~"; "{"; p = ipatt_tcon; po = SV (OPT [ "="; p = patt → p ]); "}" →
+          Qast.Node "PaLab" [Qast.Loc; p; po]
       | "?"; "{"; p = ipatt_tcon; eo = SV (OPT [ "="; e = expr → e ]); "}" →
           Qast.Node "PaOlb" [Qast.Loc; p; eo]
       | i = SV TILDEIDENTCOLON "~:" a_tic; p = SELF →
           let _ = warning_deprecated_since_6_00 loc in
           Qast.Node "PaLab"
             [Qast.Loc;
-             Qast.VaVal
-               (Qast.List
-                  [Qast.Tuple
-                     [Qast.Node "PaLid" [Qast.Loc; i];
-                      Qast.VaVal (Qast.Option (Some p))]])]
+             Qast.Node "PaLid" [Qast.Loc; i];
+             Qast.VaVal (Qast.Option (Some p))]
       | i = SV TILDEIDENT "~" a_ti →
           let _ = warning_deprecated_since_6_00 loc in
           Qast.Node "PaLab"
             [Qast.Loc;
-             Qast.VaVal
-               (Qast.List
-                  [Qast.Tuple
-                     [Qast.Node "PaLid" [Qast.Loc; i];
-                      Qast.VaVal (Qast.Option None)]])]
+             Qast.Node "PaLid" [Qast.Loc; i];
+             Qast.VaVal (Qast.Option None)]
       | p = patt_option_label →
           let _ = warning_deprecated_since_6_00 loc in
           p ] ]
   ;
-  ipatt_tcon_opt_eq_patt:
-    [ [ p = ipatt_tcon; po = SV (OPT [ "="; p = patt → p ]) →
-          Qast.Tuple [p; po] ] ]
-  ;
   ipatt_tcon:
     [ [ p = ipatt → p
       | p = ipatt; ":"; t = ctyp → Qast.Node "PaTyc" [Qast.Loc; p; t] ] ]
@@ -1808,28 +1855,22 @@ Qast.Node "PaLong" [Qast.Loc; Qast.Node "LiUid" [Qast.Loc; (Qast.VaVal (Qast.Str
   ;
   expr: AFTER "apply"
     [ "label" NONA
-      [ "~"; "{"; lpeo = SV (LIST1 ipatt_tcon_fun_binding SEP ";"); "}" →
-          Qast.Node "ExLab" [Qast.Loc; lpeo]
+      [ "~"; "{"; p = ipatt_tcon; eo = SV (OPT fun_binding); "}" →
+          Qast.Node "ExLab" [Qast.Loc; p; eo]
       | "?"; "{"; p = ipatt_tcon; eo = SV (OPT fun_binding); "}" →
           Qast.Node "ExOlb" [Qast.Loc; p; eo]
       | i = SV TILDEIDENTCOLON "~:" a_tic; e = SELF →
           let _ = warning_deprecated_since_6_00 loc in
           Qast.Node "ExLab"
             [Qast.Loc;
-             Qast.VaVal
-               (Qast.List
-                  [Qast.Tuple
-                     [Qast.Node "PaLid" [Qast.Loc; i];
-                      Qast.VaVal (Qast.Option (Some e))]])]
+             Qast.Node "PaLid" [Qast.Loc; i];
+             Qast.VaVal (Qast.Option (Some e))]
       | i = SV TILDEIDENT "~" a_ti →
           let _ = warning_deprecated_since_6_00 loc in
           Qast.Node "ExLab"
             [Qast.Loc;
-             Qast.VaVal
-               (Qast.List
-                  [Qast.Tuple
-                     [Qast.Node "PaLid" [Qast.Loc; i];
-                      Qast.VaVal (Qast.Option None)]])]
+             Qast.Node "PaLid" [Qast.Loc; i];
+             Qast.VaVal (Qast.Option None)]
       | i = SV QUESTIONIDENTCOLON "?:" a_qic; e = SELF →
           let _ = warning_deprecated_since_6_00 loc in
           Qast.Node "ExOlb"
@@ -1841,9 +1882,6 @@ Qast.Node "PaLong" [Qast.Loc; Qast.Node "LiUid" [Qast.Loc; (Qast.VaVal (Qast.Str
             [Qast.Loc; Qast.Node "PaLid" [Qast.Loc; i];
              Qast.VaVal (Qast.Option None)] ] ]
   ;
-  ipatt_tcon_fun_binding:
-    [ [ p = ipatt_tcon; eo = SV (OPT fun_binding) → Qast.Tuple [p; eo] ] ]
-  ;
   expr: LEVEL "simple"
     [ [ "`"; s = SV ident "" → Qast.Node "ExVrn" [Qast.Loc; s] ] ]
   ;
