@@ -12,8 +12,7 @@ module Meta_make (C : MetaSig) =
   struct
     module C = C ;
     open MLast;
-    value variance (var, inj) =
-      C.tuple [C.option C.bool var; C.bool inj] ;
+    value variance v = C.vala C.string v ;
     value type_var (s, tv) =
       C.tuple [C.vala (C.option C.string) s; variance tv]
     ;
@@ -68,7 +67,7 @@ module Meta_make (C : MetaSig) =
                (C.list
                   generic_constructor)
                llsltot]
-      | TyTup _ lt → C.node "TyTup" [C.vala (C.list ctyp) lt]
+      | TyTup _ lt → C.node "TyTup" [C.vala (C.list labeled_ctyp) lt]
       | TyVrn _ lpv ools →
           C.node "TyVrn"
             [C.vala (C.list poly_variant) lpv;
@@ -80,6 +79,7 @@ module Meta_make (C : MetaSig) =
       | TyOpen _ li t ->
          C.node "TyOpen" [longid li; ctyp t]
       ]
+    and labeled_ctyp (so, ct) = C.tuple [C.vala (C.option (C.vala C.string)) so; ctyp ct]
     and longid_lident (lio, s) = C.tuple [C.option (C.vala longid) lio; C.vala C.string s]
     and conv_attributes attrs =
       C.vala (C.list attribute) attrs
@@ -124,12 +124,8 @@ module Meta_make (C : MetaSig) =
       | PaEff _ p1 p2 → C.node "PaEff" [patt p1; patt p2]
       | PaFlo _ s → C.node "PaFlo" [C.vala C.string s]
       | PaInt _ s1 s2 → C.node "PaInt" [C.vala C.string s1; C.string s2]
-      | PaLab _ lpop →
-          C.node "PaLab"
-            [C.vala
-               (C.list
-                  (fun (p, op) → C.tuple [patt p; C.vala (C.option patt) op]))
-               lpop]
+      | PaLab _ p op →
+          C.node "PaLab" [patt p; C.vala (C.option patt) op]
       | PaLaz _ p → C.node "PaLaz" [patt p]
       | PaLid _ s → C.node "PaLid" [C.vala C.string s]
       | PaNty _ s → C.node "PaNty" [C.vala C.string s]
@@ -140,7 +136,7 @@ module Meta_make (C : MetaSig) =
             [C.vala (C.list (fun (p1, p2) → C.tuple [patt p1; patt p2])) lpp]
       | PaRng _ p1 p2 → C.node "PaRng" [patt p1; patt p2]
       | PaStr _ s → C.node "PaStr" [C.vala C.string s]
-      | PaTup _ lp → C.node "PaTup" [C.vala (C.list patt) lp]
+      | PaTup _ lp clflag → C.node "PaTup" [C.vala (C.list patt) lp; C.vala C.bool clflag]
       | PaTyc _ p t → C.node "PaTyc" [patt p; ctyp t]
       | PaTyp _ lili → C.node "PaTyp" [C.vala longid_lident lili]
       | PaUnp _ s omt →
@@ -183,12 +179,8 @@ module Meta_make (C : MetaSig) =
                lpoee]
       | ExIfe _ e1 e2 e3 → C.node "ExIfe" [expr e1; expr e2; expr e3]
       | ExInt _ s1 s2 → C.node "ExInt" [C.vala C.string s1; C.string s2]
-      | ExLab _ lpoe →
-          C.node "ExLab"
-            [C.vala
-               (C.list
-                  (fun (p, oe) → C.tuple [patt p; C.vala (C.option expr) oe]))
-               lpoe]
+      | ExLab _ p oe →
+          C.node "ExLab" [patt p; C.vala (C.option expr) oe]
       | ExLaz _ e → C.node "ExLaz" [expr e]
       | ExLet _ b lpe e →
           C.node "ExLet"
