@@ -658,6 +658,10 @@ let parse_antiquot s =
   with Not_found | Failure _ -> None
 ;;
 
+let int_of_string_result s =
+  try Some (int_of_string s) with Failure _ -> None
+;;
+
 let parse_antiloc s =
   try
     let i = String.index s ':' in
@@ -859,6 +863,34 @@ and linedir_quote n s =
   | _ -> false
 ;;
 
+let linedir_result n s =
+  let rec linedir n s =
+    match stream_peek_nth n s with
+      Some (' ' | '\t') -> linedir (n + 1) s
+    | Some ('0'..'9' as c) -> linedir_digits [c] (n + 1) s
+    | _ -> None
+  and linedir_digits acc n s =
+    match stream_peek_nth n s with
+      Some ('0'..'9' as c) -> linedir_digits (c :: acc) (n + 1) s
+    | _ ->
+        match int_of_string_result (rev_implode acc) with
+          Some lnum -> linedir_quote lnum n s
+        | None -> None
+  and linedir_quote lnum n s =
+    match stream_peek_nth n s with
+      Some (' ' | '\t') -> linedir_quote lnum (n + 1) s
+    | Some '"' -> linedir_qs lnum [] (n + 1) s
+    | _ -> None
+  and linedir_qs lnum acc n s =
+    match stream_peek_nth n s with
+      Some '"' -> Some (lnum, rev_implode acc)
+    | Some ('\n' | '\r') -> None
+    | Some c -> linedir_qs lnum (c :: acc) (n + 1) s
+    | _ -> None
+  in
+  linedir n s
+;;
+
 let rec any_to_nl buf (strm__ : _ Stream.t) =
   match Stream.peek strm__ with
     Some ('\r' | '\n' as c) -> Stream.junk strm__; Plexing.Lexbuf.add c buf
@@ -931,6 +963,30 @@ let raw_string_starter_p ctx strm =
   predrec 1
 ;;
 
+let zerobuf f buf strm = f Plexing.Lexbuf.empty strm;;
+
+let rec ws buf (strm__ : _ Stream.t) =
+  let buf =
+    match Stream.peek strm__ with
+      Some ' ' -> Stream.junk strm__; buf
+    | Some '\t' -> Stream.junk strm__; buf
+    | Some '\n' -> Stream.junk strm__; buf
+    | _ -> raise Stream.Failure
+  in
+  try ws buf strm__ with Stream.Failure -> buf
+;;
+
+let rec extattrident buf (strm__ : _ Stream.t) =
+  let buf = ident buf strm__ in
+  match Stream.peek strm__ with
+    Some '.' ->
+      Stream.junk strm__;
+      begin try extattrident (Plexing.Lexbuf.add '.' buf) strm__ with
+        Stream.Failure -> raise (Stream.Error "")
+      end
+  | _ -> buf
+;;
+
 let comment_rawstring ctx bp (buf : Plexing.Lexbuf.t) strm =
   if not (raw_string_starter_p ctx strm) then buf
   else
@@ -938,6 +994,31 @@ let comment_rawstring ctx bp (buf : Plexing.Lexbuf.t) strm =
     let rs = Printf.sprintf "{%s|%s|%s}" delim s delim in add_string buf rs
 ;;
 
+
+let comment_quoted_extension1 ctx bp extid buf strm =
+  let (delim, s) = rawstring0 ctx bp Plexing.Lexbuf.empty strm in
+  let exts = Printf.sprintf "{%%%s %s|%s|%s}" extid delim s delim in
+  add_string buf exts
+;;
+
+let comment_quoted_extension0 ctx bp extid buf (strm__ : _ Stream.t) =
+  match try Some (ws buf strm__) with Stream.Failure -> None with
+    Some buf ->
+      begin try
+        zerobuf (comment_quoted_extension1 ctx bp extid) buf strm__
+      with Stream.Failure -> raise (Stream.Error "")
+      end
+  | _ -> zerobuf (comment_quoted_extension1 ctx bp extid) buf strm__
+;;
+
+let comment_quoted_extension ctx bp buf (strm__ : _ Stream.t) =
+  let buf = extattrident buf strm__ in
+  try
+    zerobuf (comment_quoted_extension0 ctx bp (Plexing.Lexbuf.get buf)) buf
+      strm__
+  with Stream.Failure -> raise (Stream.Error "")
+;;
+
 let comment ctx bp =
   let rec comment buf (strm__ : _ Stream.t) =
     match Stream.peek strm__ with
@@ -951,10 +1032,17 @@ let comment ctx bp =
         end
     | Some '{' ->
         Stream.junk strm__;
-        let buf =
-          comment_rawstring ctx bp (Plexing.Lexbuf.add '{' buf) strm__
-        in
-        comment buf strm__
+        begin match Stream.peek strm__ with
+          Some '%' ->
+            Stream.junk strm__;
+            let buf = comment_quoted_extension ctx bp buf strm__ in
+            comment buf strm__
+        | _ ->
+            let buf =
+              comment_rawstring ctx bp (Plexing.Lexbuf.add '{' buf) strm__
+            in
+            comment buf strm__
+        end
     | Some '(' ->
         Stream.junk strm__;
         begin match Stream.peek strm__ with
@@ -1008,59 +1096,39 @@ let keyword_or_error_or_rawstring ctx bp (loc, s) buf strm =
     | _ -> keyword_or_error ctx loc "{"
   else
     let (delim, s) = rawstring0 ctx bp Plexing.Lexbuf.empty strm in
-    "STRING", String.escaped s
+    "RAWSTRING",
+    Printf.sprintf "%d:%s" (String.length delim) (String.escaped s)
 ;;
 
-let zerobuf f buf strm = f Plexing.Lexbuf.empty strm;;
-
-let rec ws buf (strm__ : _ Stream.t) =
-  let buf =
-    match Stream.peek strm__ with
-      Some ' ' -> Stream.junk strm__; buf
-    | Some '\t' -> Stream.junk strm__; buf
-    | Some '\n' -> Stream.junk strm__; buf
-    | _ -> raise Stream.Failure
-  in
-  try ws buf strm__ with Stream.Failure -> buf
-;;
-
-let rec extattrident buf (strm__ : _ Stream.t) =
-  let buf = ident buf strm__ in
+let rec keepws buf (strm__ : _ Stream.t) =
   match Stream.peek strm__ with
-    Some '.' ->
+    Some (' ' | '\t' | '\n' as c) ->
       Stream.junk strm__;
-      begin try extattrident (Plexing.Lexbuf.add '.' buf) strm__ with
-        Stream.Failure -> raise (Stream.Error "")
-      end
-  | _ -> buf
+      let buf = Plexing.Lexbuf.add c buf in
+      (try keepws buf strm__ with Stream.Failure -> buf)
+  | _ -> raise Stream.Failure
 ;;
 
-let quoted_extension1 ctx (bp, _) extid buf strm =
+let quoted_extension1 kind ctx (bp, _) buf strm =
   let (delim, s) = rawstring0 ctx bp Plexing.Lexbuf.empty strm in
-  "QUOTEDEXTENSION", extid ^ ":" ^ String.escaped s
+  let lhs = Plexing.Lexbuf.get buf in
+  let qext = Printf.sprintf "%s%s|%s|%s}" lhs delim s delim in kind, qext
 ;;
 
-let quoted_extension0 ctx (bp, _) extid buf (strm__ : _ Stream.t) =
-  match try Some (ws buf strm__) with Stream.Failure -> None with
+let quoted_extension0 kind ctx (bp, _) buf (strm__ : _ Stream.t) =
+  match try Some (keepws buf strm__) with Stream.Failure -> None with
     Some buf ->
       begin try
-        zerobuf (quoted_extension1 ctx (bp, Stream.count strm__) extid) buf
-          strm__
+        quoted_extension1 kind ctx (bp, Stream.count strm__) buf strm__
       with Stream.Failure -> raise (Stream.Error "")
       end
-  | _ ->
-      zerobuf (quoted_extension1 ctx (bp, Stream.count strm__) extid) buf
-        strm__
+  | _ -> quoted_extension1 kind ctx (bp, Stream.count strm__) buf strm__
 ;;
 
-let quoted_extension ctx (bp, _) buf (strm__ : _ Stream.t) =
+let quoted_extension (kind : string) ctx (bp, _) buf (strm__ : _ Stream.t) =
   let buf = extattrident buf strm__ in
-  try
-    zerobuf
-      (quoted_extension0 ctx (bp, Stream.count strm__)
-         (Plexing.Lexbuf.get buf))
-      buf strm__
-  with Stream.Failure -> raise (Stream.Error "")
+  try quoted_extension0 kind ctx (bp, Stream.count strm__) buf strm__ with
+    Stream.Failure -> raise (Stream.Error "")
 ;;
 let dotsymbolchar buf (strm__ : _ Stream.t) =
   match Stream.peek strm__ with
@@ -1458,10 +1526,25 @@ let next_token_after_spaces ctx bp buf (strm__ : _ Stream.t) =
                         | Some '%' ->
                             Stream.junk strm__;
                             begin try
-                              zerobuf
-                                (quoted_extension ctx
-                                   (bp, Stream.count strm__))
-                                buf strm__
+                              match Stream.peek strm__ with
+                                Some '%' ->
+                                  Stream.junk strm__;
+                                  begin try
+                                    quoted_extension "QUOTEDEXTENSION_ITEM"
+                                      ctx (bp, Stream.count strm__)
+                                      (Plexing.Lexbuf.add '%'
+                                         (Plexing.Lexbuf.add '%'
+                                            (Plexing.Lexbuf.add '{' buf)))
+                                      strm__
+                                  with Stream.Failure ->
+                                    raise (Stream.Error "")
+                                  end
+                              | _ ->
+                                  quoted_extension "QUOTEDEXTENSION_EXPR" ctx
+                                    (bp, Stream.count strm__)
+                                    (Plexing.Lexbuf.add '%'
+                                       (Plexing.Lexbuf.add '{' buf))
+                                    strm__
                             with Stream.Failure -> raise (Stream.Error "")
                             end
                         | Some ':' ->
@@ -1588,16 +1671,19 @@ let rec next_token ctx buf (strm__ : _ Stream.t) =
       Stream.junk strm__;
       let s = strm__ in
       let comm = get_comment buf () in
-      if linedir 1 s then
-        let buf = any_to_nl (Plexing.Lexbuf.add '#' buf) s in
-        incr !(Plexing.line_nb);
-        !(Plexing.bol_pos) := Stream.count s;
-        ctx.set_line_nb ();
-        ctx.after_space <- true;
-        next_token ctx buf s
-      else
-        let loc = ctx.make_lined_loc (bp, bp + 1) comm in
-        keyword_or_error ctx (bp, bp + 1) "#", loc
+      begin match linedir_result 1 s with
+        Some (linenum, fname) ->
+          let buf = any_to_nl (Plexing.Lexbuf.add '#' buf) s in
+          !(Plexing.line_nb) := linenum;
+          Plexing.input_file := fname;
+          !(Plexing.bol_pos) := Stream.count s;
+          ctx.set_line_nb ();
+          ctx.after_space <- true;
+          next_token ctx buf s
+      | None ->
+          let loc = ctx.make_lined_loc (bp, bp + 1) comm in
+          keyword_or_error ctx (bp, bp + 1) "#", loc
+      end
   | Some '(' ->
       Stream.junk strm__;
       begin match Stream.peek strm__ with
@@ -1899,8 +1985,9 @@ let using_token ctx kwd_table (p_con, p_prm) =
         end
   | "TILDEIDENT" | "TILDEIDENTCOLON" | "QUESTIONIDENT" |
     "QUESTIONIDENTCOLON" | "INT" | "INT_l" | "INT_L" | "INT_n" | "FLOAT" |
-    "QUOTEDEXTENSION" | "CHAR" | "STRING" | "QUOTATION" | "GIDENT" |
-    "ANTIQUOT" | "ANTIQUOT_LOC" | "EOI" ->
+    "QUOTEDEXTENSION_EXPR" | "QUOTEDEXTENSION_ITEM" | "CHAR" | "STRING" |
+    "RAWSTRING" | "QUOTATION" | "GIDENT" | "ANTIQUOT" | "ANTIQUOT_LOC" |
+    "EOI" ->
       ()
   | _ ->
       raise
@@ -1925,6 +2012,7 @@ let text =
   | "INT", s -> "'" ^ s ^ "'"
   | "FLOAT", "" -> "float"
   | "STRING", "" -> "string"
+  | "RAWSTRING", "" -> "raw string"
   | "CHAR", "" -> "char"
   | "QUOTATION", "" -> "quotation"
   | "ANTIQUOT", k -> "antiquot \"" ^ k ^ "\""
@@ -2006,12 +2094,12 @@ let gmake () =
   let glexr =
     ref
       {Plexing.tok_func =
-        (fun _ -> raise (Match_failure ("plexer.ml", 1034, 25)));
-       tok_using = (fun _ -> raise (Match_failure ("plexer.ml", 1034, 45)));
+        (fun _ -> raise (Match_failure ("plexer.ml", 1101, 25)));
+       tok_using = (fun _ -> raise (Match_failure ("plexer.ml", 1101, 45)));
        tok_removing =
-         (fun _ -> raise (Match_failure ("plexer.ml", 1034, 68)));
-       tok_match = (fun _ -> raise (Match_failure ("plexer.ml", 1035, 18)));
-       tok_text = (fun _ -> raise (Match_failure ("plexer.ml", 1035, 37)));
+         (fun _ -> raise (Match_failure ("plexer.ml", 1101, 68)));
+       tok_match = (fun _ -> raise (Match_failure ("plexer.ml", 1102, 18)));
+       tok_text = (fun _ -> raise (Match_failure ("plexer.ml", 1102, 37)));
        tok_comm = None; kwds = kwd_table}
   in
   let glex =
