@@ -1,6 +1,6 @@
 (* camlp5r *)
 (* pretty.mli,v *)
-(* Copyright (c) INRIA 2007-2014 *)
+(* Copyright (c) INRIA 2007-2017 *)
 
 (** Pretty printing on strings *)
 
@@ -11,6 +11,10 @@ val horiz_vertic : (unit -> 'a) -> (unit -> 'a) -> 'a;;
       below), then the function [h] stops and the function [v] is called
       which can print using several lines. *)
 
+val vertic : (unit -> 'a) -> 'a;;
+  (** [vertic v] [vertic v] is just like [horiz_vertic _ v], but it short-circuits
+      past printing horizontally, so you always get breaks. *)
+
 val sprintf : ('a, unit, string) format -> 'a;;
   (** [sprintf fmt ...] formats some string like [Printf.sprintf]
       does, except that, if it is called in the context of the *first*
