@@ -1,6 +1,6 @@
 (* camlp5r *)
 (* token.mli,v *)
-(* Copyright (c) INRIA 2007-2014 *)
+(* Copyright (c) INRIA 2007-2017 *)
 
 (** Module deprecated since Camlp5 version 5.00. Use now module Plexing.
     Compatibility assumed. *)
@@ -17,10 +17,11 @@ type 'te glexer =
       tok_removing : pattern -> unit;
       mutable tok_match : pattern -> 'te -> string;
       tok_text : pattern -> string;
-      mutable tok_comm : Ploc.t list option }
+      mutable tok_comm : Ploc.t list option;
+      kwds : (string, string) Hashtbl.t }
 ;;
 
-type 'te lexer_func = char Stream.t -> 'te Stream.t * location_function
+type 'te lexer_func = char Stream.t -> 'te Stream.t * Plexing.Locations.t
 and location_function = int -> Ploc.t;;
 
 val lexer_text : pattern -> string;;
@@ -35,7 +36,7 @@ val lexer_func_of_ocamllex : (Lexing.lexbuf -> 'te) -> 'te lexer_func;;
    (** Use now [Plexing.lexer_func_of_ocamllex] *)
 
 val make_stream_and_location :
-  (unit -> 'te * Ploc.t) -> 'te Stream.t * location_function;;
+  (unit -> 'te * Ploc.t) -> 'te Stream.t * Plexing.Locations.t;;
    (** Use now [Plexing.make_stream_and_location] *)
 
 val eval_char : string -> char;;
