@@ -61,18 +61,27 @@ let loc_none =
 let mkloc loc txt = {Location.txt = txt; loc = loc};;
 let mknoloc txt = mkloc loc_none txt;;
 
-let ocaml_id_or_li_of_string_list loc sl =
-  let mkli s =
-    let rec loop f =
-      function
-        i :: il -> loop (fun s -> Ldot (f i, s)) il
-      | [] -> f s
-    in
-    loop (fun s -> Lident s)
-  in
-  match List.rev sl with
+let ocaml_longident_ldot (loc : Location.t) li s = Ldot (li, s);;
+
+let ocaml_longident_lapply (loc : Location.t) li1 li2 = Lapply (li1, li2);;
+let ocaml_longident_lident (loc : Location.t) s = Lident s;;
+
+let ocaml_longident_option_of_string_list (loc : Location.t) sl =
+  match sl with
     [] -> None
-  | s :: sl -> Some (mkli s (List.rev sl))
+  | s :: sl ->
+      let rec loop acc =
+        function
+          [] -> acc
+        | s :: sl -> loop (ocaml_longident_ldot loc acc s) sl
+      in
+      Some (loop (ocaml_longident_lident loc s) sl)
+;;
+
+let ocaml_longident_of_string_list loc sl =
+  match ocaml_longident_option_of_string_list loc sl with
+    None -> failwith "ocaml_longident_of_string_list: empty argument"
+  | Some li -> li
 ;;
 
 let not_extended_longident =
@@ -255,15 +264,38 @@ let ocaml_mkfield_tag ?(alg_attributes = []) loc (lab, x) fl =
 ;;
 let ocaml_mkfield_var loc = [];;
 
-let convert_camlp5_variance (va, inj) =
-  assert (inj = false);
-  let va =
-    match va with
-      Some false -> Contravariant
-    | Some true -> Covariant
-    | _ -> Invariant
-  in
-  va
+let ocaml_variance_map =
+  ["", ("", (NoVariance, NoInjectivity));
+   "+", ("+", (Covariant, NoInjectivity));
+   "-", ("-", (Contravariant, NoInjectivity));
+   "!", ("!", (NoVariance, Injective)); "+!", ("+!", (Covariant, Injective));
+   "!+", ("+!", (Covariant, Injective));
+   "-!", ("-!", (Contravariant, Injective));
+   "!-", ("-!", (Contravariant, Injective));
+   "+!", ("+!", (Covariant, Injective)); "!+", ("+!", (Covariant, Injective));
+   "-!", ("-!", (Contravariant, Injective));
+   "!-", ("-!", (Contravariant, Injective))]
+;;
+
+let ocaml_normalize_camlp5_variance vastr =
+  match List.assoc vastr ocaml_variance_map with
+    v, _ -> v
+  | exception Not_found ->
+      failwith
+        (Printf.sprintf
+           "ocaml_normalize_camp5_variance: internal error, variance \"%s\" not found"
+           vastr)
+;;
+
+
+let convert_camlp5_variance vastr =
+  match List.assoc vastr ocaml_variance_map with
+    _, v -> v
+  | exception Not_found ->
+      failwith
+        (Printf.sprintf
+           "convert_camp5_variance: internal error, variance \"%s\" not found"
+           vastr)
 ;;
 
 let ocaml_ec_tuple ?(alg_attributes = []) loc s tyvars (x, rto) =
@@ -417,7 +449,7 @@ let ocaml_ptyp_variant loc catl clos sl_opt =
   Some (Ptyp_variant (catl, clos, sl_opt))
 ;;
 
-let ocaml_package_type li ltl =
+let ocaml_package_type loc li ltl : package_type =
   mknoloc li, List.map (fun (li, t) -> mkloc t.ptyp_loc li, t) ltl
 ;;
 
@@ -466,6 +498,18 @@ let ocaml_pexp_constraint e ot1 ot2 =
         Some t1 -> Pexp_constraint (e, t1)
       | None -> failwith "internal error: ocaml_pexp_constraint"
 ;;
+let ocaml_pexp_tuple x =
+  let x =
+    List.map
+      (function
+         None, t -> t
+       | Some _, t ->
+           failwith
+             "Pexp_tuple: labeled tuples only available with ocaml >= 5.04")
+      x
+  in
+  Pexp_tuple x
+;;
 
 let ocaml_pexp_construct loc li po chk_arity =
   Pexp_construct (mkloc loc li, po)
@@ -477,8 +521,9 @@ let ocaml_pexp_construct_args =
   | _ -> None
 ;;
 
-let mkexp_ocaml_pexp_construct_arity loc li_loc li al =
-  let a = ocaml_mkexp loc (Pexp_tuple al) in
+let mkexp_ocaml_pexp_construct_arity loc li_loc li (al : expression list) =
+  let al = List.map (fun x -> None, x) al in
+  let a = ocaml_mkexp loc (ocaml_pexp_tuple al) in
   {pexp_desc = ocaml_pexp_construct li_loc li (Some a) true; pexp_loc = loc;
    pexp_loc_stack = [];
    pexp_attributes =
@@ -546,10 +591,33 @@ let ocaml_pexp_override sel =
   let sel = List.map (fun (s, e) -> mknoloc s, e) sel in Pexp_override sel
 ;;
 
-let ocaml_pexp_pack : ('a -> 'b -> 'c, 'd) choice option =
-  Some (Right ((fun me -> Pexp_pack me), (fun pt -> Ptyp_package pt)))
+let ocaml_ptyp_pack pt = Ptyp_package pt;;
+
+let ocaml_pexp_pack loc me pto =
+  let e = Pexp_pack me in
+  if pto = None then e
+  else
+    let (pto : Parsetree.core_type_desc option) =
+      option_map ocaml_ptyp_pack pto
+    in
+    let (pto : Parsetree.core_type option) =
+      option_map (ocaml_mktyp loc) pto
+    in
+    ocaml_pexp_constraint (ocaml_mkexp loc e) pto None
 ;;
 
+let ocaml_ptyp_tuple x =
+  let x =
+    List.map
+      (function
+         None, t -> t
+       | Some _, t ->
+           failwith
+             "Ptyp_tuple: labeled tuples only available with ocaml >= 5.04")
+      x
+  in
+  Ptyp_tuple x
+;;
 let ocaml_pexp_poly = Some (fun e t -> Pexp_poly (e, t));;
 
 let ocaml_pexp_record lel eo =
@@ -610,7 +678,6 @@ let ocaml_ppat_alias p i iloc = Ppat_alias (p, mkloc iloc i);;
 let ocaml_ppat_array = Some (fun pl -> Ppat_array pl);;
 
 let ocaml_ppat_construct loc li po chk_arity =
-  let po = option_map (fun (_, p) -> p) po in
   Ppat_construct (mkloc loc li, po)
 ;;
 
@@ -620,8 +687,24 @@ let ocaml_ppat_construct_args =
   | _ -> None
 ;;
 
+let ocaml_ppat_tuple l cflag =
+  let l =
+    List.map
+      (function
+         None, p -> p
+       | Some _, _ ->
+           failwith
+             "Ppat_tuple: labeled tuples only available at ocaml >= 5.4")
+      l
+  in
+  if cflag = Open then
+    failwith "Ppat_tuple: labeled tuples only available at ocaml >= 5.4"
+  else Ppat_tuple l
+;;
+
 let mkpat_ocaml_ppat_construct_arity loc li_loc li tyvl al =
-  let a = ocaml_mkpat loc (Ppat_tuple al) in
+  let al = List.map (fun x -> None, x) al in
+  let a = ocaml_mkpat loc (ocaml_ppat_tuple al Closed) in
   {ppat_desc = ocaml_ppat_construct li_loc li (Some (tyvl, a)) true;
    ppat_loc = loc; ppat_loc_stack = [];
    ppat_attributes =
@@ -697,7 +780,11 @@ let ocaml_psig_modtype ?(item_attributes = []) loc s mto =
 ;;
 
 let ocaml_psig_modtypesubst ?(item_attributes = []) loc s mto =
-  failwith "no 'module type' substitution in this version of ocaml"
+  let pmtd =
+    {pmtd_name = mkloc loc s; pmtd_type = mto;
+     pmtd_attributes = item_attributes; pmtd_loc = loc}
+  in
+  Psig_modtypesubst pmtd
 ;;
 
 let ocaml_psig_open ?(item_attributes = []) loc li =
@@ -884,9 +971,7 @@ let ocaml_pcl_constr = Some (fun li ctl -> Pcl_constr (mknoloc li, ctl));;
 
 let ocaml_pcl_constraint = Some (fun ce ct -> Pcl_constraint (ce, ct));;
 
-let ocaml_pcl_fun =
-  Some (fun lab ceo p ce -> Pcl_fun (labelled lab, ceo, p, ce))
-;;
+let ocaml_pcl_fun lab ceo p ce = Pcl_fun (labelled lab, ceo, p, ce);;
 
 let ocaml_pcl_let = Some (fun rf pel ce -> Pcl_let (rf, pel, ce));;
 
@@ -949,12 +1034,12 @@ let ocaml_ptop_dir loc s da =
 
   let (ocaml_pwith_modtype :
  (Location.t -> Longident.t -> module_type -> with_constraint) option) =
-  None
+  Some (fun loc li mt -> Pwith_modtype (mkloc loc li, mt))
 ;;
 
 let (ocaml_pwith_modtypesubst :
  (Location.t -> Longident.t -> module_type -> with_constraint) option) =
-  None
+  Some (fun loc li mt -> Pwith_modtypesubst (mkloc loc li, mt))
 ;;
 
 let ocaml_pwith_modsubst =
