@@ -73,15 +73,22 @@ let mklazy loc e =
       let ghpat = mkpat loc in
       let ghexp = mkexp loc in
       let void_pat =
-        ghpat (ocaml_ppat_construct (mkloc loc) (Lident "()") None false)
+        ghpat
+          (ocaml_ppat_construct (mkloc loc)
+             (ocaml_longident_lident (mkloc loc) "()") None false)
       in
       let pwe = ocaml_case (void_pat, None, mkloc loc, e) in
       let f = ghexp (ocaml_pexp_function "" None [pwe]) in
-      let delayed = Ldot (Lident "Lazy", "Delayed") in
+      let delayed =
+        ocaml_longident_of_string_list (mkloc loc) ["Lazy"; "Delayed"]
+      in
       let cloc = mkloc loc in
       let df = ghexp (ocaml_pexp_construct cloc delayed (Some f) false) in
       let r =
-        ghexp (ocaml_pexp_ident cloc (Ldot (Lident "Pervasives", "ref")))
+        ghexp
+          (ocaml_pexp_ident cloc
+             (ocaml_longident_of_string_list (mkloc loc)
+                ["Pervasives"; "ref"]))
       in
       ghexp (ocaml_pexp_apply r ["", df])
 ;;
@@ -152,8 +159,9 @@ let conv_lab =
   fun s -> try Hashtbl.find t s with Not_found -> s
 ;;
 
-let array_function str name =
-  Ldot (Lident str, (if !fast then "unsafe_" ^ name else name))
+let array_function loc str name =
+  ocaml_longident_of_string_list (mkloc loc)
+    [str; if !fast then "unsafe_" ^ name else name]
 ;;
 
 let uv c =
@@ -175,48 +183,35 @@ let uident_True__True =
   | x -> x
 ;;
 
-let mkli s =
-  let rec loop f =
-    function
-      i :: il -> loop (fun s -> Ldot (f i, s)) il
-    | [] -> f s
-  in
-  loop (fun s -> Lident s)
-;;
+let mkli loc s sl = ocaml_longident_of_string_list (mkloc loc) (s :: sl);;
 
-let long_id_of_string_list loc sl =
-  match List.rev sl with
+let longident_of_string_list loc sl =
+  match sl with
     [] -> error loc "bad ast"
-  | s :: sl -> mkli s (List.rev sl)
+  | _ -> ocaml_longident_of_string_list (mkloc loc) sl
 ;;
 
-let concat_long_ids l1 l2 =
-  let rec crec lhs =
-    function
-      Lident s -> Ldot (lhs, s)
-    | Ldot (li, s) -> Ldot (crec lhs li, s)
-    | Lapply (lif, liarg) -> Lapply (crec lhs lif, liarg)
-  in
-  crec l1 l2
-;;
-
-let longid_long_id_gen ~extended =
+let longid_to_longident_gen ~extended =
   let rec lrec =
     function
-      MLast.LiApp (_, me1, me2) when extended -> Lapply (lrec me1, lrec me2)
+      MLast.LiApp (loc, me1, me2) when extended ->
+        ocaml_longident_lapply (mkloc loc) (lrec me1) (lrec me2)
     | MLast.LiApp (loc, _, _) ->
         Ploc.raise loc
           (Failure
              "longid_long_id: extended longid forbidden here (application not allowed)")
-    | MLast.LiAcc (_, me1, uid) -> Ldot (lrec me1, uident_True__True (uv uid))
-    | MLast.LiUid (_, s) -> Lident (uident_True__True (uv s))
+    | MLast.LiAcc (loc, me1, uid) ->
+        ocaml_longident_ldot (mkloc loc) (lrec me1)
+          (uident_True__True (uv uid))
+    | MLast.LiUid (loc, s) ->
+        ocaml_longident_lident (mkloc loc) (uident_True__True (uv s))
     | LiXtr (loc, _, _) ->
         Ploc.raise loc (Failure "longid_long_id: LiXtr forbidden here")
   in
   lrec
 ;;
-let longid_long_id = longid_long_id_gen ~extended:false;;
-let extended_longid_long_id = longid_long_id_gen ~extended:true;;
+let longid_to_longident = longid_to_longident_gen ~extended:false;;
+let extended_longid_to_longident = longid_to_longident_gen ~extended:true;;
 
 let rec ctyp_fa al =
   function
@@ -224,62 +219,80 @@ let rec ctyp_fa al =
   | f -> f, al
 ;;
 
-let rec module_expr_long_id =
-  function
-    MLast.MeApp (_, me1, me2) ->
-      Lapply (module_expr_long_id me1, module_expr_long_id me2)
-  | MLast.MeAcc (_, m, MLast.MeUid (_, s)) -> Ldot (module_expr_long_id m, s)
-  | MLast.MeUid (_, s) -> Lident s
-  | t -> error (loc_of_module_expr t) "bad module expr long ident"
+let module_expr_to_extended_longid =
+  let rec mrec =
+    function
+      MLast.MeApp (loc, me1, me2) -> MLast.LiApp (loc, mrec me1, mrec me2)
+    | MLast.MeAcc (loc, m, MLast.MeUid (_, s)) -> MLast.LiAcc (loc, mrec m, s)
+    | MLast.MeUid (loc, s) -> MLast.LiUid (loc, s)
+    | t -> error (loc_of_module_expr t) "bad module expr long ident"
+  in
+  mrec
 ;;
 
-let rec expr_long_id =
+let module_expr_to_longident me =
+  (me |> module_expr_to_extended_longid) |> extended_longid_to_longident
+;;
+
+let rec expr_to_longident =
   function
-    MLast.ExLong (_, li) -> longid_long_id li
+    MLast.ExLong (_, li) -> longid_to_longident li
   | _ -> failwith "expr_long_id: unexpected expr"
 ;;
 
-let longid_lident_long_id_gen ~extended (lio, s) =
+let lilongid_to_longident_gen ~extended loc (lio, s) =
   match lio, s with
-    Some li, s -> Ldot (longid_long_id_gen ~extended:extended (uv li), uv s)
-  | None, s -> Lident (uv s)
+    Some li, s ->
+      ocaml_longident_ldot (mkloc loc)
+        (longid_to_longident_gen ~extended:extended (uv li)) (uv s)
+  | None, s -> ocaml_longident_lident (mkloc loc) (uv s)
 ;;
-let longid_lident_long_id = longid_lident_long_id_gen ~extended:false;;
-let extended_longid_lident_long_id =
-  longid_lident_long_id_gen ~extended:true
+let lilongid_to_longident loc =
+  lilongid_to_longident_gen ~extended:false loc
+;;
+let extended_lilongid_to_longident loc =
+  lilongid_to_longident_gen ~extended:true loc
 ;;
 
-let rec ctyp_long_id =
-  function
-    MLast.TyApp (_, m1, m2) ->
-      let (is_cls, li1) = ctyp_long_id m1 in
-      let (_, li2) = ctyp_long_id m2 in is_cls, Lapply (li1, li2)
-  | MLast.TyAcc (loc, m, id) ->
-      let li1 = extended_longid_long_id m in
-      let (is_cls, li2) = ctyp_long_id (MLast.TyLid (loc, id)) in
-      is_cls, concat_long_ids li1 li2
-  | MLast.TyLid (_, s) -> false, Lident s
-  | TyCls (loc, cli) -> true, extended_longid_lident_long_id (uv cli)
-  | t -> error (loc_of_ctyp t) "incorrect type"
+let ctyp_to_longident =
+  let rec crec =
+    function
+      MLast.TyApp (loc, m1, m2) ->
+        let (is_cls, li1) = crec m1 in
+        let (_, li2) = crec m2 in
+        is_cls, ocaml_longident_lapply (mkloc loc) li1 li2
+    | MLast.TyAcc (loc, m, id) ->
+        let li1 = extended_longid_to_longident m in
+        false, ocaml_longident_ldot (mkloc loc) li1 id
+    | MLast.TyLid (loc, s) -> false, ocaml_longident_lident (mkloc loc) s
+    | TyCls (loc, cli) -> true, extended_lilongid_to_longident loc (uv cli)
+    | t -> error (loc_of_ctyp t) "incorrect type"
+  in
+  crec
 ;;
 
-let module_type_long_id =
+let module_type_to_longident =
   function
-    MLast.MtLongLid (_, li, lid) -> Ldot (extended_longid_long_id li, uv lid)
-  | MLast.MtLong (_, li) -> extended_longid_long_id li
-  | _ -> failwith "module_type_long_id"
+    MLast.MtLongLid (loc, li, lid) ->
+      ocaml_longident_ldot (mkloc loc) (extended_longid_to_longident li)
+        (uv lid)
+  | MLast.MtLong (_, li) -> extended_longid_to_longident li
+  | _ -> failwith "module_type_to_longident"
 ;;
 
 let class_type_long_id =
   function
-    MLast.CtLongLid (_, li, lid) -> Ldot (extended_longid_long_id li, uv lid)
-  | MLast.CtLid (_, lid) -> Lident (uv lid)
+    MLast.CtLongLid (loc, li, lid) ->
+      ocaml_longident_ldot (mkloc loc) (extended_longid_to_longident li)
+        (uv lid)
+  | MLast.CtLid (loc, lid) -> ocaml_longident_lident (mkloc loc) (uv lid)
   | _ -> failwith "class_type_long_id"
 ;;
 
 let mktype ~item_attributes loc tn tl cl tk pf tm =
   let (params, variance) = List.split tl in
   let params = List.map uv params in
+  let variance = List.map uv variance in
   match
     ocaml_type_declaration tn params cl tk pf tm (mkloc loc) variance
       item_attributes
@@ -326,12 +339,12 @@ let rec mkrangepat loc c1 c2 =
           mkrangepat loc (Char.chr (Char.code c1 + 1)) c2))
 ;;
 
-let rec patt_label_long_id =
+let patt_label_long_id =
   function
-    MLast.PaPfx (_, li, MLast.PaLid (_, s)) ->
-      Ldot (longid_long_id li, conv_lab s)
-  | MLast.PaLong (_, MLast.LiAcc (_, li, s), []) -> longid_long_id li
-  | MLast.PaLid (_, s) -> Lident (conv_lab s)
+    MLast.PaPfx (loc, li, MLast.PaLid (_, s)) ->
+      ocaml_longident_ldot (mkloc loc) (longid_to_longident li) (conv_lab s)
+  | MLast.PaLong (loc, MLast.LiAcc (_, li, s), []) -> longid_to_longident li
+  | MLast.PaLid (loc, s) -> ocaml_longident_lident (mkloc loc) (conv_lab s)
   | p -> error (loc_of_patt p) "bad label"
 ;;
 
@@ -371,7 +384,7 @@ let class_info item_attributes_fun class_expr ci =
           class_infos ~item_attributes:(item_attributes_fun ci.ciAttributes)
             (if uv ci.ciVir then Virtual else Concrete)
             (params, mkloc (fst ci.ciPrm)) (uv ci.ciNam) (class_expr ci.ciExp)
-            (mkloc ci.ciLoc) variance
+            (mkloc ci.ciLoc) (List.map uv variance)
       | None -> error ci.ciLoc "no '_' type parameter allowed"
       end
   | None -> error ci.ciLoc "no class_info in this ocaml version"
@@ -536,11 +549,11 @@ let varify_constructors var_names =
     function
       MLast.TyArr (loc, t1, t2) -> MLast.TyArr (loc, loop t1, loop t2)
     | MLast.TyApp (loc, t1, t2) -> MLast.TyApp (loc, loop t1, loop t2)
-    | MLast.TyTup (loc, tl) -> MLast.TyTup (loc, List.map loop tl)
+    | MLast.TyTup (loc, tl) -> MLast.TyTup (loc, List.map labloop tl)
     | MLast.TyLid (loc, s) as t ->
         if List.mem s var_names then MLast.TyQuo (loc, "&" ^ s) else t
     | t -> t
-  in
+  and labloop (lab, t) = lab, loop t in
   loop
 ;;
 
@@ -557,15 +570,21 @@ let ctyp_mentions s cty =
       MLast.TyQuo (_, s2) -> s = s2
     | MLast.TyApp (_, t1, t2) -> crec t1 || crec t2
     | MLast.TyArr (_, t1, t2) -> crec t1 || crec t2
-    | MLast.TyTup (_, tl) -> List.exists crec tl
+    | MLast.TyTup (_, tl) -> List.exists crec (List.map snd tl)
     | _ -> false
   in
   crec cty
 ;;
 
+let add_empty_labels l = List.map (fun x -> None, x) l;;
+let strip_empty_labels loc l =
+  assert (List.for_all (fun (lab, _) -> None = uv lab) l); List.map snd l
+;;
+
 let rec type_decl_of_with_type loc tn tpl pf ct =
   let (params, variance) = List.split (uv tpl) in
   let params = List.map uv params in
+  let variance = List.map uv variance in
   let ct = Some (ctyp ct) in
   let tk = if pf then ocaml_ptype_abstract else Ptype_abstract in
   let pf = if pf then Private else Public in
@@ -573,31 +592,31 @@ let rec type_decl_of_with_type loc tn tpl pf ct =
 and mkwithc =
   function
     WcMod (loc, id, m) ->
-      let mname = longid_long_id (uv id) in
-      mname, ocaml_pwith_module (mkloc loc) mname (module_expr_long_id m)
+      let mname = longid_to_longident (uv id) in
+      mname, ocaml_pwith_module (mkloc loc) mname (module_expr_to_longident m)
   | WcMos (loc, id, m) ->
       begin match ocaml_pwith_modsubst with
         Some pwith_modsubst ->
-          let li = longid_long_id (uv id) in
-          li, pwith_modsubst (mkloc loc) li (module_expr_long_id m)
+          let li = longid_to_longident (uv id) in
+          li, pwith_modsubst (mkloc loc) li (module_expr_to_longident m)
       | None -> error loc "no with module := in this ocaml version"
       end
   | WcMty (loc, id, mty) ->
       begin match ocaml_pwith_modtype with
         Some pwith_modtype ->
-          let mname = longid_long_id (uv id) in
+          let mname = longid_to_longident (uv id) in
           mname, pwith_modtype (mkloc loc) mname (module_type mty)
       | None -> error loc "no with module := in this ocaml version"
       end
   | WcMts (loc, id, mty) ->
       begin match ocaml_pwith_modtypesubst with
         Some pwith_modtypesubst ->
-          let li = longid_long_id (uv id) in
+          let li = longid_to_longident (uv id) in
           li, pwith_modtypesubst (mkloc loc) li (module_type mty)
       | None -> error loc "no with module type := in this ocaml version"
       end
   | WcTyp (loc, id, tpl, pf, ct) ->
-      let li = longid_lident_long_id (uv id) in
+      let li = lilongid_to_longident loc (uv id) in
       let (_, tname) = uv id in
       begin match type_decl_of_with_type loc (uv tname) tpl (uv pf) ct with
         Right td -> li, ocaml_pwith_type (mkloc loc) (li, td)
@@ -610,7 +629,7 @@ and mkwithc =
           let (_, last) = ids in
           begin match type_decl_of_with_type loc (uv last) tpl false t with
             Right td ->
-              let li = longid_lident_long_id ids in
+              let li = lilongid_to_longident loc ids in
               li, pwith_typesubst (mkloc loc) li td
           | Left msg -> error loc msg
           end
@@ -652,7 +671,7 @@ and ctyp =
   function
     TyAtt (loc, ct, a) -> ocaml_coretype_addattr (attr (uv a)) (ctyp ct)
   | TyAcc (loc, _, _) as f ->
-      let (is_cls, li) = ctyp_long_id f in
+      let (is_cls, li) = ctyp_to_longident f in
       if is_cls then mktyp loc (ocaml_ptyp_class li [] [])
       else mktyp loc (ocaml_ptyp_constr (mkloc loc) li [])
   | TyAli (loc, t1, t2) ->
@@ -666,7 +685,7 @@ and ctyp =
   | TyAny loc -> mktyp loc Ptyp_any
   | TyApp (loc, _, _) as f ->
       let (f, al) = ctyp_fa [] f in
-      let (is_cls, li) = ctyp_long_id f in
+      let (is_cls, li) = ctyp_to_longident f in
       if is_cls then mktyp loc (ocaml_ptyp_class li (List.map ctyp al) [])
       else mktyp loc (ocaml_ptyp_constr (mkloc loc) li (List.map ctyp al))
   | TyArr (loc, TyLab (loc1, lab, t1), t2) ->
@@ -679,10 +698,12 @@ and ctyp =
         (ocaml_ptyp_object (mkloc loc) (meth_list loc (uv fl) v) (uv v))
   | TyCls (loc, cli) ->
       mktyp loc
-        (ocaml_ptyp_class (extended_longid_lident_long_id (uv cli)) [] [])
+        (ocaml_ptyp_class (extended_lilongid_to_longident loc (uv cli)) [] [])
   | TyLab (loc, _, _) -> error loc "labeled type not allowed here"
   | TyLid (loc, s) ->
-      mktyp loc (ocaml_ptyp_constr (mkloc loc) (Lident (uv s)) [])
+      mktyp loc
+        (ocaml_ptyp_constr (mkloc loc)
+           (ocaml_longident_lident (mkloc loc) (uv s)) [])
   | TyMan (loc, _, _, _) -> error loc "type manifest not allowed here"
   | TyOlb (loc, lab, _) -> error loc "labeled type not allowed here"
   | TyOpn loc -> error loc "open (parsed as '..') type not allowed here"
@@ -705,7 +726,11 @@ and ctyp =
   | TyQuo (loc, s) -> mktyp loc (Ptyp_var (uv s))
   | TyRec (loc, _) -> error loc "record type not allowed here"
   | TySum (loc, _) -> error loc "sum type not allowed here"
-  | TyTup (loc, tl) -> mktyp loc (Ptyp_tuple (List.map ctyp (uv tl)))
+  | TyTup (loc, tl) ->
+      mktyp loc
+        (ocaml_ptyp_tuple
+           (List.map (fun (lab, t) -> (lab |> uv) |> option_map uv, ctyp t)
+              (uv tl)))
   | TyVrn (loc, catl, ool) ->
       let catl =
         List.map
@@ -733,7 +758,7 @@ and ctyp =
   | TyExten (loc, ebody) ->
       mktyp loc (ocaml_ptyp_extension (extension (uv ebody)))
   | TyOpen (loc, li, t) ->
-      let li = longid_long_id li in
+      let li = longid_to_longident li in
       mktyp loc (ocaml_ptyp_open (mkloc loc) li (ctyp t))
 and meth_list loc fl v =
   match fl with
@@ -765,7 +790,7 @@ and package_of_module_type loc mt =
           List.map
             (function
                WcTyp (loc, id, tpl, pf, ct) ->
-                 let id_or_li = longid_lident_long_id (uv id) in
+                 let id_or_li = lilongid_to_longident loc (uv id) in
                  if uv tpl <> [] then
                    error loc "no type parameters allowed here";
                  if uv pf then error loc "no 'private' allowed here";
@@ -781,7 +806,8 @@ and package_of_module_type loc mt =
         mt, with_con
     | _ -> mt, []
   in
-  let li = module_type_long_id mt in ocaml_package_type li with_con
+  let li = module_type_to_longident mt in
+  ocaml_package_type (mkloc loc) li with_con
 and type_decl ?(item_attributes = []) tn tl priv (cl, tdCon) =
   function
     TyMan (loc, t, pf, MLast.TyRec (_, ltl)) ->
@@ -831,19 +857,20 @@ and extension_constructor ec =
           ocaml_ec_record ~alg_attributes:alg_attrs (mkloc loc) n (x, y)
       end
   | EcRebind (loc, n, li, alg_attrs) ->
-      ocaml_ec_rebind (mkloc loc) (uv n) (longid_long_id (uv li))
+      ocaml_ec_rebind (mkloc loc) (uv n) (longid_to_longident (uv li))
 and type_extension loc te =
   let pf = uv te.tePrv in
   let ecstrs = List.map extension_constructor (uv te.teECs) in
   ocaml_type_extension ~item_attributes:(uv_item_attributes te.teAttributes)
-    (mkloc loc) (longid_lident_long_id (uv te.teNam))
-    (List.map (fun (p, v) -> uv p, v) (uv te.tePrm))
+    (mkloc loc) (lilongid_to_longident loc (uv te.teNam))
+    (List.map (fun (p, v) -> uv p, uv v) (uv te.tePrm))
     (if pf then Private else Public) ecstrs
 and patt =
   function
     PaAtt (loc, p1, a) -> ocaml_patt_addattr (attr (uv a)) (patt p1)
   | PaPfx (loc, li, p2) ->
-      mkpat loc (ocaml_ppat_open (mkloc loc) (longid_long_id li) (patt p2))
+      mkpat loc
+        (ocaml_ppat_open (mkloc loc) (longid_to_longident li) (patt p2))
   | PaLong (loc, li, _) ->
       let li =
         match li with
@@ -851,7 +878,7 @@ and patt =
         | MLast.LiAcc (loc, li, s) -> MLast.LiAcc (loc, li, conv_con s)
         | _ -> failwith "Lapply not allowed here"
       in
-      let li = longid_long_id li in
+      let li = longid_to_longident li in
       mkpat loc
         (ocaml_ppat_construct (mkloc loc) li None
            (not !(Prtools.no_constructors_arity)))
@@ -884,7 +911,9 @@ and patt =
                 let a =
                   match al with
                     [a] -> a
-                  | _ -> mkpat loc (Ppat_tuple al)
+                  | _ ->
+                      let al = add_empty_labels al in
+                      mkpat loc (ocaml_ppat_tuple al Closed)
                 in
                 mkpat loc
                   (ocaml_ppat_construct li_loc li (Some ([], a)) false)
@@ -898,7 +927,9 @@ and patt =
                       let a =
                         match al with
                           [a] -> a
-                        | _ -> mkpat loc (Ppat_tuple al)
+                        | _ ->
+                            let al = add_empty_labels al in
+                            mkpat loc (ocaml_ppat_tuple al Closed)
                       in
                       mkpat loc (ppat_variant (s, Some a))
                   | Some _ | None ->
@@ -926,7 +957,7 @@ and patt =
            (mkconst loc (pconst_of_const (mkintconst loc (uv s) c))))
   | PaFlo (loc, s) ->
       mkpat loc (Ppat_constant (mkconst loc (ocaml_pconst_float (uv s))))
-  | PaLab (loc, _) -> error loc "labeled pattern not allowed here"
+  | PaLab (loc, _, _) -> error loc "labeled pattern not allowed here"
   | PaLaz (loc, p) ->
       begin match ocaml_ppat_lazy with
         Some ppat_lazy -> mkpat loc (ppat_lazy (patt p))
@@ -959,13 +990,30 @@ and patt =
            (mkconst loc
               (ocaml_pconst_string (string_of_string_token loc (uv s))
                  (mkloc loc) None)))
-  | PaTup (loc, pl) -> mkpat loc (Ppat_tuple (List.map patt (uv pl)))
+  | PaTup (loc, pl, clflag) ->
+      let labeled_patt =
+        function
+          PaLab (ploc, (PaLid (_, s) as p), po) ->
+            let p =
+              match uv po with
+                Some p -> p
+              | None -> p
+            in
+            Some (uv s), patt p
+        | PaLab (ploc, (PaTyc (_, PaLid (_, s), ty) as p), None) ->
+            Some (uv s), patt p
+        | p -> None, patt p
+      in
+      let l = List.map labeled_patt (uv pl) in
+      let clflag = if uv clflag then Closed else Open in
+      mkpat loc (ocaml_ppat_tuple l clflag)
   | PaTyc (loc, p, t) -> mkpat loc (Ppat_constraint (patt p, ctyp t))
   | PaTyp (loc, lili) ->
       begin match ocaml_ppat_type with
         Some ppat_type ->
           mkpat loc
-            (ppat_type (mkloc loc) (extended_longid_lident_long_id (uv lili)))
+            (ppat_type (mkloc loc)
+               (extended_lilongid_to_longident loc (uv lili)))
       | None -> error loc "no #type in this ocaml version"
       end
   | PaUnp (loc, s, mto) ->
@@ -1008,12 +1056,12 @@ and expr =
         | MLast.LiAcc (loc, li, s) -> MLast.LiAcc (loc, li, conv_con s)
         | _ -> failwith "Lapply not allowed here"
       in
-      let li = longid_long_id li in
+      let li = longid_to_longident li in
       let cloc = mkloc loc in mkexp loc (ocaml_pexp_construct cloc li None ca)
   | ExOpen (loc, li, MLast.ExLid (_, id)) ->
       let vid = id in expr (ExFle (loc, ExLong (loc, li), (None, vid)))
   | ExOpen (loc, li, e) ->
-      let li = longid_long_id li in
+      let li = longid_to_longident li in
       let me = mkmod loc (ocaml_pmod_ident li) in
       begin match ocaml_pexp_open with
         Some pexp_open -> mkexp loc (pexp_open (mkoverride false) me (expr e))
@@ -1022,17 +1070,19 @@ and expr =
   | ExFle (loc, e, (None, s)) when uv s = "val" ->
       mkexp loc
         (ocaml_pexp_apply
-           (mkexp loc (ocaml_pexp_ident (mkloc loc) (Lident "!")))
+           (mkexp loc
+              (ocaml_pexp_ident (mkloc loc)
+                 (ocaml_longident_lident (mkloc loc) "!")))
            ["", expr e])
   | ExFle (loc, ExLong (_, li), (None, s)) ->
-      let li = longid_lident_long_id (Some li, s) in
+      let li = lilongid_to_longident loc (Some li, s) in
       mkexp loc (ocaml_pexp_ident (mkloc loc) li)
   | ExFle (loc, ExLong (_, li), (Some _, _)) ->
       error loc "<longid>.<longid> not valid syntax (nor parseable)"
   | ExFle (loc, e, (Some _, s)) when uv s = "val" ->
       error loc "().<lident>.val not valid syntax"
   | ExFle (loc, e, lili) ->
-      let li = longid_lident_long_id (uv lili) in
+      let li = lilongid_to_longident loc (uv lili) in
       mkexp loc (ocaml_pexp_field (mkloc loc) (expr e) li)
   | ExAnt (_, e) -> expr e
   | ExApp (loc, ExLid (_, "-"), ExInt (_, s, c)) ->
@@ -1054,7 +1104,7 @@ and expr =
             end
         | _ -> f
       in
-      let al = List.rev (List.fold_left label_expr [] al) in
+      let al = List.map label_expr al in
       begin match ocaml_pexp_construct_args (expr f).pexp_desc with
         Some (li, li_loc, None, _) ->
           let al = List.map snd al in
@@ -1062,7 +1112,7 @@ and expr =
             let a =
               match al with
                 [a] -> a
-              | _ -> mkexp loc (Pexp_tuple al)
+              | _ -> mkexp loc (ocaml_pexp_tuple (add_empty_labels al))
             in
             mkexp loc (ocaml_pexp_construct li_loc li (Some a) false)
           else mkexp_ocaml_pexp_construct_arity (mkloc loc) li_loc li al
@@ -1076,7 +1126,7 @@ and expr =
                   let a =
                     match al with
                       [a] -> a
-                    | _ -> mkexp loc (Pexp_tuple al)
+                    | _ -> mkexp loc (ocaml_pexp_tuple (add_empty_labels al))
                   in
                   mkexp loc (pexp_variant (s, Some a))
               | Some _ | None -> mkexp loc (ocaml_pexp_apply (expr f) al)
@@ -1090,7 +1140,7 @@ and expr =
           mkexp loc
             (ocaml_pexp_apply
                (mkexp loc
-                  (ocaml_pexp_ident cloc (array_function "Array" "get")))
+                  (ocaml_pexp_ident cloc (array_function loc "Array" "get")))
                ["", expr e1; "", expr e2])
       | ".", e2l -> assert false
       | dotop, [e2] ->
@@ -1112,7 +1162,9 @@ and expr =
           let cloc = mkloc loc in
           mkexp loc
             (ocaml_pexp_apply
-               (mkexp loc (ocaml_pexp_ident cloc (Lident ":=")))
+               (mkexp loc
+                  (ocaml_pexp_ident cloc
+                     (ocaml_longident_lident (mkloc loc) ":=")))
                ["", expr x; "", expr v])
       | ExFle (loc, _, _) ->
           begin match (expr e).pexp_desc with
@@ -1126,7 +1178,8 @@ and expr =
               mkexp loc
                 (ocaml_pexp_apply
                    (mkexp loc
-                      (ocaml_pexp_ident cloc (array_function "Array" "set")))
+                      (ocaml_pexp_ident cloc
+                         (array_function loc "Array" "set")))
                    ["", expr e1; "", expr e2; "", expr v])
           | ".", e2l -> assert false
           | dotop, [e2] ->
@@ -1181,7 +1234,7 @@ and expr =
                 (ocaml_pexp_apply
                    (mkexp loc
                       (ocaml_pexp_ident cloc
-                         (array_function bytes_modname "set")))
+                         (array_function loc bytes_modname "set")))
                    ["", expr e1; "", expr e2; "", expr v])
           | ".", e2l -> assert false
           | dotop, [e2] ->
@@ -1239,18 +1292,14 @@ and expr =
       mkexp loc (ocaml_pexp_for (patt i) (expr e1) (expr e2) df (expr e3))
   | ExFun (loc, pel) ->
       begin match uv pel with
-        [PaLab (ploc, lppo), w, e] ->
-          begin match uv lppo with
-            [p, po] ->
-              let lab = label_of_patt p in
-              let p =
-                match uv po with
-                  Some p -> p
-                | None -> p
-              in
-              mkexp loc (ocaml_pexp_function lab None [mkpwe (p, w, e)])
-          | _ -> error loc "bad AST"
-          end
+        [PaLab (ploc, p, po), w, e] ->
+          let lab = label_of_patt p in
+          let p =
+            match uv po with
+              Some p -> p
+            | None -> p
+          in
+          mkexp loc (ocaml_pexp_function lab None [mkpwe (p, w, e)])
       | [PaNty (loc, s), w, e] ->
           mkexp loc (ocaml_pexp_newtype (mkloc loc) (uv s) (expr e))
       | [PaOlb (loc, p, eo), w, e] ->
@@ -1282,7 +1331,7 @@ and expr =
       mkexp loc
         (Pexp_constant
            (mkconst loc (pconst_of_const (mkintconst loc (uv s) c))))
-  | ExLab (loc, _) -> error loc "labeled expression not allowed here 1"
+  | ExLab (loc, _, _) -> error loc "labeled expression not allowed here 1"
   | ExLaz (loc, e) -> mklazy loc (expr e)
   | ExLet (loc, rf, pel, e) ->
       mkexp loc (Pexp_let (mkrf (uv rf), List.map mkpe (uv pel), expr e))
@@ -1292,7 +1341,10 @@ and expr =
           (List.map ctyp (uv exntyl)) (uv_alg_attributes alg_attrs)
       in
       mkexp loc (ocaml_pexp_letexception exdef (expr body))
-  | ExLid (loc, s) -> mkexp loc (ocaml_pexp_ident (mkloc loc) (Lident (uv s)))
+  | ExLid (loc, s) ->
+      mkexp loc
+        (ocaml_pexp_ident (mkloc loc)
+           (ocaml_longident_lident (mkloc loc) (uv s)))
   | ExLmd (loc, i, me, e) ->
       begin match ocaml_pexp_letmodule with
         Some pexp_letmodule ->
@@ -1316,7 +1368,8 @@ and expr =
       in
       mkexp loc (Pexp_match (expr e, List.map mkpwe pel))
   | ExNew (loc, cli) ->
-      mkexp loc (ocaml_pexp_new (mkloc loc) (longid_lident_long_id (uv cli)))
+      mkexp loc
+        (ocaml_pexp_new (mkloc loc) (lilongid_to_longident loc (uv cli)))
   | ExObj (loc, po, cfl) ->
       begin match ocaml_pexp_object with
         Some pexp_object ->
@@ -1333,27 +1386,10 @@ and expr =
   | ExOvr (loc, iel) ->
       mkexp loc (ocaml_pexp_override (List.map mkideexp (uv iel)))
   | ExPck (loc, me, mto) ->
-      begin match ocaml_pexp_pack with
-        Some (Left pexp_pack) ->
-          begin match mto with
-            Some mt ->
-              let pt = package_of_module_type loc mt in
-              mkexp loc (pexp_pack (module_expr me) pt)
-          | None -> error loc "no such module pack in this ocaml version"
-          end
-      | Some (Right (pexp_pack, ptyp_package)) ->
-          let e = pexp_pack (module_expr me) in
-          let e =
-            match mto with
-              Some mt ->
-                let pt = package_of_module_type loc mt in
-                ocaml_pexp_constraint (mkexp loc e)
-                  (Some (mktyp loc (ptyp_package pt))) None
-            | None -> e
-          in
-          mkexp loc e
-      | None -> error loc "no module pack in this ocaml version"
-      end
+      let (pto : Parsetree.package_type option) =
+        option_map (package_of_module_type loc) mto
+      in
+      let e = ocaml_pexp_pack (mkloc loc) (module_expr me) pto in mkexp loc e
   | ExRec (loc, lel, eo) ->
       let lel = uv lel in
       if lel = [] then error loc "empty record"
@@ -1383,7 +1419,7 @@ and expr =
           mkexp loc
             (ocaml_pexp_apply
                (mkexp loc
-                  (ocaml_pexp_ident cloc (array_function "String" "get")))
+                  (ocaml_pexp_ident cloc (array_function loc "String" "get")))
                ["", expr e1; "", expr e2])
       | ".", e2l -> assert false
       | dotop, [e2] ->
@@ -1407,7 +1443,18 @@ and expr =
                  (mkloc loc) None)))
   | ExTry (loc, e, pel) ->
       mkexp loc (Pexp_try (expr e, List.map mkpwe (uv pel)))
-  | ExTup (loc, el) -> mkexp loc (Pexp_tuple (List.map expr (uv el)))
+  | ExTup (loc, el) ->
+      let el = List.map label_expr (uv el) in
+      let el =
+        List.map
+          (function
+             "", e -> None, e
+           | s, e when s.[0] = '?' ->
+               error loc "cannot use optional labels in tuple"
+           | s, e -> Some s, e)
+          el
+      in
+      mkexp loc (ocaml_pexp_tuple el)
   | ExTyc (loc, e, t) ->
       mkexp loc (ocaml_pexp_constraint (expr e) (Some (ctyp t)) None)
   | ExVrn (loc, s) ->
@@ -1424,21 +1471,16 @@ and expr =
   | ExUnr loc ->
       error loc
         "bad ast ExUnr (parses as '.'; cannot have an ExUnr except at the rhs of match-case)"
-and label_expr rev_al =
+and label_expr =
   function
-    ExLab (loc, lpeo) ->
-      List.fold_left
-        (fun rev_al (p, eo) ->
-           match p with
-             PaLid (loc, lab) ->
-               let e =
-                 match uv eo with
-                   Some e -> e
-                 | None -> ExLid (loc, lab)
-               in
-               (uv lab, expr e) :: rev_al
-           | _ -> error loc "ExLab case not impl")
-        rev_al (uv lpeo)
+    ExLab (loc, p, eo) ->
+      begin match p, uv eo with
+        PaLid (loc, lab), Some e -> uv lab, expr e
+      | PaLid (loc, lab), None -> uv lab, expr (ExLid (loc, lab))
+      | PaTyc (loc, PaLid (_, lab), ty), None ->
+          uv lab, expr (ExTyc (loc, ExLid (loc, lab), ty))
+      | _ -> error loc "ExLab case not impl"
+      end
   | ExOlb (loc, p, eo) ->
       begin match p with
         PaLid (loc, lab) ->
@@ -1447,10 +1489,10 @@ and label_expr rev_al =
               Some e -> e
             | None -> ExLid (loc, lab)
           in
-          ("?" ^ uv lab, expr e) :: rev_al
+          "?" ^ uv lab, expr e
       | _ -> error loc "ExOlb case not impl"
       end
-  | e -> ("", expr e) :: rev_al
+  | e -> "", expr e
 and mkpe (p, e, attrs) =
   let loc = Ploc.encl (loc_of_patt p) (loc_of_expr e) in
   (*
@@ -1514,10 +1556,13 @@ and module_type =
   function
     MtAtt (loc, e, a) -> ocaml_pmty_addattr (attr (uv a)) (module_type e)
   | MtLongLid (loc, _, _) as f ->
-      mkmty loc (ocaml_pmty_ident (mkloc loc) (module_type_long_id f))
+      mkmty loc (ocaml_pmty_ident (mkloc loc) (module_type_to_longident f))
   | MtLong (loc, _) as f ->
-      mkmty loc (ocaml_pmty_ident (mkloc loc) (module_type_long_id f))
-  | MtLid (loc, s) -> mkmty loc (ocaml_pmty_ident (mkloc loc) (Lident (uv s)))
+      mkmty loc (ocaml_pmty_ident (mkloc loc) (module_type_to_longident f))
+  | MtLid (loc, s) ->
+      mkmty loc
+        (ocaml_pmty_ident (mkloc loc)
+           (ocaml_longident_lident (mkloc loc) (uv s)))
   | MtFun (loc, arg, mt) ->
       let arg =
         option_map
@@ -1621,7 +1666,7 @@ and sig_item s l =
            mto) ::
       l
   | SgMtyAlias (loc, n, li, item_attrs) ->
-      let li = longid_long_id (uv li) in
+      let li = longid_to_longident (uv li) in
       let mty = mkmty loc (ocaml_pmty_alias (mkloc loc) li) in
       let m =
         ocaml_psig_module ~item_attributes:(uv_item_attributes item_attrs)
@@ -1629,14 +1674,14 @@ and sig_item s l =
       in
       mksig loc m :: l
   | SgModSubst (loc, s, li, item_attrs) ->
-      let li = longid_long_id li in
+      let li = longid_to_longident li in
       let attrs = uv_item_attributes item_attrs in
       let m =
         ocaml_psig_modsubst ~item_attributes:attrs (mkloc loc) (uv s) li
       in
       mksig loc m :: l
   | SgOpn (loc, lid, attrs) ->
-      let lid = extended_longid_long_id lid in
+      let lid = extended_longid_to_longident lid in
       mksig loc
         (ocaml_psig_open ~item_attributes:(uv_item_attributes attrs)
            (mkloc loc) lid) ::
@@ -1675,7 +1720,7 @@ and module_expr =
   function
     MeAtt (loc, e, a) -> ocaml_pmod_addattr (attr (uv a)) (module_expr e)
   | MeAcc (loc, _, _) as f ->
-      mkmod loc (ocaml_pmod_ident (module_expr_long_id f))
+      mkmod loc (ocaml_pmod_ident (module_expr_to_longident f))
   | MeApp (loc, me1, me2) ->
       mkmod loc (Pmod_apply (module_expr me1, module_expr me2))
   | MeFun (loc, arg, me) ->
@@ -1689,7 +1734,8 @@ and module_expr =
       mkmod loc (Pmod_structure (List.fold_right str_item (uv sl) []))
   | MeTyc (loc, me, mt) ->
       mkmod loc (Pmod_constraint (module_expr me, module_type mt))
-  | MeUid (loc, s) -> mkmod loc (ocaml_pmod_ident (Lident (uv s)))
+  | MeUid (loc, s) ->
+      mkmod loc (ocaml_pmod_ident (ocaml_longident_lident (mkloc loc) (uv s)))
   | MeUnp (loc, e, mto1, mto2) ->
       begin match ocaml_pmod_unpack with
         Some (Left pmod_unpack) ->
@@ -1756,7 +1802,8 @@ and str_item s l =
           let si =
             match ocaml_pstr_exn_rebind with
               Some pstr_exn_rebind ->
-                pstr_exn_rebind (mkloc loc) (uv n) (longid_long_id (uv li))
+                pstr_exn_rebind (mkloc loc) (uv n)
+                  (longid_to_longident (uv li))
             | None -> error loc "no exception renaming in this ocaml version"
           in
           mkstr loc si :: l
@@ -1856,7 +1903,7 @@ and class_type =
       | None -> error loc "no class type desc in this ocaml version"
       end
   | CtLop (loc, ovf, me, ct) ->
-      let li = longid_long_id me in
+      let li = longid_to_longident me in
       mkcty loc
         (ocaml_pcty_open (mkloc loc) li (mkoverride (uv ovf)) (class_type ct))
   | CtCon (loc, ct, tl) ->
@@ -1955,7 +2002,7 @@ and class_expr =
       begin match ocaml_pcl_apply with
         Some pcl_apply ->
           let (ce, el) = class_expr_fa [] c in
-          let el = List.rev (List.fold_left label_expr [] el) in
+          let el = List.map label_expr el in
           mkpcl loc (pcl_apply (class_expr ce) el)
       | None -> error loc "no class expr desc in this ocaml version"
       end
@@ -1963,45 +2010,30 @@ and class_expr =
       begin match ocaml_pcl_constr with
         Some pcl_constr ->
           mkpcl loc
-            (pcl_constr (longid_lident_long_id (uv cli))
+            (pcl_constr (lilongid_to_longident loc (uv cli))
                (List.map ctyp (uv tl)))
       | None -> error loc "no class expr desc in this ocaml version"
       end
-  | CeFun (loc, PaLab (ploc, lppo), ce) ->
-      begin match ocaml_pcl_fun with
-        Some pcl_fun ->
-          begin match uv lppo with
-            [p, po] ->
-              let lab = label_of_patt p in
-              let p =
-                match uv po with
-                  Some p -> p
-                | None -> p
-              in
-              mkpcl loc (pcl_fun lab None (patt p) (class_expr ce))
-          | [] | _ :: _ -> error ploc "case class multi lab not yet impl"
-          end
-      | None -> error loc "no class expr desc in this ocaml version"
-      end
+  | CeFun (loc, PaLab (ploc, p, po), ce) ->
+      let lab = label_of_patt p in
+      let p =
+        match uv po with
+          Some p -> p
+        | None -> p
+      in
+      mkpcl loc (ocaml_pcl_fun lab None (patt p) (class_expr ce))
   | CeFun (loc, PaOlb (_, p, eo), ce) ->
-      begin match ocaml_pcl_fun with
-        Some pcl_fun ->
-          let lab = label_of_patt p in
-          let (p, eo) =
-            match uv eo with
-              Some (ExOlb (_, p, eo)) -> p, eo
-            | Some _ | None -> p, eo
-          in
-          mkpcl loc
-            (pcl_fun ("?" ^ lab) (option_map expr (uv eo)) (patt p)
-               (class_expr ce))
-      | None -> error loc "no class expr desc in this ocaml version"
-      end
+      let lab = label_of_patt p in
+      let (p, eo) =
+        match uv eo with
+          Some (ExOlb (_, p, eo)) -> p, eo
+        | Some _ | None -> p, eo
+      in
+      mkpcl loc
+        (ocaml_pcl_fun ("?" ^ lab) (option_map expr (uv eo)) (patt p)
+           (class_expr ce))
   | CeFun (loc, p, ce) ->
-      begin match ocaml_pcl_fun with
-        Some pcl_fun -> mkpcl loc (pcl_fun "" None (patt p) (class_expr ce))
-      | None -> error loc "no class expr desc in this ocaml version"
-      end
+      mkpcl loc (ocaml_pcl_fun "" None (patt p) (class_expr ce))
   | CeLet (loc, rf, pel, ce) ->
       begin match ocaml_pcl_let with
         Some pcl_let ->
@@ -2010,7 +2042,7 @@ and class_expr =
       | None -> error loc "no class expr desc in this ocaml version"
       end
   | CeLop (loc, ovf, me, ce) ->
-      let li = longid_long_id me in
+      let li = longid_to_longident me in
       mkpcl loc
         (ocaml_pcl_open (mkloc loc) li (mkoverride (uv ovf)) (class_expr ce))
   | CeStr (loc, po, cfl) ->
@@ -2156,10 +2188,10 @@ let directive loc =
         Some pdir_bool -> pdir_bool false
       | None -> error loc "no such kind of directive in this ocaml version"
       end
-  | MLast.ExLong (_, li) -> Pdir_ident (longid_long_id li)
+  | MLast.ExLong (_, li) -> Pdir_ident (longid_to_longident li)
   | MLast.ExFle (_, MLast.ExLong (_, li), (None, id)) ->
-      Pdir_ident (longid_lident_long_id (Some li, id))
-  | MLast.ExLid (_, id) -> Pdir_ident (Lident id)
+      Pdir_ident (lilongid_to_longident loc (Some li, id))
+  | MLast.ExLid (_, id) -> Pdir_ident (ocaml_longident_lident (mkloc loc) id)
   | e ->
       let sl =
         let rec loop =
@@ -2170,7 +2202,7 @@ let directive loc =
         in
         loop e
       in
-      Pdir_ident (long_id_of_string_list loc sl)
+      Pdir_ident (longident_of_string_list loc sl)
 ;;
 
 let directive_args loc d =
