@@ -5,22 +5,24 @@
 (* expand parser ast into normal one *)
 
 type spat_comp =
-    SpTrm of MLast.loc * MLast.patt * MLast.expr option MLast.v
-  | SpNtr of MLast.loc * MLast.patt * MLast.expr
-  | SpLet of MLast.loc * MLast.patt * MLast.expr
-  | SpLhd of MLast.loc * MLast.patt list list
-  | SpStr of MLast.loc * MLast.patt
+  Exparser_types.spat_comp =
+      SpTrm of MLast.loc * MLast.patt * MLast.expr option MLast.v
+    | SpNtr of MLast.loc * MLast.patt * MLast.expr
+    | SpLet of MLast.loc * MLast.patt * MLast.expr
+    | SpLhd of MLast.loc * MLast.patt list list * MLast.expr option MLast.v
+    | SpStr of MLast.loc * MLast.patt
 ;;
-
 type sexp_comp =
-    SeTrm of MLast.loc * MLast.expr
-  | SeNtr of MLast.loc * MLast.expr
+  Exparser_types.sexp_comp =
+      SeTrm of MLast.loc * MLast.expr
+    | SeNtr of MLast.loc * MLast.expr
 ;;
 
 type spat_comp_opt =
-    SpoNoth
-  | SpoBang
-  | SpoQues of MLast.expr
+  Exparser_types.spat_comp_opt =
+      SpoNoth
+    | SpoBang
+    | SpoQues of MLast.expr
 ;;
 
 type spat_parser_ast =
