@@ -17,7 +17,7 @@ type loc = Ploc.t;;
 
 type 'a v = 'a;;
 
-type type_var = string option * (bool option * bool);;
+type type_var = string option * string;;
 
 type 'a class_infos =
   { ciLoc : loc;
@@ -50,7 +50,7 @@ and ctyp =
   | TyQuo of loc * string
   | TyRec of loc * (loc * string * bool * ctyp * attributes) list
   | TySum of loc * generic_constructor list
-  | TyTup of loc * ctyp list
+  | TyTup of loc * (string option * ctyp) list
   | TyVrn of loc * poly_variant list * string list option option
   | TyXtr of loc * string * ctyp option
   | TyAtt of loc * ctyp * attribute
@@ -72,7 +72,7 @@ and patt =
   | PaEff of loc * patt * patt
   | PaFlo of loc * string
   | PaInt of loc * string * string
-  | PaLab of loc * (patt * patt option) list
+  | PaLab of loc * patt * patt option
   | PaLaz of loc * patt
   | PaLid of loc * string
   | PaNty of loc * string
@@ -81,7 +81,7 @@ and patt =
   | PaRec of loc * (patt * patt) list
   | PaRng of loc * patt * patt
   | PaStr of loc * string
-  | PaTup of loc * patt list
+  | PaTup of loc * patt list * bool
   | PaTyc of loc * patt * ctyp
   | PaTyp of loc * longid_lident
   | PaUnp of loc * string option * module_type option
@@ -107,7 +107,7 @@ and expr =
   | ExFun of loc * case_branch list
   | ExIfe of loc * expr * expr * expr
   | ExInt of loc * string * string
-  | ExLab of loc * (patt * expr option) list
+  | ExLab of loc * patt * expr option
   | ExLaz of loc * expr
   | ExLet of loc * bool * (patt * expr * attributes) list * expr
   | ExLEx of loc * string * ctyp list * expr * attributes
