@@ -1,6 +1,6 @@
 (* camlp5r *)
 (* prtools.mli,v *)
-(* Copyright (c) INRIA 2007-2014 *)
+(* Copyright (c) INRIA 2007-2017 *)
 
 type pr_context =
   Pprintf.pr_context =
@@ -89,10 +89,6 @@ val expand_module_prefix :
 val do_split_or_patterns_with_bindings :
   (MLast.patt * 'a * 'b) list -> (MLast.patt * 'a * 'b) list;;
 
-val record_without_with :
-  Ploc.t -> MLast.expr -> (MLast.patt * MLast.expr) list ->
-    MLast.expr option;;
-
 val no_constructors_arity : bool ref;;
    (** Flag to generate nodes telling that constructor arity is not taken
        into account in the AST (e.g. True for normal syntax, False for
