@@ -94,7 +94,10 @@ let rec ctyp floc sh =
         let loc = floc loc in
         TySum (loc, vala_map (List.map (generic_constructor floc sh)) x1)
     | TyTup (loc, x1) ->
-        let loc = floc loc in TyTup (loc, vala_map (List.map self) x1)
+        let loc = floc loc in
+        TyTup
+          (loc,
+           vala_map (List.map (fun (a, b) -> v_option_v_string a, self b)) x1)
     | TyVrn (loc, x1, x2) ->
         let loc = floc loc in
         TyVrn (loc, vala_map (List.map (poly_variant floc sh)) x1, x2)
@@ -106,6 +109,7 @@ let rec ctyp floc sh =
         let loc = floc loc in TyOpen (loc, longid floc sh li, ctyp floc sh t)
   in
   self
+and v_option_v_string x = vala_map (option_map (vala_map (fun x -> x))) x
 and generic_constructor floc sh (loc, x1, x2, x3, x4, x5) =
   floc loc, x1, x2, vala_map (List.map (ctyp floc sh)) x3,
   vala_map (option_map (ctyp floc sh)) x4, attributes floc sh x5
@@ -145,14 +149,9 @@ and patt floc sh =
         let loc = floc loc in PaEff (loc, self x1, self x2)
     | PaFlo (loc, x1) -> let loc = floc loc in PaFlo (loc, x1)
     | PaInt (loc, x1, x2) -> let loc = floc loc in PaInt (loc, x1, x2)
-    | PaLab (loc, x1) ->
+    | PaLab (loc, x1, x2) ->
         let loc = floc loc in
-        PaLab
-          (loc,
-           vala_map
-             (List.map
-                (fun (x1, x2) -> self x1, vala_map (option_map self) x2))
-             x1)
+        PaLab (loc, self x1, vala_map (option_map self) x2)
     | PaLaz (loc, x1) -> let loc = floc loc in PaLaz (loc, self x1)
     | PaLid (loc, x1) -> let loc = floc loc in PaLid (loc, x1)
     | PaNty (loc, x1) -> let loc = floc loc in PaNty (loc, x1)
@@ -167,8 +166,8 @@ and patt floc sh =
     | PaRng (loc, x1, x2) ->
         let loc = floc loc in PaRng (loc, self x1, self x2)
     | PaStr (loc, x1) -> let loc = floc loc in PaStr (loc, x1)
-    | PaTup (loc, x1) ->
-        let loc = floc loc in PaTup (loc, vala_map (List.map self) x1)
+    | PaTup (loc, x1, x2) ->
+        let loc = floc loc in PaTup (loc, vala_map (List.map self) x1, x2)
     | PaTyc (loc, x1, x2) ->
         let loc = floc loc in PaTyc (loc, self x1, ctyp floc sh x2)
     | PaTyp (loc, x1) ->
@@ -233,15 +232,9 @@ and expr floc sh =
     | ExIfe (loc, x1, x2, x3) ->
         let loc = floc loc in ExIfe (loc, self x1, self x2, self x3)
     | ExInt (loc, x1, x2) -> let loc = floc loc in ExInt (loc, x1, x2)
-    | ExLab (loc, x1) ->
+    | ExLab (loc, x1, x2) ->
         let loc = floc loc in
-        ExLab
-          (loc,
-           vala_map
-             (List.map
-                (fun (x1, x2) ->
-                   patt floc sh x1, vala_map (option_map self) x2))
-             x1)
+        ExLab (loc, patt floc sh x1, vala_map (option_map self) x2)
     | ExLaz (loc, x1) -> let loc = floc loc in ExLaz (loc, self x1)
     | ExLet (loc, x1, x2, x3) ->
         let loc = floc loc in
