@@ -156,6 +156,14 @@ let make_any loc norec sl cl errk =
   let s = SpTrm (loc, p, None), errk in s :: sl, cl
 ;;
 
+let make_named c wo loc norec sl cl errk =
+  let (p, cl) =
+    if norec then MLast.PaLid (loc, c), cl
+    else MLast.PaLid (loc, c), MLast.ExLid (loc, c) :: cl
+  in
+  let s = SpTrm (loc, p, wo), errk in s :: sl, cl
+;;
+
 let fold_string_chars f s a =
   let rec loop i a =
     if i = String.length s then a
@@ -199,8 +207,8 @@ let make_sub_lexer loc f sl cl errk =
   s :: sl, []
 ;;
 
-let make_lookahd loc pll sl cl errk =
-  let s = SpLhd (loc, pll), errk in s :: sl, cl
+let make_lookahd loc pll wo sl cl errk =
+  let s = SpLhd (loc, pll, wo), errk in s :: sl, cl
 ;;
 
 let gcl = ref [];;
@@ -211,7 +219,9 @@ Grammar.safe_extend
    let grammar_entry_create s =
      Grammar.create_local_entry (Grammar.of_entry expr) s
    in
-   let rules : 'rules Grammar.Entry.e = grammar_entry_create "rules"
+   let string_or_rawstring : 'string_or_rawstring Grammar.Entry.e =
+     grammar_entry_create "string_or_rawstring"
+   and rules : 'rules Grammar.Entry.e = grammar_entry_create "rules"
    and rule : 'rule Grammar.Entry.e = grammar_entry_create "rule"
    and symb_list : 'symb_list Grammar.Entry.e =
      grammar_entry_create "symb_list"
@@ -266,6 +276,18 @@ Grammar.safe_extend
                MLast.ExFun
                  (loc, [MLast.PaLid (loc, var ()), None, mk_lexer loc rl]) :
                'expr)))]];
+    Grammar.extension
+      (string_or_rawstring : 'string_or_rawstring Grammar.Entry.e) None
+      [None, None,
+       [Grammar.production
+          (Grammar.r_next Grammar.r_stop (Grammar.s_token ("RAWSTRING", "")),
+           "194fe98d",
+           (fun (s : string) (loc : Ploc.t) ->
+              (snd (Asttools.split_rawstring s) : 'string_or_rawstring)));
+        Grammar.production
+          (Grammar.r_next Grammar.r_stop (Grammar.s_token ("STRING", "")),
+           "194fe98d",
+           (fun (s : string) (loc : Ploc.t) -> (s : 'string_or_rawstring)))]];
     Grammar.extension (expr : 'expr Grammar.Entry.e)
       (Some (Gramext.Level "simple"))
       [None, None,
@@ -308,9 +330,10 @@ Grammar.safe_extend
              (Grammar.r_next
                 (Grammar.r_next Grammar.r_stop (Grammar.s_token ("", "$")))
                 (Grammar.s_token ("LIDENT", "add")))
-             (Grammar.s_token ("STRING", "")),
+             (Grammar.s_nterm
+                (string_or_rawstring : 'string_or_rawstring Grammar.Entry.e)),
            "194fe98d",
-           (fun (s : string) _ _ (loc : Ploc.t) ->
+           (fun (s : 'string_or_rawstring) _ _ (loc : Ploc.t) ->
               (let s = Plexing.eval_string loc s in
                let rec loop v i =
                  if i = String.length s then v
@@ -375,16 +398,29 @@ Grammar.safe_extend
           (Grammar.r_next
              (Grammar.r_next
                 (Grammar.r_next
-                   (Grammar.r_next Grammar.r_stop
-                      (Grammar.s_token ("", "?=")))
-                   (Grammar.s_token ("", "[")))
-                (Grammar.s_list1sep
-                   (Grammar.s_nterm (lookahead : 'lookahead Grammar.Entry.e))
-                   (Grammar.s_token ("", "|")) false))
-             (Grammar.s_token ("", "]")),
+                   (Grammar.r_next
+                      (Grammar.r_next Grammar.r_stop
+                         (Grammar.s_token ("", "?=")))
+                      (Grammar.s_token ("", "[")))
+                   (Grammar.s_list1sep
+                      (Grammar.s_nterm
+                         (lookahead : 'lookahead Grammar.Entry.e))
+                      (Grammar.s_token ("", "|")) false))
+                (Grammar.s_token ("", "]")))
+             (Grammar.s_opt
+                (Grammar.s_rules
+                   [Grammar.production
+                      (Grammar.r_next
+                         (Grammar.r_next Grammar.r_stop
+                            (Grammar.s_token ("", "when")))
+                         (Grammar.s_nterml (expr : 'expr Grammar.Entry.e)
+                            "simple"),
+                       "194fe98d",
+                       (fun (e : 'expr) _ (loc : Ploc.t) -> (e : 'e__2)))])),
            "194fe98d",
-           (fun _ (pll : 'lookahead list) _ _ (loc : Ploc.t) ->
-              (make_lookahd loc pll : 'symb)));
+           (fun (eo : 'e__2 option) _ (pll : 'lookahead list) _ _
+                (loc : Ploc.t) ->
+              (make_lookahd loc pll eo : 'symb)));
         Grammar.production
           (Grammar.r_next Grammar.r_stop
              (Grammar.s_nterm (simple_expr : 'simple_expr Grammar.Entry.e)),
@@ -413,11 +449,39 @@ Grammar.safe_extend
               (make_char loc c norec : 'symb)));
         Grammar.production
           (Grammar.r_next
-             (Grammar.r_next Grammar.r_stop (Grammar.s_token ("STRING", "")))
+             (Grammar.r_next Grammar.r_stop
+                (Grammar.s_nterm
+                   (string_or_rawstring :
+                    'string_or_rawstring Grammar.Entry.e)))
              (Grammar.s_nterm (no_rec : 'no_rec Grammar.Entry.e)),
            "194fe98d",
-           (fun (norec : 'no_rec) (s : string) (loc : Ploc.t) ->
+           (fun (norec : 'no_rec) (s : 'string_or_rawstring) (loc : Ploc.t) ->
               (make_chars loc s norec : 'symb)));
+        Grammar.production
+          (Grammar.r_next
+             (Grammar.r_next
+                (Grammar.r_next
+                   (Grammar.r_next
+                      (Grammar.r_next Grammar.r_stop
+                         (Grammar.s_token ("", "_")))
+                      (Grammar.s_token ("", "as")))
+                   (Grammar.s_token ("LIDENT", "")))
+                (Grammar.s_opt
+                   (Grammar.s_rules
+                      [Grammar.production
+                         (Grammar.r_next
+                            (Grammar.r_next Grammar.r_stop
+                               (Grammar.s_token ("", "when")))
+                            (Grammar.s_nterml (expr : 'expr Grammar.Entry.e)
+                               "simple"),
+                          "194fe98d",
+                          (fun (e : 'expr) _ (loc : Ploc.t) ->
+                             (e : 'e__1)))])))
+             (Grammar.s_nterm (no_rec : 'no_rec Grammar.Entry.e)),
+           "194fe98d",
+           (fun (norec : 'no_rec) (eo : 'e__1 option) (id : string) _ _
+                (loc : Ploc.t) ->
+              (make_named id eo loc norec : 'symb)));
         Grammar.production
           (Grammar.r_next
              (Grammar.r_next Grammar.r_stop (Grammar.s_token ("", "_")))
@@ -448,9 +512,11 @@ Grammar.safe_extend
     Grammar.extension (lookahead : 'lookahead Grammar.Entry.e) None
       [None, None,
        [Grammar.production
-          (Grammar.r_next Grammar.r_stop (Grammar.s_token ("STRING", "")),
+          (Grammar.r_next Grammar.r_stop
+             (Grammar.s_nterm
+                (string_or_rawstring : 'string_or_rawstring Grammar.Entry.e)),
            "194fe98d",
-           (fun (s : string) (loc : Ploc.t) ->
+           (fun (s : 'string_or_rawstring) (loc : Ploc.t) ->
               (let s = Plexing.eval_string loc s in
                List.rev
                  (fold_string_chars (fun c pl -> MLast.PaChr (loc, c) :: pl) s
@@ -470,6 +536,11 @@ Grammar.safe_extend
           (Grammar.r_next Grammar.r_stop (Grammar.s_token ("", "_")),
            "194fe98d",
            (fun _ (loc : Ploc.t) -> (MLast.PaAny loc : 'lookahead_char)));
+        Grammar.production
+          (Grammar.r_next Grammar.r_stop (Grammar.s_token ("LIDENT", "")),
+           "194fe98d",
+           (fun (id : string) (loc : Ploc.t) ->
+              (MLast.PaLid (loc, id) : 'lookahead_char)));
         Grammar.production
           (Grammar.r_next
              (Grammar.r_next
@@ -508,10 +579,11 @@ Grammar.safe_extend
         Grammar.production
           (Grammar.r_next
              (Grammar.r_next Grammar.r_stop (Grammar.s_token ("", "?")))
-             (Grammar.s_token ("STRING", "")),
+             (Grammar.s_nterm
+                (string_or_rawstring : 'string_or_rawstring Grammar.Entry.e)),
            "194fe98d",
-           (fun (s : string) _ (loc : Ploc.t) ->
-              (SpoQues (MLast.ExStr (loc, s)) : 'err_kont)));
+           (fun (s : 'string_or_rawstring) _ (loc : Ploc.t) ->
+              (SpoQues (MLast.ExStr (loc, (loc, s))) : 'err_kont)));
         Grammar.production
           (Grammar.r_next Grammar.r_stop (Grammar.s_token ("", "!")),
            "194fe98d", (fun _ (loc : Ploc.t) -> (SpoBang : 'err_kont)))]];
