@@ -222,7 +222,7 @@ let substp mloc env =
     | MLast.ExInt (_, x, "") -> MLast.PaInt (loc, x, "")
     | MLast.ExChr (_, x) -> MLast.PaChr (loc, x)
     | MLast.ExStr (_, (_, x)) -> MLast.PaStr (loc, x)
-    | MLast.ExTup (_, x) -> MLast.PaTup (loc, List.map loop x)
+    | MLast.ExTup (_, x) -> MLast.PaTup (loc, List.map loop x, true)
     | MLast.ExRec (_, pel, None) ->
         let ppl = List.map (fun (p, e) -> p, loop e) pel in
         MLast.PaRec (loc, ppl)
@@ -239,10 +239,10 @@ let substt mloc env =
     function
       MLast.TyArr (_, t1, t2) -> MLast.TyArr (loc, loop t1, loop t2)
     | MLast.TyApp (_, t1, t2) -> MLast.TyApp (loc, loop t1, loop t2)
-    | MLast.TyTup (_, tl) -> MLast.TyTup (loc, List.map loop tl)
+    | MLast.TyTup (_, tl) -> MLast.TyTup (loc, List.map labloop tl)
     | MLast.TyLid (_, x) as t -> (try List.assoc x env with Not_found -> t)
     | t -> t
-  in
+  and labloop (lab, t) = lab, loop t in
   loop
 ;;
 
@@ -368,7 +368,7 @@ let define eo x =
                 (fun (param : 'patt) _ (loc : Ploc.t) ->
                    (let pl =
                       match param with
-                        MLast.PaTup (_, pl) -> pl
+                        MLast.PaTup (_, pl, true) -> pl
                       | p -> [p]
                     in
                     if List.length pl = List.length sl then
