@@ -316,7 +316,7 @@ value tests = "test pa_r+quotations-to-pr_r" >::: (List.map mktest
 |foo} ;
           code = {foo|fun [ <:sig_item< type $lid:x$ $list:_$ = $priv:_$ .. $_itemattrs:_$ >> -> 1 ] ;|foo}
         }
-      ; { name = "typedecl-1" ;
+      ; { name = "typedecl-2" ;
           expect = {foo|{MLast.tdIsDecl = Ploc.VaVal True; MLast.tdNam = Ploc.VaVal x;
  MLast.tdPrm = Ploc.VaVal pl; MLast.tdPrv = Ploc.VaVal False;
  MLast.tdDef = tk; MLast.tdCon = Ploc.VaVal [];
@@ -324,7 +324,7 @@ value tests = "test pa_r+quotations-to-pr_r" >::: (List.map mktest
 |foo} ;
           code = {foo|<:type_decl< $tp:x$ $list:pl$ = $tk$ >>;|foo}
         }
-      ; { name = "typedecl-2" ;
+      ; { name = "typedecl-3" ;
           expect = {foo|{MLast.tdIsDecl = Ploc.VaVal True; MLast.tdNam = Ploc.VaVal (loc, x);
  MLast.tdPrm = Ploc.VaVal pl; MLast.tdPrv = Ploc.VaVal False;
  MLast.tdDef = tk; MLast.tdCon = Ploc.VaVal [];
@@ -332,6 +332,14 @@ value tests = "test pa_r+quotations-to-pr_r" >::: (List.map mktest
 |foo} ;
           code = {foo|<:type_decl< $tp:(loc,x)$ $list:pl$ = $tk$ >>;|foo}
         }
+      ; { name = "typedecl-3" ;
+          expect = {foo|{MLast.tdIsDecl = Ploc.VaVal True; MLast.tdNam = Ploc.VaVal (loc, Ploc.VaVal "t");
+ MLast.tdPrm = Ploc.VaVal [(Ploc.VaVal v, Ploc.VaVal vastr)]; MLast.tdPrv = Ploc.VaVal False;
+ MLast.tdDef = tk; MLast.tdCon = Ploc.VaVal [];
+ MLast.tdAttributes = Ploc.VaVal []};
+|foo} ;
+          code = {foo|<:type_decl< t $variance:vastr$ $var:v$ = $tk$ >>;|foo}
+        }
       ; { name = "attribute-body-1" ;
           expect = {foo|fun
 [ (Ploc.VaVal (_, "add"), MLast.StAttr _ (Ploc.VaVal [si])) -> si ];
